@@ -6,13 +6,14 @@ defmodule HelloWorld.Mixfile do
 
   def project do
     [app: :'Elixir.HelloWorld',
-     version: "2.0.1",
+     version: "2.0.5",
      elixirc_paths: ["lib/"],
      deps: deps()]
   end
 
   def application do
-    [applications: []]
+    [applications: [
+       :cloudi_core]]
   end
 
   defp deps do
