@@ -5,7 +5,7 @@ package erlang
 //
 // MIT License
 //
-// Copyright (c) 2017-2018 Michael Truog <mjtruog at gmail dot com>
+// Copyright (c) 2017-2022 Michael Truog <mjtruog at protonmail dot com>
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software"),
@@ -37,38 +37,41 @@ import (
 	"strconv"
 )
 
+var undefined = "undefined" // Change with SetUndefined
+
 const (
 	// tag values here http://www.erlang.org/doc/apps/erts/erl_ext_dist.html
-	tagVersion          = 131
-	tagCompressedZlib   = 80
-	tagNewFloatExt      = 70
-	tagBitBinaryExt     = 77
-	tagAtomCacheRef     = 78
-	tagSmallIntegerExt  = 97
-	tagIntegerExt       = 98
-	tagFloatExt         = 99
-	tagAtomExt          = 100
-	tagReferenceExt     = 101
-	tagPortExt          = 102
-	tagPidExt           = 103
-	tagSmallTupleExt    = 104
-	tagLargeTupleExt    = 105
-	tagNilExt           = 106
-	tagStringExt        = 107
-	tagListExt          = 108
-	tagBinaryExt        = 109
-	tagSmallBigExt      = 110
-	tagLargeBigExt      = 111
-	tagNewFunExt        = 112
-	tagExportExt        = 113
-	tagNewReferenceExt  = 114
-	tagSmallAtomExt     = 115
-	tagMapExt           = 116
-	tagFunExt           = 117
-	tagAtomUtf8Ext      = 118
-	tagSmallAtomUtf8Ext = 119
-
-	bufferSize = 65536
+	tagVersion           = 131
+	tagCompressedZlib    = 80
+	tagNewFloatExt       = 70
+	tagBitBinaryExt      = 77
+	tagAtomCacheRef      = 78
+	tagNewPidExt         = 88
+	tagNewPortExt        = 89
+	tagNewerReferenceExt = 90
+	tagSmallIntegerExt   = 97
+	tagIntegerExt        = 98
+	tagFloatExt          = 99
+	tagAtomExt           = 100
+	tagReferenceExt      = 101
+	tagPortExt           = 102
+	tagPidExt            = 103
+	tagSmallTupleExt     = 104
+	tagLargeTupleExt     = 105
+	tagNilExt            = 106
+	tagStringExt         = 107
+	tagListExt           = 108
+	tagBinaryExt         = 109
+	tagSmallBigExt       = 110
+	tagLargeBigExt       = 111
+	tagNewFunExt         = 112
+	tagExportExt         = 113
+	tagNewReferenceExt   = 114
+	tagSmallAtomExt      = 115
+	tagMapExt            = 116
+	tagFunExt            = 117
+	tagAtomUtf8Ext       = 118
+	tagSmallAtomUtf8Ext  = 119
 )
 
 // Erlang term structs listed alphabetically
@@ -103,29 +106,30 @@ type OtpErlangList struct {
 // OtpErlangMap represents MAP_EXT
 type OtpErlangMap map[interface{}]interface{}
 
-// OtpErlangPid represents PID_EXT
+// OtpErlangPid represents NEW_PID_EXT or PID_EXT
 type OtpErlangPid struct {
 	NodeTag  uint8
 	Node     []byte
 	ID       []byte
 	Serial   []byte
-	Creation byte
+	Creation []byte
 }
 
-// OtpErlangPort represents PORT_EXT
+// OtpErlangPort represents NEW_PORT_EXT or PORT_EXT
 type OtpErlangPort struct {
 	NodeTag  uint8
 	Node     []byte
 	ID       []byte
-	Creation byte
+	Creation []byte
 }
 
-// OtpErlangReference represents REFERENCE_EXT or NEW_REFERENCE_EXT
+// OtpErlangReference represents
+// NEWER_REFERENCE_EXT, REFERENCE_EXT or NEW_REFERENCE_EXT
 type OtpErlangReference struct {
 	NodeTag  uint8
 	Node     []byte
 	ID       []byte
-	Creation byte
+	Creation []byte
 }
 
 // OtpErlangTuple represents SMALL_TUPLE_EXT or LARGE_TUPLE_EXT
@@ -171,7 +175,7 @@ func (e *ParseError) Error() string {
 
 // core functionality
 
-// BinaryToTerm decodes the Erlang Binary Term Format into Go types
+// BinaryToTerm decodes the Erlang External Term Format into Go types
 func BinaryToTerm(data []byte) (interface{}, error) {
 	size := len(data)
 	if size <= 1 {
@@ -197,13 +201,11 @@ func BinaryToTerm(data []byte) (interface{}, error) {
 	return term, nil
 }
 
-// TermToBinary encodes Go types into the Erlang Binary Term Format
+// TermToBinary encodes Go types into the Erlang External Term Format
 func TermToBinary(term interface{}, compressed int) ([]byte, error) {
 	if compressed < -1 || compressed > 9 {
 		return nil, inputErrorNew("compressed in [-1..9]")
 	}
-	var dataBuffer = new(bytes.Buffer)
-	dataBuffer.Grow(bufferSize)
 	dataUncompressed, err := termsToBinary(term, new(bytes.Buffer))
 	if err != nil {
 		return nil, err
@@ -243,6 +245,11 @@ func TermToBinary(term interface{}, compressed int) ([]byte, error) {
 	return result.Bytes(), nil
 }
 
+// SetUndefined assigns the undefined atom name, Elixir use can set to "nil"
+func SetUndefined(value string) {
+	undefined = value
+}
+
 // BinaryToTerm implementation functions
 
 func binaryToTerms(i int, reader *bytes.Reader) (int, interface{}, error) {
@@ -310,21 +317,8 @@ func binaryToTerms(i int, reader *bytes.Reader) (int, interface{}, error) {
 			return i, nil, err
 		}
 		return i + 31, value, nil
-	case tagAtomExt:
-		var j uint16
-		err = binary.Read(reader, binary.BigEndian, &j)
-		if err != nil {
-			return i, nil, err
-		}
-		i += 2
-		value := make([]byte, j)
-		if j > 0 {
-			_, err = reader.Read(value)
-			if err != nil {
-				return i, nil, err
-			}
-		}
-		return i + int(j), OtpErlangAtom(value), nil
+	case tagNewPortExt:
+		fallthrough
 	case tagReferenceExt:
 		fallthrough
 	case tagPortExt:
@@ -340,20 +334,30 @@ func binaryToTerms(i int, reader *bytes.Reader) (int, interface{}, error) {
 			return i, nil, err
 		}
 		i += 4
-		var creation byte
-		creation, err = reader.ReadByte()
-		if err != nil {
-			return i, nil, err
-		}
-		i += 1
+		var creation []byte
 		switch tag {
-		case tagReferenceExt:
-			return i, OtpErlangReference{NodeTag: nodeTag, Node: node, ID: id, Creation: creation}, nil
-		case tagPortExt:
-			return i, OtpErlangPort{NodeTag: nodeTag, Node: node, ID: id, Creation: creation}, nil
+		case tagNewPortExt:
+			creation = make([]byte, 4)
+			_, err = reader.Read(creation)
+			if err != nil {
+				return i, nil, err
+			}
+			i += 4
 		default:
-			return i, nil, parseErrorNew("invalid tag case")
+			creation = make([]byte, 1)
+			_, err = reader.Read(creation)
+			if err != nil {
+				return i, nil, err
+			}
+			i += 1
+			if tag == tagReferenceExt {
+				return i, OtpErlangReference{NodeTag: nodeTag, Node: node, ID: id, Creation: creation}, nil
+			}
 		}
+		// tag == tagNewPortExt || tag == tagPortExt
+		return i, OtpErlangPort{NodeTag: nodeTag, Node: node, ID: id, Creation: creation}, nil
+	case tagNewPidExt:
+		fallthrough
 	case tagPidExt:
 		var nodeTag uint8
 		var node []byte
@@ -373,12 +377,23 @@ func binaryToTerms(i int, reader *bytes.Reader) (int, interface{}, error) {
 			return i, nil, err
 		}
 		i += 4
-		var creation byte
-		creation, err = reader.ReadByte()
-		if err != nil {
-			return i, nil, err
+		var creation []byte
+		switch tag {
+		case tagNewPidExt:
+			creation = make([]byte, 4)
+			_, err = reader.Read(creation)
+			if err != nil {
+				return i, nil, err
+			}
+			i += 4
+		case tagPidExt:
+			creation = make([]byte, 1)
+			_, err = reader.Read(creation)
+			if err != nil {
+				return i, nil, err
+			}
+			i += 1
 		}
-		i += 1
 		return i, OtpErlangPid{NodeTag: nodeTag, Node: node, ID: id, Serial: serial, Creation: creation}, nil
 	case tagSmallTupleExt:
 		fallthrough
@@ -564,6 +579,8 @@ func binaryToTerms(i int, reader *bytes.Reader) (int, interface{}, error) {
 			return i, nil, err
 		}
 		return i, OtpErlangFunction{Tag: tag, Value: value}, nil
+	case tagNewerReferenceExt:
+		fallthrough
 	case tagNewReferenceExt:
 		var j uint16
 		err = binary.Read(reader, binary.BigEndian, &j)
@@ -578,12 +595,23 @@ func binaryToTerms(i int, reader *bytes.Reader) (int, interface{}, error) {
 		if err != nil {
 			return i, nil, err
 		}
-		var creation byte
-		creation, err = reader.ReadByte()
-		if err != nil {
-			return i, nil, err
+		var creation []byte
+		switch tag {
+		case tagNewerReferenceExt:
+			creation = make([]byte, 4)
+			_, err = reader.Read(creation)
+			if err != nil {
+				return i, nil, err
+			}
+			i += 4
+		case tagNewReferenceExt:
+			creation = make([]byte, 1)
+			_, err = reader.Read(creation)
+			if err != nil {
+				return i, nil, err
+			}
+			i += 1
 		}
-		i += 1
 		id := make([]byte, j)
 		if j > 0 {
 			_, err = reader.Read(id)
@@ -592,21 +620,6 @@ func binaryToTerms(i int, reader *bytes.Reader) (int, interface{}, error) {
 			}
 		}
 		return i + int(j), OtpErlangReference{NodeTag: nodeTag, Node: node, ID: id, Creation: creation}, nil
-	case tagSmallAtomExt:
-		var j uint8
-		j, err = reader.ReadByte()
-		if err != nil {
-			return i, nil, err
-		}
-		i += 1
-		value := make([]byte, j)
-		if j > 0 {
-			_, err = reader.Read(value)
-			if err != nil {
-				return i, nil, err
-			}
-		}
-		return i + int(j), OtpErlangAtom(value), nil
 	case tagMapExt:
 		var length uint32
 		err = binary.Read(reader, binary.BigEndian, &length)
@@ -669,6 +682,8 @@ func binaryToTerms(i int, reader *bytes.Reader) (int, interface{}, error) {
 		}
 		return i, OtpErlangFunction{Tag: tag, Value: value}, nil
 	case tagAtomUtf8Ext:
+		fallthrough
+	case tagAtomExt:
 		var j uint16
 		err = binary.Read(reader, binary.BigEndian, &j)
 		if err != nil {
@@ -682,8 +697,26 @@ func binaryToTerms(i int, reader *bytes.Reader) (int, interface{}, error) {
 				return i, nil, err
 			}
 		}
-		return i + int(j), OtpErlangAtomUTF8(value), nil
+		if string(value) == "true" {
+			return i + int(j), true, nil
+		}
+		if string(value) == "false" {
+			return i + int(j), false, nil
+		}
+		if string(value) == undefined {
+			return i + int(j), nil, nil
+		}
+		switch tag {
+		case tagAtomUtf8Ext:
+			return i + int(j), OtpErlangAtomUTF8(value), nil
+		case tagAtomExt:
+			return i + int(j), OtpErlangAtom(value), nil
+		default:
+			return i, nil, parseErrorNew("invalid tag case")
+		}
 	case tagSmallAtomUtf8Ext:
+		fallthrough
+	case tagSmallAtomExt:
 		var j uint8
 		j, err = reader.ReadByte()
 		if err != nil {
@@ -697,7 +730,23 @@ func binaryToTerms(i int, reader *bytes.Reader) (int, interface{}, error) {
 				return i, nil, err
 			}
 		}
-		return i + int(j), OtpErlangAtomUTF8(value), nil
+		if string(value) == "true" {
+			return i + int(j), true, nil
+		}
+		if string(value) == "false" {
+			return i + int(j), false, nil
+		}
+		if string(value) == undefined {
+			return i + int(j), nil, nil
+		}
+		switch tag {
+		case tagSmallAtomUtf8Ext:
+			return i + int(j), OtpErlangAtomUTF8(value), nil
+		case tagSmallAtomExt:
+			return i + int(j), OtpErlangAtom(value), nil
+		default:
+			return i, nil, parseErrorNew("invalid tag case")
+		}
 	case tagCompressedZlib:
 		var sizeUncompressed uint32
 		err = binary.Read(reader, binary.BigEndian, &sizeUncompressed)
@@ -788,6 +837,32 @@ func binaryToPid(i int, reader *bytes.Reader) (int, interface{}, error) {
 	}
 	i += 1
 	switch tag {
+	case tagNewPidExt:
+		var nodeTag uint8
+		var node []byte
+		i, nodeTag, node, err = binaryToAtom(i, reader)
+		if err != nil {
+			return i, nil, err
+		}
+		id := make([]byte, 4)
+		_, err = reader.Read(id)
+		if err != nil {
+			return i, nil, err
+		}
+		i += 4
+		serial := make([]byte, 4)
+		_, err = reader.Read(serial)
+		if err != nil {
+			return i, nil, err
+		}
+		i += 4
+		creation := make([]byte, 4)
+		_, err = reader.Read(creation)
+		if err != nil {
+			return i, nil, err
+		}
+		i += 4
+		return i, OtpErlangPid{NodeTag: nodeTag, Node: node, ID: id, Serial: serial, Creation: creation}, nil
 	case tagPidExt:
 		var nodeTag uint8
 		var node []byte
@@ -807,8 +882,8 @@ func binaryToPid(i int, reader *bytes.Reader) (int, interface{}, error) {
 			return i, nil, err
 		}
 		i += 4
-		var creation byte
-		creation, err = reader.ReadByte()
+		creation := make([]byte, 1)
+		_, err = reader.Read(creation)
 		if err != nil {
 			return i, nil, err
 		}
@@ -826,9 +901,9 @@ func binaryToAtom(i int, reader *bytes.Reader) (int, uint8, []byte, error) {
 	}
 	i += 1
 	switch tag {
-	case tagAtomExt:
-		fallthrough
 	case tagAtomUtf8Ext:
+		fallthrough
+	case tagAtomExt:
 		var j uint16
 		err = binary.Read(reader, binary.BigEndian, &j)
 		if err != nil {
@@ -845,9 +920,9 @@ func binaryToAtom(i int, reader *bytes.Reader) (int, uint8, []byte, error) {
 			return i, tag, nil, err
 		}
 		return i + int(j), tag, value, nil
-	case tagSmallAtomExt:
-		fallthrough
 	case tagSmallAtomUtf8Ext:
+		fallthrough
+	case tagSmallAtomExt:
 		var j uint8
 		j, err = reader.ReadByte()
 		if err != nil {
@@ -916,11 +991,11 @@ func termsToBinary(termI interface{}, buffer *bytes.Buffer) (*bytes.Buffer, erro
 		return floatToBinary(term, buffer)
 	case bool:
 		if term {
-			return atomToBinary("true", buffer)
+			return atomUtf8ToBinary("true", buffer)
 		}
-		return atomToBinary("false", buffer)
+		return atomUtf8ToBinary("false", buffer)
 	case nil:
-		return atomToBinary("undefined", buffer)
+		return atomUtf8ToBinary(undefined, buffer)
 	case OtpErlangAtom:
 		return atomToBinary(string(term), buffer)
 	case OtpErlangAtomUTF8:
@@ -1158,6 +1233,8 @@ func floatToBinary(term float64, buffer *bytes.Buffer) (*bytes.Buffer, error) {
 }
 
 func atomToBinary(term string, buffer *bytes.Buffer) (*bytes.Buffer, error) {
+	// deprecated
+	// (not used in Erlang/OTP 26, i.e., minor_version 2)
 	switch length := len(term); {
 	case length <= math.MaxUint8:
 		_, err := buffer.Write([]byte{tagSmallAtomExt, uint8(length)})
@@ -1253,9 +1330,20 @@ func functionToBinary(term OtpErlangFunction, buffer *bytes.Buffer) (*bytes.Buff
 }
 
 func pidToBinary(term OtpErlangPid, buffer *bytes.Buffer) (*bytes.Buffer, error) {
-	err := buffer.WriteByte(tagPidExt)
-	if err != nil {
-		return buffer, err
+	var err error
+	switch creationSize := len(term.Creation); {
+	case creationSize == 1:
+		err = buffer.WriteByte(tagPidExt)
+		if err != nil {
+			return buffer, err
+		}
+	case creationSize == 4:
+		err = buffer.WriteByte(tagNewPidExt)
+		if err != nil {
+			return buffer, err
+		}
+	default:
+		return buffer, outputErrorNew("unknown pid type")
 	}
 	err = buffer.WriteByte(term.NodeTag)
 	if err != nil {
@@ -1273,14 +1361,25 @@ func pidToBinary(term OtpErlangPid, buffer *bytes.Buffer) (*bytes.Buffer, error)
 	if err != nil {
 		return buffer, err
 	}
-	err = buffer.WriteByte(term.Creation)
+	_, err = buffer.Write(term.Creation)
 	return buffer, err
 }
 
 func portToBinary(term OtpErlangPort, buffer *bytes.Buffer) (*bytes.Buffer, error) {
-	err := buffer.WriteByte(tagPortExt)
-	if err != nil {
-		return buffer, err
+	var err error
+	switch creationSize := len(term.Creation); {
+	case creationSize == 1:
+		err = buffer.WriteByte(tagPortExt)
+		if err != nil {
+			return buffer, err
+		}
+	case creationSize == 4:
+		err = buffer.WriteByte(tagNewPortExt)
+		if err != nil {
+			return buffer, err
+		}
+	default:
+		return buffer, outputErrorNew("unknown port type")
 	}
 	err = buffer.WriteByte(term.NodeTag)
 	if err != nil {
@@ -1294,7 +1393,7 @@ func portToBinary(term OtpErlangPort, buffer *bytes.Buffer) (*bytes.Buffer, erro
 	if err != nil {
 		return buffer, err
 	}
-	err = buffer.WriteByte(term.Creation)
+	_, err = buffer.Write(term.Creation)
 	return buffer, err
 }
 
@@ -1317,12 +1416,23 @@ func referenceToBinary(term OtpErlangReference, buffer *bytes.Buffer) (*bytes.Bu
 		if err != nil {
 			return buffer, err
 		}
-		err = buffer.WriteByte(term.Creation)
+		_, err = buffer.Write(term.Creation)
 		return buffer, err
 	case length <= math.MaxUint16:
-		err := buffer.WriteByte(tagNewReferenceExt)
-		if err != nil {
-			return buffer, err
+		var err error
+		switch creationSize := len(term.Creation); {
+		case creationSize == 1:
+			err = buffer.WriteByte(tagNewReferenceExt)
+			if err != nil {
+				return buffer, err
+			}
+		case creationSize == 4:
+			err = buffer.WriteByte(tagNewerReferenceExt)
+			if err != nil {
+				return buffer, err
+			}
+		default:
+			return buffer, outputErrorNew("unknown reference type")
 		}
 		err = binary.Write(buffer, binary.BigEndian, uint16(length))
 		if err != nil {
@@ -1336,7 +1446,7 @@ func referenceToBinary(term OtpErlangReference, buffer *bytes.Buffer) (*bytes.Bu
 		if err != nil {
 			return buffer, err
 		}
-		err = buffer.WriteByte(term.Creation)
+		_, err = buffer.Write(term.Creation)
 		if err != nil {
 			return buffer, err
 		}
