@@ -5,7 +5,7 @@
 
   MIT License
 
-  Copyright (c) 2017-2020 Michael Truog <mjtruog at protonmail dot com>
+  Copyright (c) 2017-2022 Michael Truog <mjtruog at protonmail dot com>
 
   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
@@ -42,6 +42,7 @@ module Foreign.CloudI
     , messageDecodingError
     , terminateError
     , Exception
+    , FatalError
     , Result
     , api
     , threadCount
@@ -67,6 +68,7 @@ module Foreign.CloudI
     , timeoutAsync
     , timeoutSync
     , timeoutTerminate
+    , priorityDefault
     , poll
     , shutdown
     , threadCreate
@@ -77,7 +79,7 @@ module Foreign.CloudI
 
 import Prelude hiding (init,length,(<>))
 import Data.Bits (shiftL,(.|.))
-import Data.Maybe (fromMaybe)
+import Data.Maybe (fromJust,fromMaybe)
 import Data.Typeable (Typeable)
 import qualified Control.Exception as Exception
 import qualified Control.Concurrent as Concurrent
@@ -101,6 +103,8 @@ import qualified System.IO as SysIO
 import qualified System.IO.Error as SysIOErr
 import qualified System.IO.Unsafe as Unsafe
 import qualified System.Posix.Env as POSIX (getEnv)
+import qualified System.Posix.Process as POSIX (exitImmediately)
+import qualified System.Exit as Exit
 type Array = IArray.Array
 type Builder = Builder.Builder
 type ByteString = ByteString.ByteString
@@ -110,6 +114,10 @@ type LazyByteString = LazyByteString.ByteString
 type Map = Map.Map
 type RequestType = Instance.RequestType
 type SomeException = Exception.SomeException
+type SomeAsyncException = Exception.SomeAsyncException
+type AsyncException = Exception.AsyncException
+type AssertionFailed = Exception.AssertionFailed
+type ExitCode = Exit.ExitCode
 type Source = Instance.Source
 type ThreadId = Concurrent.ThreadId
 type Word32 = Word.Word32
@@ -165,6 +173,13 @@ data Exception s =
 
 instance Typeable s => Exception.Exception (Exception s)
 
+data FatalError = FatalError
+
+instance Exception.Exception FatalError
+
+instance Show FatalError where
+    showsPrec _ FatalError = showString "FatalError"
+
 printException :: String -> IO ()
 printException str =
     SysIO.hPutStrLn SysIO.stderr ("Exception: " ++ str)
@@ -219,7 +234,9 @@ api threadIndex state terminateReturnValueOpt = do
                             return $ Left err
                         Right (_, api1) ->
                             return $ Right api1
-        (_, _) ->
+        (_, _) -> do
+            SysIO.hPutStrLn SysIO.stderr
+                "CloudI service execution must occur in CloudI"
             return $ Left invalidInputError
 
 -- | returns the thread count from the service configuration
@@ -294,11 +311,11 @@ sendAsync :: Typeable s => Instance.T s -> ByteString -> ByteString ->
 sendAsync api0@Instance.T{
       Instance.terminateException = terminateException
     , Instance.timeoutAsync = timeoutAsync'
-    , Instance.priorityDefault = priorityDefault}
+    , Instance.priorityDefault = priorityDefault'}
     name request timeoutOpt requestInfoOpt priorityOpt =
     let timeout = fromMaybe timeoutAsync' timeoutOpt
         requestInfo = fromMaybe ByteString.empty requestInfoOpt
-        priority = fromMaybe priorityDefault priorityOpt
+        priority = fromMaybe priorityDefault' priorityOpt
         sendAsyncTerms = Erlang.OtpErlangTuple
             [ Erlang.OtpErlangAtom (Char8.pack "send_async")
             , Erlang.OtpErlangString name
@@ -329,11 +346,11 @@ sendSync :: Typeable s => Instance.T s -> ByteString -> ByteString ->
 sendSync api0@Instance.T{
       Instance.terminateException = terminateException
     , Instance.timeoutSync = timeoutSync'
-    , Instance.priorityDefault = priorityDefault}
+    , Instance.priorityDefault = priorityDefault'}
     name request timeoutOpt requestInfoOpt priorityOpt =
     let timeout = fromMaybe timeoutSync' timeoutOpt
         requestInfo = fromMaybe ByteString.empty requestInfoOpt
-        priority = fromMaybe priorityDefault priorityOpt
+        priority = fromMaybe priorityDefault' priorityOpt
         sendSyncTerms = Erlang.OtpErlangTuple
             [ Erlang.OtpErlangAtom (Char8.pack "send_sync")
             , Erlang.OtpErlangString name
@@ -368,11 +385,11 @@ mcastAsync :: Typeable s => Instance.T s -> ByteString -> ByteString ->
 mcastAsync api0@Instance.T{
       Instance.terminateException = terminateException
     , Instance.timeoutAsync = timeoutAsync'
-    , Instance.priorityDefault = priorityDefault}
+    , Instance.priorityDefault = priorityDefault'}
     name request timeoutOpt requestInfoOpt priorityOpt =
     let timeout = fromMaybe timeoutAsync' timeoutOpt
         requestInfo = fromMaybe ByteString.empty requestInfoOpt
-        priority = fromMaybe priorityDefault priorityOpt
+        priority = fromMaybe priorityDefault' priorityOpt
         mcastAsyncTerms = Erlang.OtpErlangTuple
             [ Erlang.OtpErlangAtom (Char8.pack "mcast_async")
             , Erlang.OtpErlangString name
@@ -407,7 +424,8 @@ forward_ api0 Instance.SYNC = forwardSync api0
 forwardAsyncI :: Instance.T s -> ByteString ->
     ByteString -> ByteString -> Int -> Int -> ByteString -> Source ->
     IO (Result (Instance.T s))
-forwardAsyncI api0 name responseInfo response timeout priority transId pid = do
+forwardAsyncI api0 name responseInfo response
+    timeout priority transId source = do
     let forwardTerms = Erlang.OtpErlangTuple
             [ Erlang.OtpErlangAtom (Char8.pack "forward_async")
             , Erlang.OtpErlangString name
@@ -416,7 +434,7 @@ forwardAsyncI api0 name responseInfo response timeout priority transId pid = do
             , Erlang.OtpErlangInteger timeout
             , Erlang.OtpErlangInteger priority
             , Erlang.OtpErlangBinary transId
-            , Erlang.OtpErlangPid pid]
+            , Erlang.OtpErlangPid source]
     case Erlang.termToBinary forwardTerms (-1) of
         Left err ->
             return $ Left $ show err
@@ -428,9 +446,10 @@ forwardAsyncI api0 name responseInfo response timeout priority transId pid = do
 forwardAsync :: Typeable s => Instance.T s -> ByteString ->
     ByteString -> ByteString -> Int -> Int -> ByteString -> Source ->
     IO ()
-forwardAsync api0 name responseInfo response timeout priority transId pid = do
+forwardAsync api0 name responseInfo response
+    timeout priority transId source = do
     result <- forwardAsyncI api0
-        name responseInfo response timeout priority transId pid
+        name responseInfo response timeout priority transId source
     case result of
         Left err ->
             error err
@@ -440,7 +459,8 @@ forwardAsync api0 name responseInfo response timeout priority transId pid = do
 forwardSyncI :: Instance.T s -> ByteString ->
     ByteString -> ByteString -> Int -> Int -> ByteString -> Source ->
     IO (Result (Instance.T s))
-forwardSyncI api0 name responseInfo response timeout priority transId pid = do
+forwardSyncI api0 name responseInfo response
+    timeout priority transId source = do
     let forwardTerms = Erlang.OtpErlangTuple
             [ Erlang.OtpErlangAtom (Char8.pack "forward_sync")
             , Erlang.OtpErlangString name
@@ -449,7 +469,7 @@ forwardSyncI api0 name responseInfo response timeout priority transId pid = do
             , Erlang.OtpErlangInteger timeout
             , Erlang.OtpErlangInteger priority
             , Erlang.OtpErlangBinary transId
-            , Erlang.OtpErlangPid pid]
+            , Erlang.OtpErlangPid source]
     case Erlang.termToBinary forwardTerms (-1) of
         Left err ->
             return $ Left $ show err
@@ -461,9 +481,10 @@ forwardSyncI api0 name responseInfo response timeout priority transId pid = do
 forwardSync :: Typeable s => Instance.T s -> ByteString ->
     ByteString -> ByteString -> Int -> Int -> ByteString -> Source ->
     IO ()
-forwardSync api0 name responseInfo response timeout priority transId pid = do
+forwardSync api0 name responseInfo response
+    timeout priority transId source = do
     result <- forwardSyncI api0
-        name responseInfo response timeout priority transId pid
+        name responseInfo response timeout priority transId source
     case result of
         Left err ->
             error err
@@ -481,7 +502,8 @@ return_ api0 Instance.SYNC = returnSync api0
 returnAsyncI :: Instance.T s -> ByteString -> ByteString ->
     ByteString -> ByteString -> Int -> ByteString -> Source ->
     IO (Result (Instance.T s))
-returnAsyncI api0 name pattern responseInfo response timeout transId pid = do
+returnAsyncI api0 name pattern responseInfo response
+    timeout transId source = do
     let returnTerms = Erlang.OtpErlangTuple
             [ Erlang.OtpErlangAtom (Char8.pack "return_async")
             , Erlang.OtpErlangString name
@@ -490,7 +512,7 @@ returnAsyncI api0 name pattern responseInfo response timeout transId pid = do
             , Erlang.OtpErlangBinary response
             , Erlang.OtpErlangInteger timeout
             , Erlang.OtpErlangBinary transId
-            , Erlang.OtpErlangPid pid]
+            , Erlang.OtpErlangPid source]
     case Erlang.termToBinary returnTerms (-1) of
         Left err ->
             return $ Left $ show err
@@ -502,9 +524,10 @@ returnAsyncI api0 name pattern responseInfo response timeout transId pid = do
 returnAsync :: Typeable s => Instance.T s -> ByteString -> ByteString ->
     ByteString -> ByteString -> Int -> ByteString -> Source ->
     IO ()
-returnAsync api0 name pattern responseInfo response timeout transId pid = do
+returnAsync api0 name pattern responseInfo response
+    timeout transId source = do
     result <- returnAsyncI api0
-        name pattern responseInfo response timeout transId pid
+        name pattern responseInfo response timeout transId source
     case result of
         Left err ->
             error err
@@ -514,7 +537,8 @@ returnAsync api0 name pattern responseInfo response timeout transId pid = do
 returnSyncI :: Instance.T s -> ByteString -> ByteString ->
     ByteString -> ByteString -> Int -> ByteString -> Source ->
     IO (Result (Instance.T s))
-returnSyncI api0 name pattern responseInfo response timeout transId pid = do
+returnSyncI api0 name pattern responseInfo response
+    timeout transId source = do
     let returnTerms = Erlang.OtpErlangTuple
             [ Erlang.OtpErlangAtom (Char8.pack "return_sync")
             , Erlang.OtpErlangString name
@@ -523,7 +547,7 @@ returnSyncI api0 name pattern responseInfo response timeout transId pid = do
             , Erlang.OtpErlangBinary response
             , Erlang.OtpErlangInteger timeout
             , Erlang.OtpErlangBinary transId
-            , Erlang.OtpErlangPid pid]
+            , Erlang.OtpErlangPid source]
     case Erlang.termToBinary returnTerms (-1) of
         Left err ->
             return $ Left $ show err
@@ -535,9 +559,10 @@ returnSyncI api0 name pattern responseInfo response timeout transId pid = do
 returnSync :: Typeable s => Instance.T s -> ByteString -> ByteString ->
     ByteString -> ByteString -> Int -> ByteString -> Source ->
     IO ()
-returnSync api0 name pattern responseInfo response timeout transId pid = do
+returnSync api0 name pattern responseInfo response
+    timeout transId source = do
     result <- returnSyncI api0
-        name pattern responseInfo response timeout transId pid
+        name pattern responseInfo response timeout transId source
     case result of
         Left err ->
             error err
@@ -630,12 +655,42 @@ timeoutTerminate :: Instance.T s -> Int
 timeoutTerminate Instance.T{Instance.timeoutTerminate = timeoutTerminate'} =
     timeoutTerminate'
 
+-- | returns the default service request send priority
+-- from the service configuration
+priorityDefault :: Instance.T s -> Int
+priorityDefault Instance.T{Instance.priorityDefault = priorityDefault'} =
+    priorityDefault'
+
 nullResponse :: RequestType -> ByteString -> ByteString ->
     ByteString -> ByteString -> Int -> Int -> ByteString -> Source ->
     s -> Instance.T s -> IO (Instance.Response s)
 nullResponse _ _ _ _ _ _ _ _ _ state api0 =
     return $ Instance.Null (state, api0)
 
+callbackExceptionFatal :: SomeException -> IO ()
+callbackExceptionFatal e = do
+    printException $ show e
+    POSIX.exitImmediately (Exit.ExitFailure 1)
+
+callbackException :: Instance.T s -> SomeException -> IO ()
+callbackException Instance.T{
+      Instance.fatalExceptions = fatalExceptions} e
+    | Just _ <- (Exception.fromException e :: Maybe AsyncException) =
+        callbackExceptionFatal e
+    | Just _ <- (Exception.fromException e :: Maybe SomeAsyncException) =
+        callbackExceptionFatal e
+    | Just _ <- (Exception.fromException e :: Maybe AssertionFailed) =
+        callbackExceptionFatal e
+    | Just _ <- (Exception.fromException e :: Maybe FatalError) =
+        callbackExceptionFatal e
+    | Just exitCode <- (Exception.fromException e :: Maybe ExitCode) =
+        POSIX.exitImmediately exitCode
+    | otherwise =
+        if fatalExceptions then
+            callbackExceptionFatal e
+        else
+            printException $ show e
+
 callback :: Typeable s => Instance.T s ->
     (RequestType, ByteString, ByteString, ByteString, ByteString,
      Int, Int, ByteString, Source) -> IO (Result (Instance.T s))
@@ -643,14 +698,14 @@ callback api0@Instance.T{
       Instance.state = state
     , Instance.callbacks = callbacks}
     (requestType, name, pattern, requestInfo, request,
-     timeout, priority, transId, pid) = do
+     timeout, priority, transId, source) = do
     let (callbackF, callbacksNew) = case Map.lookup pattern callbacks of
             Nothing ->
                 (nullResponse, callbacks)
             Just functionQueue ->
                 let f = Sequence.index functionQueue 0
                     functionQueueNew = (Sequence.|>)
-                        (Sequence.drop 0 functionQueue) f
+                        (Sequence.drop 1 functionQueue) f
                 in
                 (f, Map.insert pattern functionQueueNew callbacks)
         api1 = api0{Instance.callbacks = callbacksNew}
@@ -659,7 +714,7 @@ callback api0@Instance.T{
         Instance.ASYNC -> do
             callbackResultAsyncValue <- Exception.try $
                 callbackF requestType name pattern
-                requestInfo request timeout priority transId pid
+                requestInfo request timeout priority transId source
                 state api1
             case callbackResultAsyncValue of
                 Left (Terminate api2) ->
@@ -690,7 +745,7 @@ callback api0@Instance.T{
         Instance.SYNC -> do
             callbackResultSyncValue <- Exception.try $
                 callbackF requestType name pattern
-                requestInfo request timeout priority transId pid
+                requestInfo request timeout priority transId source
                 state api1
             case callbackResultSyncValue of
                 Left (Terminate api2) ->
@@ -720,7 +775,7 @@ callback api0@Instance.T{
                     return $ ReturnI (empty, empty, v0, v1)
     callbackResultType <- case callbackResultValue of
         Left exception -> do
-            printException $ show (exception :: Exception.SomeException)
+            callbackException api1 exception
             return $ ReturnI (empty, empty, state, api1)
         Right callbackResult ->
             return $ callbackResult
@@ -731,24 +786,26 @@ callback api0@Instance.T{
                     return $ Right api3
                 ReturnI (responseInfo, response, state', api3) ->
                     returnAsyncI api3{Instance.state = state'}
-                        name pattern responseInfo response timeout transId pid
+                        name pattern responseInfo response
+                        timeout transId source
                 ForwardI (name', requestInfo', request', timeout', priority',
                           state', api3) ->
                     forwardAsyncI api3{Instance.state = state'}
                         name' requestInfo' request'
-                        timeout' priority' transId pid
+                        timeout' priority' transId source
         Instance.SYNC ->
             case callbackResultType of
                 Finished api3 ->
                     return $ Right api3
                 ReturnI (responseInfo, response, state', api3) ->
                     returnSyncI api3{Instance.state = state'}
-                        name pattern responseInfo response timeout transId pid
+                        name pattern responseInfo response
+                        timeout transId source
                 ForwardI (name', requestInfo', request', timeout', priority',
                           state', api3) ->
                     forwardSyncI api3{Instance.state = state'}
                         name' requestInfo' request'
-                        timeout' priority' transId pid
+                        timeout' priority' transId source
 
 handleEvents :: [Message] -> Instance.T s -> Bool -> Word32 ->
     Get ([Message], Instance.T s)
@@ -764,12 +821,14 @@ handleEvents messages api0 external cmd0 = do
             processCount' <- Get.getWord32host
             timeoutAsync' <- Get.getWord32host
             timeoutSync' <- Get.getWord32host
-            priorityDefault <- Get.getInt8
+            priorityDefault' <- Get.getInt8
+            fatalExceptions' <- Get.getInt8
             let api1 = Instance.reinit api0
                     processCount'
                     timeoutAsync'
                     timeoutSync'
-                    priorityDefault
+                    priorityDefault'
+                    fatalExceptions'
             empty <- Get.isEmpty
             if not empty then
                 handleEvents messages api1 external 0
@@ -802,7 +861,8 @@ pollRequestDataGet messages api0 external = do
             timeoutAsync' <- Get.getWord32host
             timeoutSync' <- Get.getWord32host
             timeoutTerminate' <- Get.getWord32host
-            priorityDefault <- Get.getInt8
+            priorityDefault' <- Get.getInt8
+            fatalExceptions' <- Get.getInt8
             let api1 = Instance.init api0
                     processIndex'
                     processCount'
@@ -813,7 +873,8 @@ pollRequestDataGet messages api0 external = do
                     timeoutAsync'
                     timeoutSync'
                     timeoutTerminate'
-                    priorityDefault
+                    priorityDefault'
+                    fatalExceptions'
             empty <- Get.isEmpty
             if not empty then
                 handleEvents messages api1 external 0
@@ -835,13 +896,13 @@ pollRequestDataGet messages api0 external = do
             timeout <- Get.getWord32host
             priority <- Get.getInt8
             transId <- Get.getByteString 16
-            pidSize <- Get.getWord32host
-            pidData <- Get.getLazyByteString $ fromIntegral pidSize
+            sourceSize <- Get.getWord32host
+            sourceData <- Get.getLazyByteString $ fromIntegral sourceSize
             empty <- Get.isEmpty
-            case Erlang.binaryToTerm pidData of
+            case Erlang.binaryToTerm sourceData of
                 Left err ->
                     fail $ show err
-                Right (Erlang.OtpErlangPid (pid)) ->
+                Right (Erlang.OtpErlangPid (source)) ->
                     let requestType =
                             if cmd == messageSendAsync then
                                 Instance.ASYNC
@@ -850,7 +911,7 @@ pollRequestDataGet messages api0 external = do
                         messagesNew = (MessageSend (
                             requestType, name, pattern, requestInfo, request,
                             fromIntegral timeout, fromIntegral priority,
-                            transId, pid)):messages in
+                            transId, source)):messages in
                     if not empty then
                         handleEvents messagesNew api0 external 0
                     else
@@ -906,12 +967,14 @@ pollRequestDataGet messages api0 external = do
             processCount' <- Get.getWord32host
             timeoutAsync' <- Get.getWord32host
             timeoutSync' <- Get.getWord32host
-            priorityDefault <- Get.getInt8
+            priorityDefault' <- Get.getInt8
+            fatalExceptions' <- Get.getInt8
             let api1 = Instance.reinit api0
                     processCount'
                     timeoutAsync'
                     timeoutSync'
-                    priorityDefault
+                    priorityDefault'
+                    fatalExceptions'
             empty <- Get.isEmpty
             if not empty then
                 pollRequestDataGet messages api1 external
@@ -1112,17 +1175,17 @@ recv api0@Instance.T{
     (bufferRecvHeader, bufferRecvHeaderSize) <- recvBuffer
         bufferRecv bufferRecvSize 4 socketHandle bufferSize
     let header0 = Builder.toLazyByteString bufferRecvHeader
-        Just (byte0, header1) = LazyByteString.uncons header0
-        Just (byte1, header2) = LazyByteString.uncons header1
-        Just (byte2, header3) = LazyByteString.uncons header2
-        Just (byte3, headerRemaining) = LazyByteString.uncons header3
+        (byte0, header1) = fromJust $ LazyByteString.uncons header0
+        (byte1, header2) = fromJust $ LazyByteString.uncons header1
+        (byte2, header3) = fromJust $ LazyByteString.uncons header2
+        (byte3, remaining) = fromJust $ LazyByteString.uncons header3
         total = fromIntegral $
             (fromIntegral byte0 :: Word32) `shiftL` 24 .|.
             (fromIntegral byte1 :: Word32) `shiftL` 16 .|.
             (fromIntegral byte2 :: Word32) `shiftL` 8 .|.
             (fromIntegral byte3 :: Word32) :: Int
     (bufferRecvAll, bufferRecvAllSize) <- recvBuffer
-        (Builder.lazyByteString headerRemaining)
+        (Builder.lazyByteString remaining)
         (bufferRecvHeaderSize - 4) total socketHandle bufferSize
     let bufferRecvAllStr = Builder.toLazyByteString bufferRecvAll
         (bufferRecvData, bufferRecvNew) =
