@@ -3,7 +3,7 @@
 //
 // MIT License
 //
-// Copyright (c) 2014-2020 Michael Truog <mjtruog at protonmail dot com>
+// Copyright (c) 2014-2022 Michael Truog <mjtruog at protonmail dot com>
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software"),
@@ -91,7 +91,7 @@ if (Erlang.nodejs_version_after('10.0.0',true)) {
             //
             // Until the exception handling of the domain module is
             // provided by a different module (like async_hooks) or
-            // exceptions (throw/try/catch) are removed from Javascript,
+            // exceptions (throw/try/catch) are removed from JavaScript,
             // the use of the domain module needs to remain
             // (to remain consistent with other CloudI API implementations).
             // The stderr runtime deprecation information is ignored
@@ -249,6 +249,7 @@ CloudI.API = function API (thread_index, callback) {
                                   readable: true,
                                   writable: true});
     API._initialization_complete = false;
+    API._fatal_exceptions = false;
     API._terminate = false;
     API._terminate_callback = undefined;
     API._size = parseInt(buffer_size_str);
@@ -423,16 +424,17 @@ CloudI.API.prototype.mcast_async = function (name, request, callback,
 
 CloudI.API.prototype.forward_ = function (request_type, name,
                                           request_info, request,
-                                          timeout, priority, trans_id, pid) {
+                                          timeout, priority,
+                                          trans_id, source) {
     var API = this;
     switch (request_type) {
         case API.ASYNC:
             API.forward_async(name, request_info, request,
-                              timeout, priority, trans_id, pid);
+                              timeout, priority, trans_id, source);
             return;
         case API.SYNC:
             API.forward_sync(name, request_info, request,
-                             timeout, priority, trans_id, pid);
+                             timeout, priority, trans_id, source);
             return;
         default:
             throw new InvalidInputException();
@@ -441,42 +443,42 @@ CloudI.API.prototype.forward_ = function (request_type, name,
 
 CloudI.API.prototype.forward_async = function (name, request_info, request,
                                                timeout, priority,
-                                               trans_id, pid) {
+                                               trans_id, source) {
     this._poll_wait(function (API) {
         API._send([new Erlang.OtpErlangAtom('forward_async'), name,
                    new Erlang.OtpErlangBinary(request_info),
                    new Erlang.OtpErlangBinary(request),
                    timeout, priority,
-                   new Erlang.OtpErlangBinary(trans_id), pid]);
+                   new Erlang.OtpErlangBinary(trans_id), source]);
         throw new ForwardAsyncException();
     });
 };
 
 CloudI.API.prototype.forward_sync = function (name, request_info, request,
                                               timeout, priority,
-                                              trans_id, pid) {
+                                              trans_id, source) {
     this._poll_wait(function (API) {
         API._send([new Erlang.OtpErlangAtom('forward_sync'), name,
                    new Erlang.OtpErlangBinary(request_info),
                    new Erlang.OtpErlangBinary(request),
                    timeout, priority,
-                   new Erlang.OtpErlangBinary(trans_id), pid]);
+                   new Erlang.OtpErlangBinary(trans_id), source]);
         throw new ForwardSyncException();
     });
 };
 
 CloudI.API.prototype.return_ = function (request_type, name, pattern,
                                          response_info, response,
-                                         timeout, trans_id, pid) {
+                                         timeout, trans_id, source) {
     var API = this;
     switch (request_type) {
         case API.ASYNC:
             API.return_async(name, pattern, response_info, response,
-                             timeout, trans_id, pid);
+                             timeout, trans_id, source);
             return;
         case API.SYNC:
             API.return_sync(name, pattern, response_info, response,
-                            timeout, trans_id, pid);
+                            timeout, trans_id, source);
             return;
         default:
             throw new InvalidInputException();
@@ -485,24 +487,24 @@ CloudI.API.prototype.return_ = function (request_type, name, pattern,
 
 CloudI.API.prototype.return_async = function (name, pattern,
                                               response_info, response,
-                                              timeout, trans_id, pid) {
+                                              timeout, trans_id, source) {
     this._poll_wait(function (API) {
         API._send([new Erlang.OtpErlangAtom('return_async'), name, pattern,
                    new Erlang.OtpErlangBinary(response_info),
                    new Erlang.OtpErlangBinary(response),
-                   timeout, new Erlang.OtpErlangBinary(trans_id), pid]);
+                   timeout, new Erlang.OtpErlangBinary(trans_id), source]);
         throw new ReturnAsyncException();
     });
 };
 
 CloudI.API.prototype.return_sync = function (name, pattern,
                                              response_info, response,
-                                             timeout, trans_id, pid) {
+                                             timeout, trans_id, source) {
     this._poll_wait(function (API) {
         API._send([new Erlang.OtpErlangAtom('return_sync'), name, pattern,
                    new Erlang.OtpErlangBinary(response_info),
                    new Erlang.OtpErlangBinary(response),
-                   timeout, new Erlang.OtpErlangBinary(trans_id), pid]);
+                   timeout, new Erlang.OtpErlangBinary(trans_id), source]);
         throw new ReturnSyncException();
     });
 };
@@ -561,16 +563,21 @@ CloudI.API.prototype.timeout_terminate = function () {
     return this._timeout_terminate;
 };
 
+CloudI.API.prototype.priority_default = function () {
+    return this._priority_default;
+};
+
 CloudI.API.prototype._null_response = function (request_type, name, pattern,
                                                 request_info, request,
                                                 timeout, priority,
-                                                trans_id, pid) {
+                                                trans_id, source) {
     return '';
 };
 
 CloudI.API.prototype._callback = function (command, name, pattern,
                                            request_info, request,
-                                           timeout, priority, trans_id, pid) {
+                                           timeout, priority,
+                                           trans_id, source) {
     var API = this;
     var function_queue = API._callbacks[pattern];
     var f;
@@ -615,8 +622,11 @@ CloudI.API.prototype._callback = function (command, name, pattern,
                     process.exit(1);
                 }
                 else {
-                    return_null_response = true;
                     API._exception(err);
+                    if (API._fatal_exceptions) {
+                        process.exit(1);
+                    }
+                    return_null_response = true;
                 }
                 if (return_null_response) {
                     var response_info = '';
@@ -624,7 +634,7 @@ CloudI.API.prototype._callback = function (command, name, pattern,
                     try {
                         API.return_async(name, pattern,
                                          response_info, response,
-                                         timeout, trans_id, pid);
+                                         timeout, trans_id, source);
                     }
                     catch (err_new) {
                         err_new = undefined;
@@ -639,7 +649,8 @@ CloudI.API.prototype._callback = function (command, name, pattern,
                 var response = f.obj_f.call(f.obj,
                                             API.ASYNC, name, pattern,
                                             request_info, request,
-                                            timeout, priority, trans_id, pid);
+                                            timeout, priority,
+                                            trans_id, source);
                 if (typeof response === 'undefined') {
                     return;
                 }
@@ -662,7 +673,7 @@ CloudI.API.prototype._callback = function (command, name, pattern,
                 try {
                     API.return_async(name, pattern,
                                      response_info, response,
-                                     timeout, trans_id, pid);
+                                     timeout, trans_id, source);
                 }
                 catch (err_new) {
                     err_new = undefined;
@@ -702,8 +713,11 @@ CloudI.API.prototype._callback = function (command, name, pattern,
                     process.exit(1);
                 }
                 else {
-                    return_null_response = true;
                     API._exception(err);
+                    if (API._fatal_exceptions) {
+                        process.exit(1);
+                    }
+                    return_null_response = true;
                 }
                 if (return_null_response) {
                     var response_info = '';
@@ -711,7 +725,7 @@ CloudI.API.prototype._callback = function (command, name, pattern,
                     try {
                         API.return_sync(name, pattern,
                                         response_info, response,
-                                        timeout, trans_id, pid);
+                                        timeout, trans_id, source);
                     }
                     catch (err_new) {
                         err_new = undefined;
@@ -726,7 +740,8 @@ CloudI.API.prototype._callback = function (command, name, pattern,
                 var response = f.obj_f.call(f.obj,
                                             API.SYNC, name, pattern,
                                             request_info, request,
-                                            timeout, priority, trans_id, pid);
+                                            timeout, priority,
+                                            trans_id, source);
                 if (typeof response === 'undefined') {
                     return;
                 }
@@ -749,7 +764,7 @@ CloudI.API.prototype._callback = function (command, name, pattern,
                 try {
                     API.return_sync(name, pattern,
                                     response_info, response,
-                                    timeout, trans_id, pid);
+                                    timeout, trans_id, source);
                 }
                 catch (err_new) {
                     err_new = undefined;
@@ -784,6 +799,8 @@ CloudI.API.prototype._handle_events = function (data, data_size, i, command) {
                 i += 4;
                 API._priority_default = unpackInt8(i, data);
                 i += 1;
+                API._fatal_exceptions = unpackUint8(i, data) != 0;
+                i += 1;
                 break;
             case MESSAGE_KEEPALIVE:
                 API._send(new Erlang.OtpErlangAtom('keepalive'));
@@ -837,6 +854,8 @@ CloudI.API.prototype._poll_request = function (data) {
                 i += 4;
                 API._priority_default = unpackInt8(i, data);
                 i += 1;
+                API._fatal_exceptions = unpackUint8(i, data) != 0;
+                i += 1;
                 if (i != data_size) {
                     API._handle_events(data, data_size, i);
                 }
@@ -872,15 +891,15 @@ CloudI.API.prototype._poll_request = function (data) {
                 j = i + 16;
                 var trans_id = data.slice(i, j);
                 i = j;
-                var pid_size = unpackUint32(i, data);
+                var source_size = unpackUint32(i, data);
                 i += 4;
-                j = i + pid_size;
-                var pid = data.slice(i, j);
+                j = i + source_size;
+                var source = data.slice(i, j);
                 i = j;
                 if (i != data_size) {
                     API._handle_events(data, data_size, i);
                 }
-                Erlang.binary_to_term(pid, function (err, pid_object) {
+                Erlang.binary_to_term(source, function (err, source_object) {
                     if (err) {
                         API._exception(err);
                         API._poll_terminate();
@@ -889,7 +908,7 @@ CloudI.API.prototype._poll_request = function (data) {
                         API._callback(command, name, pattern,
                                       request_info, request,
                                       request_timeout, priority,
-                                      trans_id, pid_object);
+                                      trans_id, source_object);
                     }
                 });
                 return;
@@ -963,6 +982,8 @@ CloudI.API.prototype._poll_request = function (data) {
                 i += 4;
                 API._priority_default = unpackInt8(i, data);
                 i += 1;
+                API._fatal_exceptions = unpackUint8(i, data) != 0;
+                i += 1;
                 if (i == data_size) {
                     return;
                 }
