@@ -3,7 +3,7 @@
 //
 // MIT License
 //
-// Copyright (c) 2014-2018 Michael Truog <mjtruog at protonmail dot com>
+// Copyright (c) 2014-2022 Michael Truog <mjtruog at protonmail dot com>
 // Copyright (c) 2009-2013 Dmitry Vasiliev <dima@hlabs.org>
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
@@ -25,7 +25,7 @@
 // DEALINGS IN THE SOFTWARE.
 //
 
-var Erlang = require('./../Erlang.js').Erlang; 
+var Erlang = require('./../Erlang.js').Erlang;
 var assert = require('assert');
 
 // many of the test cases were adapted
@@ -33,6 +33,13 @@ var assert = require('assert');
 // to make the tests more exhaustive
 
 var toNativeString = {}.toString;
+var bufferFrom;
+if (Erlang.nodejs_version_after('5.10.0',true)) {
+    bufferFrom = Buffer.from;
+}
+else {
+    bufferFrom = Buffer;
+}
 String.prototype.repeat = function(i) {
     return new Array(i + 1).join(this);
 };
@@ -66,7 +73,7 @@ var hex = function hex(buffer) {
         var atom1 = new Erlang.OtpErlangAtom('test');
         assert.equal(toNativeString.call(atom1), '[object Object]');
         assert.deepEqual(new Erlang.OtpErlangAtom('test'), atom1);
-        assert.equal('OtpErlangAtom(test,false)', '' + atom1);
+        assert.equal('OtpErlangAtom(test,true)', '' + atom1);
         assert.ok(atom1 instanceof Erlang.OtpErlangAtom);
         var atom2 = new Erlang.OtpErlangAtom('test2');
         var atom1_new = new Erlang.OtpErlangAtom('test');
@@ -149,35 +156,51 @@ var hex = function hex(buffer) {
             assert.ok(err instanceof Erlang.ParseException);
             assert.strictEqual(term, undefined);
         });
+        Erlang.binary_to_term('\x83v\0\0', function(err, term) {
+            assert.strictEqual(err, undefined);
+            assert.deepEqual(term, new Erlang.OtpErlangAtom(''));
+        });
         Erlang.binary_to_term('\x83d\0\0', function(err, term) {
+            assert.strictEqual(err, undefined);
+            assert.deepEqual(term, new Erlang.OtpErlangAtom('', false));
+        });
+        Erlang.binary_to_term('\x83w\0', function(err, term) {
             assert.strictEqual(err, undefined);
             assert.deepEqual(term, new Erlang.OtpErlangAtom(''));
         });
         Erlang.binary_to_term('\x83s\0', function(err, term) {
             assert.strictEqual(err, undefined);
-            assert.deepEqual(term, new Erlang.OtpErlangAtom(''));
+            assert.deepEqual(term, new Erlang.OtpErlangAtom('', false));
+        });
+        Erlang.binary_to_term('\x83v\0\4test', function(err, term) {
+            assert.strictEqual(err, undefined);
+            assert.deepEqual(term, new Erlang.OtpErlangAtom('test'));
         });
         Erlang.binary_to_term('\x83d\0\4test', function(err, term) {
+            assert.strictEqual(err, undefined);
+            assert.deepEqual(term, new Erlang.OtpErlangAtom('test', false));
+        });
+        Erlang.binary_to_term('\x83w\4test', function(err, term) {
             assert.strictEqual(err, undefined);
             assert.deepEqual(term, new Erlang.OtpErlangAtom('test'));
         });
         Erlang.binary_to_term('\x83s\4test', function(err, term) {
             assert.strictEqual(err, undefined);
-            assert.deepEqual(term, new Erlang.OtpErlangAtom('test'));
+            assert.deepEqual(term, new Erlang.OtpErlangAtom('test', false));
         });
     }).call(this);
     (function test_binary_to_term_predefined_atom () {
-        Erlang.binary_to_term('\x83s\4true', function(err, term) {
+        Erlang.binary_to_term('\x83w\4true', function(err, term) {
             assert.strictEqual(err, undefined);
             assert.strictEqual(term, true);
         });
-        Erlang.binary_to_term('\x83s\5false', function(err, term) {
+        Erlang.binary_to_term('\x83w\5false', function(err, term) {
             assert.strictEqual(err, undefined);
             assert.strictEqual(term, false);
         });
-        Erlang.binary_to_term('\x83d\0\11undefined', function(err, term) {
+        Erlang.binary_to_term('\x83v\0\11undefined', function(err, term) {
             assert.strictEqual(err, undefined);
-            assert.deepEqual(term, new Erlang.OtpErlangAtom('undefined'));
+            assert.deepEqual(term, null);
         });
     }).call(this);
     (function test_binary_to_term_empty_list () {
@@ -245,7 +268,7 @@ var hex = function hex(buffer) {
             assert.ok(err instanceof Erlang.ParseException);
             assert.strictEqual(term, undefined);
         });
-        Erlang.binary_to_term('\x83l\0\0\0\1jd\0\4tail', function(err, lst) {
+        Erlang.binary_to_term('\x83l\0\0\0\1jv\0\4tail', function(err, lst) {
             assert.strictEqual(err, undefined);
             assert.ok(lst instanceof Erlang.OtpErlangList);
             assert.deepEqual([new Erlang.OtpErlangList([]),
@@ -374,12 +397,13 @@ var hex = function hex(buffer) {
         });
         Erlang.binary_to_term('\x83m\0\0\0\0', function(err, term) {
             assert.strictEqual(err, undefined);
-            assert.deepEqual(term, new Erlang.OtpErlangBinary(new Buffer([])));
+            assert.deepEqual(term, new Erlang.OtpErlangBinary(
+                                       new bufferFrom([])));
         });
         Erlang.binary_to_term('\x83m\0\0\0\4data', function(err, term) {
             assert.strictEqual(err, undefined);
             assert.deepEqual(term, new Erlang.OtpErlangBinary(
-                                       new Buffer('data', 'binary')));
+                                       new bufferFrom('data', 'binary')));
         });
     }).call(this);
     (function test_binary_to_term_float () {
@@ -494,6 +518,109 @@ var hex = function hex(buffer) {
             assert.equal(term, -6618611909121);
         });
     }).call(this);
+    (function test_binary_to_term_pid () {
+        var pid_old_binary = (
+            '\x83\x67\x64\x00\x0D\x6E\x6F\x6E\x6F\x64\x65\x40\x6E\x6F' +
+            '\x68\x6F\x73\x74\x00\x00\x00\x4E\x00\x00\x00\x00\x00'
+        );
+        Erlang.binary_to_term(pid_old_binary,
+                              function(err, term) {
+            assert.strictEqual(err, undefined);
+            assert.ok(term instanceof Erlang.OtpErlangPid);
+            Erlang.term_to_binary(term,
+                                  function(err, binary) {
+                assert.strictEqual(err, undefined);
+                assert.equal(binary.toString('binary'),
+                    '\x83gs\rnonode@nohost\x00\x00\x00N' +
+                    '\x00\x00\x00\x00\x00');
+            });
+        });
+        var pid_new_binary = (
+            '\x83\x58\x64\x00\x0D\x6E\x6F\x6E\x6F\x64\x65\x40\x6E\x6F\x68' +
+            '\x6F\x73\x74\x00\x00\x00\x4E\x00\x00\x00\x00\x00\x00\x00\x00'
+        );
+        Erlang.binary_to_term(pid_new_binary,
+                              function(err, term) {
+            assert.strictEqual(err, undefined);
+            assert.ok(term instanceof Erlang.OtpErlangPid);
+            Erlang.term_to_binary(term,
+                                  function(err, binary) {
+                assert.strictEqual(err, undefined);
+                assert.equal(binary.toString('binary'),
+                    '\x83Xs\rnonode@nohost\x00\x00\x00N' +
+                    '\x00\x00\x00\x00\x00\x00\x00\x00');
+            });
+        });
+    }).call(this);
+    (function test_binary_to_term_port () {
+        var port_old_binary = (
+            '\x83\x66\x64\x00\x0D\x6E\x6F\x6E\x6F\x64\x65\x40\x6E\x6F\x68' +
+            '\x6F\x73\x74\x00\x00\x00\x06\x00'
+        );
+        Erlang.binary_to_term(port_old_binary,
+                              function(err, term) {
+            assert.strictEqual(err, undefined);
+            assert.ok(term instanceof Erlang.OtpErlangPort);
+            Erlang.term_to_binary(term,
+                                  function(err, binary) {
+                assert.strictEqual(err, undefined);
+                assert.equal(binary.toString('binary'),
+                    '\x83fs\rnonode@nohost\x00\x00\x00\x06\x00');
+            });
+        });
+        var port_new_binary = (
+            '\x83\x59\x64\x00\x0D\x6E\x6F\x6E\x6F\x64\x65\x40\x6E\x6F\x68' +
+            '\x6F\x73\x74\x00\x00\x00\x06\x00\x00\x00\x00'
+        );
+        Erlang.binary_to_term(port_new_binary,
+                              function(err, term) {
+            assert.strictEqual(err, undefined);
+            assert.ok(term instanceof Erlang.OtpErlangPort);
+            Erlang.term_to_binary(term,
+                                  function(err, binary) {
+                assert.strictEqual(err, undefined);
+                assert.equal(binary.toString('binary'),
+                    '\x83Ys\rnonode@nohost\x00\x00\x00\x06' +
+                    '\x00\x00\x00\x00');
+            });
+        });
+    }).call(this);
+    (function test_binary_to_term_ref () {
+        var ref_new_binary = (
+            '\x83\x72\x00\x03\x64\x00\x0D\x6E\x6F\x6E\x6F\x64\x65\x40\x6E' +
+            '\x6F\x68\x6F\x73\x74\x00\x00\x03\xE8\x4E\xE7\x68\x00\x02\xA4' +
+            '\xC8\x53\x40'
+        );
+        Erlang.binary_to_term(ref_new_binary,
+                              function(err, term) {
+            assert.strictEqual(err, undefined);
+            assert.ok(term instanceof Erlang.OtpErlangReference);
+            Erlang.term_to_binary(term,
+                                  function(err, binary) {
+                assert.strictEqual(err, undefined);
+                assert.equal(binary.toString('binary'),
+                    '\x83r\x00\x03s\rnonode@nohost\x00\x00\x03\xe8' +
+                    'N\xe7h\x00\x02\xa4\xc8S@');
+            });
+        });
+        var ref_newer_binary = (
+            '\x83\x5A\x00\x03\x64\x00\x0D\x6E\x6F\x6E\x6F\x64\x65\x40\x6E' +
+            '\x6F\x68\x6F\x73\x74\x00\x00\x00\x00\x00\x01\xAC\x03\xC7\x00' +
+            '\x00\x04\xBB\xB2\xCA\xEE'
+        );
+        Erlang.binary_to_term(ref_newer_binary,
+                              function(err, term) {
+            assert.strictEqual(err, undefined);
+            assert.ok(term instanceof Erlang.OtpErlangReference);
+            Erlang.term_to_binary(term,
+                                  function(err, binary) {
+                assert.strictEqual(err, undefined);
+                assert.equal(binary.toString('binary'),
+                    '\x83Z\x00\x03s\rnonode@nohost\x00\x00\x00\x00\x00' +
+                    '\x01\xac\x03\xc7\x00\x00\x04\xbb\xb2\xca\xee');
+            });
+        });
+    }).call(this);
     (function test_binary_to_term_compressed_term () {
         Erlang.binary_to_term('\x83P', function(err, term) {
             assert.ok(err instanceof Erlang.ParseException);
@@ -767,20 +894,40 @@ var hex = function hex(buffer) {
             assert.strictEqual(err, undefined);
             assert.equal(binary.toString('binary'),
                 '\x83l\x00\x02\x00\x00' + 'a\xd0a\x90'.repeat(65536) + 'j');
-                         
+
         });
     }).call(this);
     (function test_term_to_binary_atom () {
         Erlang.term_to_binary(new Erlang.OtpErlangAtom(''),
                               function(err, binary) {
             assert.strictEqual(err, undefined);
+            assert.equal(binary.toString('binary'), '\x83w\0');
+        });
+        Erlang.term_to_binary(new Erlang.OtpErlangAtom('', false),
+                              function(err, binary) {
+            assert.strictEqual(err, undefined);
             assert.equal(binary.toString('binary'), '\x83s\0');
         });
         Erlang.term_to_binary(new Erlang.OtpErlangAtom('test'),
                               function(err, binary) {
             assert.strictEqual(err, undefined);
+            assert.equal(binary.toString('binary'), '\x83w\4test');
+        });
+        Erlang.term_to_binary(new Erlang.OtpErlangAtom('test', false),
+                              function(err, binary) {
+            assert.strictEqual(err, undefined);
             assert.equal(binary.toString('binary'), '\x83s\4test');
         });
+        Erlang.term_to_binary(new Erlang.OtpErlangAtomLarge('test'),
+                              function(err, binary) {
+            assert.strictEqual(err, undefined);
+            assert.equal(binary.toString('binary'), '\x83v\0\4test');
+        });
+        Erlang.term_to_binary(new Erlang.OtpErlangAtomLarge('test', false),
+                              function(err, binary) {
+            assert.strictEqual(err, undefined);
+            assert.equal(binary.toString('binary'), '\x83d\0\4test');
+        });
     }).call(this);
     (function test_term_to_binary_string_basic () {
         Erlang.term_to_binary('', function(err, binary) {
@@ -852,15 +999,15 @@ var hex = function hex(buffer) {
     (function test_term_to_binary_predefined_atom () {
         Erlang.term_to_binary(true, function(err, binary) {
             assert.strictEqual(err, undefined);
-            assert.equal(binary.toString('binary'), '\x83s\4true');
+            assert.equal(binary.toString('binary'), '\x83w\4true');
         });
         Erlang.term_to_binary(false, function(err, binary) {
             assert.strictEqual(err, undefined);
-            assert.equal(binary.toString('binary'), '\x83s\5false');
+            assert.equal(binary.toString('binary'), '\x83w\5false');
         });
-        Erlang.term_to_binary(undefined, function(err, binary) {
+        Erlang.term_to_binary(null, function(err, binary) {
             assert.strictEqual(err, undefined);
-            assert.equal(binary.toString('binary'), '\x83s\x09undefined');
+            assert.equal(binary.toString('binary'), '\x83w\x09undefined');
         });
     }).call(this);
     (function test_term_to_binary_short_integer () {
