@@ -5,7 +5,7 @@
 
   MIT License
 
-  Copyright (c) 2017-2021 Michael Truog <mjtruog at protonmail dot com>
+  Copyright (c) 2017-2022 Michael Truog <mjtruog at protonmail dot com>
 
   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
@@ -64,6 +64,7 @@ module Instance = struct
       socket : Unix.file_descr;
       use_header : bool;
       mutable initialization_complete : bool;
+      mutable fatal_exceptions : bool;
       mutable terminate : bool;
       fragment_size : int;
       fragment_recv : bytes;
@@ -96,6 +97,7 @@ module Instance = struct
     ~callbacks ~buffer_recv ~timeout_terminate =
     {state; terminate_exception; socket; use_header;
      initialization_complete = false;
+     fatal_exceptions = false;
      terminate = false;
      fragment_size; fragment_recv; callbacks; buffer_recv;
      process_index = 0;
@@ -116,7 +118,7 @@ module Instance = struct
   let init api process_index process_count
     process_count_max process_count_min prefix
     timeout_initialize timeout_async timeout_sync timeout_terminate
-    priority_default =
+    priority_default fatal_exceptions =
     api.process_index <- process_index ;
     api.process_count <- process_count ;
     api.process_count_max <- process_count_max ;
@@ -127,13 +129,15 @@ module Instance = struct
     api.timeout_sync <- timeout_sync ;
     api.timeout_terminate <- timeout_terminate ;
     api.priority_default <- priority_default ;
+    api.fatal_exceptions <- fatal_exceptions ;
     ()
   let reinit api process_count timeout_async timeout_sync
-    priority_default =
+    priority_default fatal_exceptions =
     api.process_count <- process_count ;
     api.timeout_async <- timeout_async ;
     api.timeout_sync <- timeout_sync ;
     api.priority_default <- priority_default ;
+    api.fatal_exceptions <- fatal_exceptions ;
     ()
   let set_response api response_info response trans_id =
     api.response_info <- response_info ;
@@ -268,6 +272,9 @@ let unpack_uint32_big i binary : (int, string) result =
       )
     )
 
+let unpack_uint8 i binary : int =
+  int_of_char binary.[i]
+
 let unpack_int8 i binary : int =
   let byte0 = int_of_char binary.[i] in
   if (byte0 lsr 7) = 1 then
@@ -365,7 +372,7 @@ let recv api : (string * int, string) result =
       Ok ((data, String.length data)))
 
 let forward_async_i
-  api name request_info request timeout priority trans_id pid :
+  api name request_info request timeout priority trans_id source :
   (unit, string) result =
   match Erlang.term_to_binary (
     Erlang.OtpErlangTuple ([
@@ -376,24 +383,24 @@ let forward_async_i
       Erlang.OtpErlangInteger (timeout);
       Erlang.OtpErlangInteger (priority);
       Erlang.OtpErlangBinary (trans_id);
-      Erlang.OtpErlangPid (pid)])) with
+      Erlang.OtpErlangPid (source)])) with
   | Error (error) ->
     Error (error)
   | Ok (forward) ->
     send api forward
 
 let forward_async
-  api name request_info request timeout priority trans_id pid :
+  api name request_info request timeout priority trans_id source :
   (unit, string) result =
   match forward_async_i
-    api name request_info request timeout priority trans_id pid with
+    api name request_info request timeout priority trans_id source with
   | Error (error) ->
     Error (error)
   | Ok _ ->
     raise ForwardAsync
 
 let forward_sync_i
-  api name request_info request timeout priority trans_id pid :
+  api name request_info request timeout priority trans_id source :
   (unit, string) result =
   match Erlang.term_to_binary (
     Erlang.OtpErlangTuple ([
@@ -404,33 +411,35 @@ let forward_sync_i
       Erlang.OtpErlangInteger (timeout);
       Erlang.OtpErlangInteger (priority);
       Erlang.OtpErlangBinary (trans_id);
-      Erlang.OtpErlangPid (pid)])) with
+      Erlang.OtpErlangPid (source)])) with
   | Error (error) ->
     Error (error)
   | Ok (forward) ->
     send api forward
 
 let forward_sync
-  api name request_info request timeout priority trans_id pid :
+  api name request_info request timeout priority trans_id source :
   (unit, string) result =
   match forward_sync_i
-    api name request_info request timeout priority trans_id pid with
+    api name request_info request timeout priority trans_id source with
   | Error (error) ->
     Error (error)
   | Ok _ ->
     raise ForwardSync
 
 let forward_
-  api request_type name request_info request timeout priority trans_id pid :
+  api request_type name request_info request timeout priority trans_id source :
   (unit, string) result =
   match request_type with
   | ASYNC ->
-    forward_async api name request_info request timeout priority trans_id pid
+    forward_async api name request_info request
+      timeout priority trans_id source
   | SYNC ->
-    forward_sync api name request_info request timeout priority trans_id pid
+    forward_sync api name request_info request
+      timeout priority trans_id source
 
 let return_async_i
-  api name pattern response_info response timeout trans_id pid :
+  api name pattern response_info response timeout trans_id source :
   (unit, string) result =
   match Erlang.term_to_binary (
     Erlang.OtpErlangTuple ([
@@ -441,24 +450,24 @@ let return_async_i
       Erlang.OtpErlangBinary (response);
       Erlang.OtpErlangInteger (timeout);
       Erlang.OtpErlangBinary (trans_id);
-      Erlang.OtpErlangPid (pid)])) with
+      Erlang.OtpErlangPid (source)])) with
   | Error (error) ->
     Error (error)
   | Ok (return) ->
     send api return
 
 let return_async
-  api name pattern response_info response timeout trans_id pid :
+  api name pattern response_info response timeout trans_id source :
   (unit, string) result =
   match return_async_i
-    api name pattern response_info response timeout trans_id pid with
+    api name pattern response_info response timeout trans_id source with
   | Error (error) ->
     Error (error)
   | Ok _ ->
     raise ReturnAsync
 
 let return_sync_i
-  api name pattern response_info response timeout trans_id pid :
+  api name pattern response_info response timeout trans_id source :
   (unit, string) result =
   match Erlang.term_to_binary (
     Erlang.OtpErlangTuple ([
@@ -469,30 +478,32 @@ let return_sync_i
       Erlang.OtpErlangBinary (response);
       Erlang.OtpErlangInteger (timeout);
       Erlang.OtpErlangBinary (trans_id);
-      Erlang.OtpErlangPid (pid)])) with
+      Erlang.OtpErlangPid (source)])) with
   | Error (error) ->
     Error (error)
   | Ok (return) ->
     send api return
 
 let return_sync
-  api name pattern response_info response timeout trans_id pid :
+  api name pattern response_info response timeout trans_id source :
   (unit, string) result =
   match return_sync_i
-    api name pattern response_info response timeout trans_id pid with
+    api name pattern response_info response timeout trans_id source with
   | Error (error) ->
     Error (error)
   | Ok _ ->
     raise ReturnSync
 
 let return_
-  api request_type name pattern response_info response timeout trans_id pid :
+  api request_type name pattern response_info response timeout trans_id source :
   (unit, string) result =
   match request_type with
   | ASYNC ->
-    return_async api name pattern response_info response timeout trans_id pid
+    return_async api name pattern response_info response
+      timeout trans_id source
   | SYNC ->
-    return_sync api name pattern response_info response timeout trans_id pid
+    return_sync api name pattern response_info response
+      timeout trans_id source
 
 let handle_events api ext data data_size i cmd : (bool, string) result =
   let i_cmd =
@@ -530,12 +541,14 @@ let handle_events api ext data data_size i cmd : (bool, string) result =
               Error (error)
             | Ok (timeout_sync) ->
               let priority_default = unpack_int8 (i1 + 12) data
-              and i2 = i1 + 13 in
+              and fatal_exceptions = (unpack_uint8 (i1 + 13) data) != 0
+              and i2 = i1 + 14 in
               Instance.reinit api
                 process_count
                 timeout_async
                 timeout_sync
-                priority_default ;
+                priority_default
+                fatal_exceptions ;
               loop_cmd i2)
       else if cmd_event = message_keepalive then
         match Erlang.term_to_binary (Erlang.OtpErlangAtom ("keepalive")) with
@@ -620,8 +633,8 @@ let rec poll_request_loop api timeout ext poll_timer: (bool, string) result =
 
 and callback
   api request_type name pattern request_info request
-  timeout priority trans_id pid : (bool option, string) result =
-  let {Instance.state; callbacks; _} = api in
+  timeout priority trans_id source : (bool option, string) result =
+  let {Instance.fatal_exceptions; state; callbacks; _} = api in
   let callback_get () =
     let function_queue = Hashtbl.find callbacks pattern in
     let f = Queue.pop function_queue in
@@ -638,7 +651,7 @@ and callback
       try Some (
         callback_f
           request_type name pattern request_info request
-          timeout priority trans_id pid state api)
+          timeout priority trans_id source state api)
       with
         | Terminate ->
           Some (Null)
@@ -661,13 +674,15 @@ and callback
             exit 1 ;
           | FatalError ->
             exit 1 ;
+          | _ when fatal_exceptions ->
+            exit 1 ;
           | _ ->
             Some (Null))
     | SYNC -> (
       try Some (
         callback_f
           request_type name pattern request_info request
-          timeout priority trans_id pid state api)
+          timeout priority trans_id source state api)
       with
         | Terminate ->
           Some (Null)
@@ -690,6 +705,8 @@ and callback
             exit 1 ;
           | FatalError ->
             exit 1 ;
+          | _ when fatal_exceptions ->
+            exit 1 ;
           | _ ->
             Some (Null))
   in
@@ -719,20 +736,20 @@ and callback
         Ok (())
       | ReturnI (response_info, response) ->
         return_async_i
-          api name pattern response_info response timeout trans_id pid
+          api name pattern response_info response timeout trans_id source
       | ForwardI (name_, request_info_, request_, timeout_, priority_) ->
         forward_async_i
-          api name_ request_info_ request_ timeout_ priority_ trans_id pid)
+          api name_ request_info_ request_ timeout_ priority_ trans_id source)
     | SYNC -> (
       match callback_result_type with
       | Finished ->
         Ok (())
       | ReturnI (response_info, response) ->
         return_sync_i
-          api name pattern response_info response timeout trans_id pid
+          api name pattern response_info response timeout trans_id source
       | ForwardI (name_, request_info_, request_, timeout_, priority_) ->
         forward_sync_i
-          api name_ request_info_ request_ timeout_ priority_ trans_id pid)
+          api name_ request_info_ request_ timeout_ priority_ trans_id source)
   in
   match return_result with
   | Error (error) ->
@@ -787,7 +804,9 @@ and poll_request_data api ext data data_size i : (bool option, string) result =
                       | Ok (timeout_terminate) ->
                         let priority_default =
                           unpack_int8 (i1 + 16) data
-                        and i2 = i1 + 17 in
+                        and fatal_exceptions =
+                          (unpack_uint8 (i1 + 17) data) != 0
+                        and i2 = i1 + 18 in
                         Instance.init api
                           process_index
                           process_count
@@ -798,7 +817,8 @@ and poll_request_data api ext data data_size i : (bool option, string) result =
                           timeout_async
                           timeout_sync
                           timeout_terminate
-                          priority_default ;
+                          priority_default
+                          fatal_exceptions ;
                         if i2 <> data_size then
                           match handle_events api ext data data_size i2 0 with
                           | Error (error) ->
@@ -846,11 +866,11 @@ and poll_request_data api ext data data_size i : (bool option, string) result =
                 match unpack_uint32_native i8 data with
                 | Error (error) ->
                   Error (error)
-                | Ok (pid_size) ->
+                | Ok (source_size) ->
                   let i9 = i8 + 4 in
-                  let pid_data = String.sub data i9 pid_size
-                  and i10 = i9 + pid_size in
-                  match Erlang.binary_to_term pid_data with
+                  let source_data = String.sub data i9 source_size
+                  and i10 = i9 + source_size in
+                  match Erlang.binary_to_term source_data with
                   | Error (error) ->
                     Error (error)
                   | Ok (
@@ -872,7 +892,7 @@ and poll_request_data api ext data data_size i : (bool option, string) result =
                     | Erlang.OtpErlangReference _
                     | Erlang.OtpErlangFunction _) ->
                     Error (message_decoding_error)
-                  | Ok (Erlang.OtpErlangPid (pid)) ->
+                  | Ok (Erlang.OtpErlangPid (source)) ->
                     let handled =
                       if i10 <> data_size then
                         handle_events api ext data data_size i10 0
@@ -894,7 +914,7 @@ and poll_request_data api ext data data_size i : (bool option, string) result =
                       let callback_result =
                         callback
                           api request_type name pattern request_info request
-                          timeout priority trans_id pid in
+                          timeout priority trans_id source in
                       if api.Instance.terminate then
                         Ok (Some false)
                       else
@@ -1000,12 +1020,14 @@ and poll_request_data api ext data data_size i : (bool option, string) result =
             Error (error)
           | Ok (timeout_sync) ->
             let priority_default = unpack_int8 (i + 16) data
-            and i1 = i + 17 in
+            and fatal_exceptions = (unpack_uint8 (i + 17) data) != 0
+            and i1 = i + 18 in
             Instance.reinit api
               process_count
               timeout_async
               timeout_sync
-              priority_default ;
+              priority_default
+              fatal_exceptions ;
             if i1 = data_size then
               Ok (None)
             else if i1 < data_size then
