@@ -3,7 +3,7 @@
 #
 # MIT License
 #
-# Copyright (c) 2014-2021 Michael Truog <mjtruog at protonmail dot com>
+# Copyright (c) 2014-2022 Michael Truog <mjtruog at protonmail dot com>
 #
 # Permission is hereby granted, free of charge, to any person obtaining a
 # copy of this software and associated documentation files (the "Software"),
@@ -29,7 +29,7 @@ use strict;
 use warnings;
 use 5.010;
 
-$CloudI::API::VERSION = '2.04';
+$CloudI::API::VERSION = '2.05';
 
 use POSIX qw(getenv);
 use IO::Handle;
@@ -111,6 +111,7 @@ sub new
         _use_header => $use_header,
         _s => $s,
         _initialization_complete => 0,
+        _fatal_exceptions => 0,
         _terminate => 0,
         _size => $buffer_size,
         _callbacks => \%callbacks,
@@ -135,7 +136,8 @@ sub new
      $self->{_timeout_async},
      $self->{_timeout_sync},
      $self->{_timeout_terminate},
-     $self->{_priority_default}) = $self->_poll_request(undef, 0);
+     $self->{_priority_default},
+     $self->{_fatal_exceptions}) = $self->_poll_request(undef, 0);
     return $self;
 }
 
@@ -295,16 +297,16 @@ sub forward_
 {
     my $self = shift;
     my ($request_type, $name, $request_info, $request,
-        $timeout, $priority, $trans_id, $pid) = @_;
+        $timeout, $priority, $trans_id, $source) = @_;
     if ($request_type == ASYNC)
     {
         $self->forward_async($name, $request_info, $request,
-                             $timeout, $priority, $trans_id, $pid);
+                             $timeout, $priority, $trans_id, $source);
     }
     elsif ($request_type == SYNC)
     {
         $self->forward_sync($name, $request_info, $request,
-                            $timeout, $priority, $trans_id, $pid);
+                            $timeout, $priority, $trans_id, $source);
     }
     else
     {
@@ -316,12 +318,12 @@ sub forward_async
 {
     my $self = shift;
     my ($name, $request_info, $request,
-        $timeout, $priority, $trans_id, $pid) = @_;
+        $timeout, $priority, $trans_id, $source) = @_;
     $self->_send(Erlang::term_to_binary([
         Erlang::OtpErlangAtom->new('forward_async'), $name,
         Erlang::OtpErlangBinary->new($request_info),
         Erlang::OtpErlangBinary->new($request), $timeout, $priority,
-        Erlang::OtpErlangBinary->new($trans_id), $pid]));
+        Erlang::OtpErlangBinary->new($trans_id), $source]));
     die CloudI::ForwardAsyncException->new();
 }
 
@@ -329,12 +331,12 @@ sub forward_sync
 {
     my $self = shift;
     my ($name, $request_info, $request,
-        $timeout, $priority, $trans_id, $pid) = @_;
+        $timeout, $priority, $trans_id, $source) = @_;
     $self->_send(Erlang::term_to_binary([
         Erlang::OtpErlangAtom->new('forward_sync'), $name,
         Erlang::OtpErlangBinary->new($request_info),
         Erlang::OtpErlangBinary->new($request), $timeout, $priority,
-        Erlang::OtpErlangBinary->new($trans_id), $pid]));
+        Erlang::OtpErlangBinary->new($trans_id), $source]));
     die CloudI::ForwardSyncException->new();
 }
 
@@ -342,16 +344,16 @@ sub return_
 {
     my $self = shift;
     my ($request_type, $name, $pattern, $response_info, $response,
-        $timeout, $trans_id, $pid) = @_;
+        $timeout, $trans_id, $source) = @_;
     if ($request_type == ASYNC)
     {
         $self->return_async($name, $pattern, $response_info, $response,
-                            $timeout, $trans_id, $pid);
+                            $timeout, $trans_id, $source);
     }
     elsif ($request_type == SYNC)
     {
         $self->return_sync($name, $pattern, $response_info, $response,
-                           $timeout, $trans_id, $pid);
+                           $timeout, $trans_id, $source);
     }
     else
     {
@@ -363,12 +365,12 @@ sub return_async
 {
     my $self = shift;
     my ($name, $pattern, $response_info, $response,
-        $timeout, $trans_id, $pid) = @_;
+        $timeout, $trans_id, $source) = @_;
     $self->_send(Erlang::term_to_binary([
         Erlang::OtpErlangAtom->new('return_async'), $name, $pattern,
         Erlang::OtpErlangBinary->new($response_info),
         Erlang::OtpErlangBinary->new($response), $timeout,
-        Erlang::OtpErlangBinary->new($trans_id), $pid]));
+        Erlang::OtpErlangBinary->new($trans_id), $source]));
     die CloudI::ReturnAsyncException->new();
 }
 
@@ -376,12 +378,12 @@ sub return_sync
 {
     my $self = shift;
     my ($name, $pattern, $response_info, $response,
-        $timeout, $trans_id, $pid) = @_;
+        $timeout, $trans_id, $source) = @_;
     $self->_send(Erlang::term_to_binary([
         Erlang::OtpErlangAtom->new('return_sync'), $name, $pattern,
         Erlang::OtpErlangBinary->new($response_info),
         Erlang::OtpErlangBinary->new($response), $timeout,
-        Erlang::OtpErlangBinary->new($trans_id), $pid]));
+        Erlang::OtpErlangBinary->new($trans_id), $source]));
     die CloudI::ReturnSyncException->new();
 }
 
@@ -474,7 +476,7 @@ sub _null_response
 {
     my $self = shift;
     my ($request_type, $name, $pattern, $request_info, $request,
-        $timeout, $priority, $trans_id, $pid) = @_;
+        $timeout, $priority, $trans_id, $source) = @_;
     return '';
 }
 
@@ -482,7 +484,7 @@ sub _callback
 {
     my $self = shift;
     my ($command, $name, $pattern, $request_info, $request,
-        $timeout, $priority, $trans_id, $pid) = @_;
+        $timeout, $priority, $trans_id, $source) = @_;
     my $function;
     if (! defined($self->{_callbacks}{$pattern}))
     {
@@ -503,7 +505,7 @@ sub _callback
             ($response_info,
              $response) = &$function(ASYNC, $name, $pattern,
                                      $request_info, $request,
-                                     $timeout, $priority, $trans_id, $pid);
+                                     $timeout, $priority, $trans_id, $source);
             if (! defined($response))
             {
                 $response = $response_info;
@@ -554,6 +556,10 @@ sub _callback
             else
             {
                 print STDERR "$e";
+                if ($self->{_fatal_exceptions})
+                {
+                    exit(1);
+                }
             }
             $response_info = '';
             $response = '';
@@ -561,7 +567,7 @@ sub _callback
         eval
         {
             $self->return_async($name, $pattern, $response_info, $response,
-                                $timeout, $trans_id, $pid);
+                                $timeout, $trans_id, $source);
         };
         $e = $@;
         if ($e)
@@ -580,7 +586,7 @@ sub _callback
             ($response_info,
              $response) = &$function(SYNC, $name, $pattern,
                                      $request_info, $request,
-                                     $timeout, $priority, $trans_id, $pid);
+                                     $timeout, $priority, $trans_id, $source);
             if (! defined($response))
             {
                 $response = $response_info;
@@ -631,6 +637,10 @@ sub _callback
             else
             {
                 print STDERR "$e";
+                if ($self->{_fatal_exceptions})
+                {
+                    exit(1);
+                }
             }
             $response_info = '';
             $response = '';
@@ -638,7 +648,7 @@ sub _callback
         eval
         {
             $self->return_sync($name, $pattern, $response_info, $response,
-                               $timeout, $trans_id, $pid);
+                               $timeout, $trans_id, $source);
         };
         $e = $@;
         if ($e)
@@ -691,11 +701,12 @@ sub _handle_events
         }
         elsif ($command == MESSAGE_REINIT)
         {
-            $i += $j; $j = 4 + 4 + 4 + 1;
+            $i += $j; $j = 4 + 4 + 4 + 1 + 1;
             ($self->{_process_count},
              $self->{_timeout_async},
              $self->{_timeout_sync},
-             $self->{_priority_default}) = unpack("L3 c",
+             $self->{_priority_default},
+             $self->{_fatal_exceptions}) = unpack("L3 c C",
                                                   substr($data, $i, $j));
             $i += $j;
         }
@@ -793,13 +804,14 @@ sub _poll_request
                 $process_count_max,
                 $process_count_min,
                 $prefix_size) = unpack('L5', substr($data, $i, $j));
-            $i += $j; $j = $prefix_size + 4 + 4 + 4 + 4 + 1;
+            $i += $j; $j = $prefix_size + 4 + 4 + 4 + 4 + 1 + 1;
             my ($prefix,
                 $timeout_initialize,
                 $timeout_async,
                 $timeout_sync,
                 $timeout_terminate,
-                $priority_default) = unpack("Z$prefix_size L4 c",
+                $priority_default,
+                $fatal_exceptions) = unpack("Z$prefix_size L4 c C",
                                             substr($data, $i, $j));
             $i += $j;
             if ($i != $data_size)
@@ -810,8 +822,8 @@ sub _poll_request
             return ($process_index, $process_count,
                     $process_count_max, $process_count_min,
                     $prefix, $timeout_initialize,
-                    $timeout_sync, $timeout_async,
-                    $timeout_terminate, $priority_default);
+                    $timeout_sync, $timeout_async, $timeout_terminate,
+                    $priority_default, $fatal_exceptions);
         }
         elsif ($command == MESSAGE_SEND_ASYNC or
                $command == MESSAGE_SEND_SYNC)
@@ -834,10 +846,10 @@ sub _poll_request
                 $request_timeout,
                 $priority,
                 $trans_id,
-                $pid_size) = unpack("a$request_size x L c a16 L",
+                $source_size) = unpack("a$request_size x L c a16 L",
                                     substr($data, $i, $j));
-            $i += $j; $j = $pid_size;
-            my $pid = substr($data, $i, $j);
+            $i += $j; $j = $source_size;
+            my $source = substr($data, $i, $j);
             $i += $j;
             if ($i != $data_size)
             {
@@ -851,7 +863,7 @@ sub _poll_request
             $self->_callback($command, $name, $pattern,
                              $request_info, $request,
                              $request_timeout, $priority, $trans_id,
-                             Erlang::binary_to_term($pid));
+                             Erlang::binary_to_term($source));
             if ($self->{_terminate})
             {
                 return 0;
@@ -928,11 +940,12 @@ sub _poll_request
         }
         elsif ($command == MESSAGE_REINIT)
         {
-            $i += $j; $j = 4 + 4 + 4 + 1;
+            $i += $j; $j = 4 + 4 + 4 + 1 + 1;
             ($self->{_process_count},
              $self->{_timeout_async},
              $self->{_timeout_sync},
-             $self->{_priority_default}) = unpack("L3 c",
+             $self->{_priority_default},
+             $self->{_fatal_exceptions}) = unpack("L3 c C",
                                                   substr($data, $i, $j));
             $i += $j; $j = 4;
             if ($i == $data_size)
