@@ -3,7 +3,7 @@
 //
 // MIT License
 //
-// Copyright (c) 2014-2020 Michael Truog <mjtruog at protonmail dot com>
+// Copyright (c) 2014-2022 Michael Truog <mjtruog at protonmail dot com>
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software"),
@@ -63,6 +63,7 @@ class API
     private $use_header;
     private $s;
     private $initialization_complete;
+    private $fatal_exceptions;
     private $terminate;
     private $size;
     private $callbacks;
@@ -98,6 +99,7 @@ class API
         else
             throw new InvalidInputException();
         $this->initialization_complete = false;
+        $this->fatal_exceptions = false;
         $this->terminate = false;
         $this->size = intval($buffer_size_str);
         $this->callbacks = array();
@@ -112,7 +114,8 @@ class API
              $this->timeout_async,
              $this->timeout_sync,
              $this->timeout_terminate,
-             $this->priority_default
+             $this->priority_default,
+             $this->fatal_exceptions
              ) = $this->poll_request(null, false);
     }
 
@@ -212,75 +215,75 @@ class API
     }
 
     public function forward_($request_type, $name, $request_info, $request,
-                             $timeout, $priority, $trans_id, $pid)
+                             $timeout, $priority, $trans_id, $source)
     {
         switch ($request_type)
         {
             case API::$ASYNC:
                 $this->forward_async($name, $request_info, $request,
-                                     $timeout, $priority, $trans_id, $pid);
+                                     $timeout, $priority, $trans_id, $source);
             case API::$SYNC:
                 $this->forward_sync($name, $request_info, $request,
-                                    $timeout, $priority, $trans_id, $pid);
+                                    $timeout, $priority, $trans_id, $source);
         }
     }
 
     public function forward_async($name, $request_info, $request,
-                                  $timeout, $priority, $trans_id, $pid)
+                                  $timeout, $priority, $trans_id, $source)
     {
         $this->send(\Erlang\term_to_binary(
             array(new \Erlang\OtpErlangAtom('forward_async'), $name,
                   new \Erlang\OtpErlangBinary($request_info),
                   new \Erlang\OtpErlangBinary($request), $timeout, $priority,
-                  new \Erlang\OtpErlangBinary($trans_id), $pid)));
+                  new \Erlang\OtpErlangBinary($trans_id), $source)));
         throw new ForwardAsyncException();
     }
 
     public function forward_sync($name, $request_info, $request,
-                                 $timeout, $priority, $trans_id, $pid)
+                                 $timeout, $priority, $trans_id, $source)
     {
         $this->send(\Erlang\term_to_binary(
             array(new \Erlang\OtpErlangAtom('forward_sync'), $name,
                   new \Erlang\OtpErlangBinary($request_info),
                   new \Erlang\OtpErlangBinary($request), $timeout, $priority,
-                  new \Erlang\OtpErlangBinary($trans_id), $pid)));
+                  new \Erlang\OtpErlangBinary($trans_id), $source)));
         throw new ForwardSyncException();
     }
 
     public function return_($request_type, $name, $pattern,
                             $response_info, $response,
-                            $timeout, $trans_id, $pid)
+                            $timeout, $trans_id, $source)
     {
         switch ($request_type)
         {
             case API::$ASYNC:
                 $this->return_async($name, $pattern, $response_info, $response,
-                                    $timeout, $trans_id, $pid);
+                                    $timeout, $trans_id, $source);
             case API::$SYNC:
                 $this->return_sync($name, $pattern, $response_info, $response,
-                                   $timeout, $trans_id, $pid);
+                                   $timeout, $trans_id, $source);
         }
     }
 
     public function return_async($name, $pattern, $response_info, $response,
-                                 $timeout, $trans_id, $pid)
+                                 $timeout, $trans_id, $source)
     {
         $this->send(\Erlang\term_to_binary(
             array(new \Erlang\OtpErlangAtom('return_async'), $name, $pattern,
                   new \Erlang\OtpErlangBinary($response_info),
                   new \Erlang\OtpErlangBinary($response), $timeout,
-                  new \Erlang\OtpErlangBinary($trans_id), $pid)));
+                  new \Erlang\OtpErlangBinary($trans_id), $source)));
         throw new ReturnAsyncException();
     }
 
     public function return_sync($name, $pattern, $response_info, $response,
-                                $timeout, $trans_id, $pid)
+                                $timeout, $trans_id, $source)
     {
         $this->send(\Erlang\term_to_binary(
             array(new \Erlang\OtpErlangAtom('return_sync'), $name, $pattern,
                   new \Erlang\OtpErlangBinary($response_info),
                   new \Erlang\OtpErlangBinary($response), $timeout,
-                  new \Erlang\OtpErlangBinary($trans_id), $pid)));
+                  new \Erlang\OtpErlangBinary($trans_id), $source)));
         throw new ReturnSyncException();
     }
 
@@ -342,16 +345,21 @@ class API
         return $this->timeout_terminate;
     }
 
+    public function priority_default()
+    {
+        return $this->priority_default;
+    }
+
     private function null_response($request_type, $name, $pattern,
                                    $request_info, $request,
-                                   $timeout, $priority, $trans_id, $pid)
+                                   $timeout, $priority, $trans_id, $source)
     {
         return '';
     }
 
     private function callback($command, $name, $pattern,
                               $request_info, $request,
-                              $timeout, $priority, $trans_id, $pid)
+                              $timeout, $priority, $trans_id, $source)
     {
         if (! isset($this->callbacks[$pattern]))
         {
@@ -373,7 +381,7 @@ class API
                                                API::$ASYNC, $name, $pattern,
                                                $request_info, $request,
                                                $timeout, $priority,
-                                               $trans_id, $pid);
+                                               $trans_id, $source);
                     if (is_array($response))
                     {
                         assert(count($response) == 2);
@@ -425,8 +433,12 @@ class API
                 }
                 catch (\Throwable $e)
                 {
-                    $return_null_response = true;
                     fwrite(STDERR, "{$e->getMessage()}\n{$e}\n");
+                    if ($this->fatal_exceptions)
+                    {
+                        exit(1);
+                    }
+                    $return_null_response = true;
                 }
                 if ($return_null_response)
                 {
@@ -437,7 +449,7 @@ class API
                 {
                     $this->return_async($name, $pattern,
                                         $response_info, $response,
-                                        $timeout, $trans_id, $pid);
+                                        $timeout, $trans_id, $source);
                 }
                 catch (ReturnAsyncException $e)
                 {
@@ -450,7 +462,7 @@ class API
                                                API::$SYNC, $name, $pattern,
                                                $request_info, $request,
                                                $timeout, $priority,
-                                               $trans_id, $pid);
+                                               $trans_id, $source);
                     if (is_array($response))
                     {
                         assert(count($response) == 2);
@@ -502,8 +514,12 @@ class API
                 }
                 catch (\Throwable $e)
                 {
-                    $return_null_response = true;
                     fwrite(STDERR, "{$e->getMessage()}\n{$e}\n");
+                    if ($this->fatal_exceptions)
+                    {
+                        exit(1);
+                    }
+                    $return_null_response = true;
                 }
                 if ($return_null_response)
                 {
@@ -514,7 +530,7 @@ class API
                 {
                     $this->return_sync($name, $pattern,
                                        $response_info, $response,
-                                       $timeout, $trans_id, $pid);
+                                       $timeout, $trans_id, $source);
                 }
                 catch (ReturnSyncException $e)
                 {
@@ -550,12 +566,13 @@ class API
                     else
                         throw new TerminateException($this->timeout_terminate);
                 case MESSAGE_REINIT:
-                    $i += $j; $j = 4 + 4 + 4 + 1;
-                    $tmp = unpack('L3a/cb', substr($data, $i, $j));
+                    $i += $j; $j = 4 + 4 + 4 + 1 + 1;
+                    $tmp = unpack('L3a/cb/Cc', substr($data, $i, $j));
                     $this->process_count = $tmp['a1'];
                     $this->timeout_async = $tmp['a2'];
                     $this->timeout_sync = $tmp['a3'];
                     $this->priority_default = $tmp['b'];
+                    $this->fatal_exceptions = $tmp['c'];
                     $i += $j;
                     break;
                 case MESSAGE_KEEPALIVE:
@@ -640,13 +657,14 @@ class API
                          $prefix_size) = unpack('L5', substr($data, $i, $j));
                     $i += $j; $j = $prefix_size;
                     $prefix = substr($data, $i, $j - 1);
-                    $i += $j; $j = 4 + 4 + 4 + 4 + 1;
-                    $tmp = unpack('L4a/cb', substr($data, $i, $j));
+                    $i += $j; $j = 4 + 4 + 4 + 4 + 1 + 1;
+                    $tmp = unpack('L4a/cb/Cc', substr($data, $i, $j));
                     $timeout_initialize = $tmp['a1'];
                     $timeout_async = $tmp['a2'];
                     $timeout_sync = $tmp['a3'];
                     $timeout_terminate = $tmp['a4'];
                     $priority_default = $tmp['b'];
+                    $fatal_exceptions = $tmp['c'];
                     $i += $j;
                     if ($i != $data_size)
                     {
@@ -657,7 +675,8 @@ class API
                                  $process_count_max, $process_count_min,
                                  $prefix, $timeout_initialize,
                                  $timeout_sync, $timeout_async,
-                                 $timeout_terminate, $priority_default);
+                                 $timeout_terminate,
+                                 $priority_default, $fatal_exceptions);
                 case MESSAGE_SEND_ASYNC:
                 case MESSAGE_SEND_SYNC:
                     $i += $j; $j = 4;
@@ -686,9 +705,9 @@ class API
                     $i += $j; $j = 16;
                     $trans_id = substr($data, $i, $j);
                     $i += $j; $j = 4;
-                    list(, $pid_size) = unpack('L', substr($data, $i, $j));
-                    $i += $j; $j = $pid_size;
-                    $pid = substr($data, $i, $j);
+                    list(, $source_size) = unpack('L', substr($data, $i, $j));
+                    $i += $j; $j = $source_size;
+                    $source = substr($data, $i, $j);
                     $i += $j;
                     if ($i != $data_size)
                     {
@@ -703,7 +722,7 @@ class API
                     $this->callback($command, $name, $pattern,
                                     $request_info, $request,
                                     $request_timeout, $priority, $trans_id,
-                                    \Erlang\binary_to_term($pid));
+                                    \Erlang\binary_to_term($source));
                     if ($this->terminate)
                     {
                         return false;
@@ -776,12 +795,13 @@ class API
                     }
                     assert(false);
                 case MESSAGE_REINIT:
-                    $i += $j; $j = 4 + 4 + 4 + 1;
-                    $tmp = unpack('L3a/cb', substr($data, $i, $j));
+                    $i += $j; $j = 4 + 4 + 4 + 1 + 1;
+                    $tmp = unpack('L3a/cb/Cc', substr($data, $i, $j));
                     $this->process_count = $tmp['a1'];
                     $this->timeout_async = $tmp['a2'];
                     $this->timeout_sync = $tmp['a3'];
                     $this->priority_default = $tmp['b'];
+                    $this->fatal_exceptions = $tmp['c'];
                     $i += $j; $j = 4;
                     if ($i == $data_size)
                     {
