@@ -4,7 +4,7 @@
 #
 # MIT License
 #
-# Copyright (c) 2011-2020 Michael Truog <mjtruog at protonmail dot com>
+# Copyright (c) 2011-2022 Michael Truog <mjtruog at protonmail dot com>
 #
 # Permission is hereby granted, free of charge, to any person obtaining a
 # copy of this software and associated documentation files (the "Software"),
@@ -124,7 +124,8 @@ class API(object):
          self.__timeout_initialize,
          self.__timeout_async, self.__timeout_sync,
          self.__timeout_terminate,
-         self.__priority_default) = self.__poll_request(None, False)
+         self.__priority_default,
+         self.__fatal_exceptions) = self.__poll_request(None, False)
 
     @staticmethod
     def thread_count():
@@ -232,7 +233,7 @@ class API(object):
         return self.__poll_request(None, False)
 
     def forward_(self, request_type, name, request_info, request,
-                 timeout, priority, trans_id, pid):
+                 timeout, priority, trans_id, source):
         """
         forwards a service request to a different service name
         """
@@ -240,16 +241,16 @@ class API(object):
         if request_type == API.ASYNC:
             self.forward_async(name,
                                request_info, request,
-                               timeout, priority, trans_id, pid)
+                               timeout, priority, trans_id, source)
         elif request_type == API.SYNC:
             self.forward_sync(name,
                               request_info, request,
-                              timeout, priority, trans_id, pid)
+                              timeout, priority, trans_id, source)
         else:
             raise InvalidInputException()
 
     def forward_async(self, name, request_info, request,
-                      timeout, priority, trans_id, pid):
+                      timeout, priority, trans_id, source):
         """
         forwards an asynchronous service request to a different service name
         """
@@ -258,11 +259,11 @@ class API(object):
                                     OtpErlangBinary(request_info),
                                     OtpErlangBinary(request),
                                     timeout, priority,
-                                    OtpErlangBinary(trans_id), pid)))
+                                    OtpErlangBinary(trans_id), source)))
         raise ForwardAsyncException()
 
     def forward_sync(self, name, request_info, request,
-                     timeout, priority, trans_id, pid):
+                     timeout, priority, trans_id, source):
         """
         forwards a synchronous service request to a different service name
         """
@@ -271,11 +272,11 @@ class API(object):
                                     OtpErlangBinary(request_info),
                                     OtpErlangBinary(request),
                                     timeout, priority,
-                                    OtpErlangBinary(trans_id), pid)))
+                                    OtpErlangBinary(trans_id), source)))
         raise ForwardSyncException()
 
     def return_(self, request_type, name, pattern, response_info, response,
-                timeout, trans_id, pid):
+                timeout, trans_id, source):
         """
         provides a response to a service request
         """
@@ -283,16 +284,16 @@ class API(object):
         if request_type == API.ASYNC:
             self.return_async(name, pattern,
                               response_info, response,
-                              timeout, trans_id, pid)
+                              timeout, trans_id, source)
         elif request_type == API.SYNC:
             self.return_sync(name, pattern,
                              response_info, response,
-                             timeout, trans_id, pid)
+                             timeout, trans_id, source)
         else:
             raise InvalidInputException()
 
     def return_async(self, name, pattern, response_info, response,
-                     timeout, trans_id, pid):
+                     timeout, trans_id, source):
         """
         provides a response to an asynchronous service request
         """
@@ -301,11 +302,11 @@ class API(object):
                                     name, pattern,
                                     OtpErlangBinary(response_info),
                                     OtpErlangBinary(response), timeout,
-                                    OtpErlangBinary(trans_id), pid)))
+                                    OtpErlangBinary(trans_id), source)))
         raise ReturnAsyncException()
 
     def return_sync(self, name, pattern, response_info, response,
-                    timeout, trans_id, pid):
+                    timeout, trans_id, source):
         """
         provides a response to a synchronous service request
         """
@@ -314,7 +315,7 @@ class API(object):
                                     name, pattern,
                                     OtpErlangBinary(response_info),
                                     OtpErlangBinary(response), timeout,
-                                    OtpErlangBinary(trans_id), pid)))
+                                    OtpErlangBinary(trans_id), source)))
         raise ReturnSyncException()
 
     def recv_async(self, timeout=None, trans_id=None, consume=True):
@@ -383,16 +384,22 @@ class API(object):
         """
         return self.__timeout_terminate
 
+    def priority_default(self):
+        """
+        returns the default service request send priority
+        """
+        return self.__priority_default
+
     def __null_response(self, request_type, name, pattern,
                         request_info, request,
-                        timeout, priority, trans_id, pid):
+                        timeout, priority, trans_id, source):
         # pylint: disable=no-self-use
         # pylint: disable=too-many-arguments
         # pylint: disable=unused-argument
         return b''
 
     def __callback(self, command, name, pattern, request_info, request,
-                   timeout, priority, trans_id, pid):
+                   timeout, priority, trans_id, source):
         # pylint: disable=too-many-arguments
         # pylint: disable=bare-except
         # pylint: disable=too-many-return-statements
@@ -411,7 +418,7 @@ class API(object):
             try:
                 response = function(API.ASYNC, name, pattern,
                                     request_info, request,
-                                    timeout, priority, trans_id, pid)
+                                    timeout, priority, trans_id, source)
                 if isinstance(response, tuple):
                     response_info, response = response
                     if not isinstance(response_info, (bytes, TypeUnicode)):
@@ -444,8 +451,10 @@ class API(object):
                 traceback.print_exc(file=sys.stderr)
                 raise
             except Exception:
-                return_null_response = True
                 traceback.print_exc(file=sys.stderr)
+                if self.__fatal_exceptions:
+                    sys.exit(1)
+                return_null_response = True
             except:
                 traceback.print_exc(file=sys.stderr)
                 sys.exit(1)
@@ -455,7 +464,7 @@ class API(object):
             try:
                 self.return_async(name, pattern,
                                   response_info, response,
-                                  timeout, trans_id, pid)
+                                  timeout, trans_id, source)
             except ReturnAsyncException:
                 pass
             return
@@ -463,7 +472,7 @@ class API(object):
             try:
                 response = function(API.SYNC, name, pattern,
                                     request_info, request,
-                                    timeout, priority, trans_id, pid)
+                                    timeout, priority, trans_id, source)
                 if isinstance(response, tuple):
                     response_info, response = response
                     if not isinstance(response_info, (bytes, TypeUnicode)):
@@ -496,8 +505,10 @@ class API(object):
                 traceback.print_exc(file=sys.stderr)
                 raise
             except Exception:
-                return_null_response = True
                 traceback.print_exc(file=sys.stderr)
+                if self.__fatal_exceptions:
+                    sys.exit(1)
+                return_null_response = True
             except:
                 traceback.print_exc(file=sys.stderr)
                 sys.exit(1)
@@ -507,7 +518,7 @@ class API(object):
             try:
                 self.return_sync(name, pattern,
                                  response_info, response,
-                                 timeout, trans_id, pid)
+                                 timeout, trans_id, source)
             except ReturnSyncException:
                 pass
             return
@@ -527,11 +538,12 @@ class API(object):
                     return False
                 raise TerminateException(self.__timeout_terminate)
             if command == _MESSAGE_REINIT:
-                i, j = j, j + 4 + 4 + 4 + 1
+                i, j = j, j + 4 + 4 + 4 + 1 + 1
                 (self.__process_count,
                  self.__timeout_async, self.__timeout_sync,
-                 self.__priority_default) = struct.unpack(
-                     b'=IIIb', data[i:j]
+                 self.__priority_default,
+                 self.__fatal_exceptions) = struct.unpack(
+                     b'=IIIbB', data[i:j]
                  )
             elif command == _MESSAGE_KEEPALIVE:
                 self.__send(term_to_binary(OtpErlangAtom(b'keepalive')))
@@ -586,11 +598,11 @@ class API(object):
                 (process_index, process_count,
                  process_count_max, process_count_min,
                  prefix_size) = struct.unpack(b'=IIIII', data[i:j])
-                i, j = j, j + prefix_size + 4 + 4 + 4 + 4 + 1
+                i, j = j, j + prefix_size + 4 + 4 + 4 + 4 + 1 + 1
                 (prefix, _, timeout_initialize,
                  timeout_async, timeout_sync, timeout_terminate,
-                 priority_default) = struct.unpack(
-                     '=%dscIIIIb' % (prefix_size - 1), data[i:j]
+                 priority_default, fatal_exceptions) = struct.unpack(
+                     '=%dscIIIIbB' % (prefix_size - 1), data[i:j]
                  )
                 if j != data_size:
                     assert external is False
@@ -599,7 +611,7 @@ class API(object):
                         process_count_max, process_count_min,
                         prefix.decode('utf-8'), timeout_initialize,
                         timeout_sync, timeout_async, timeout_terminate,
-                        priority_default)
+                        priority_default, fatal_exceptions)
             if command in (_MESSAGE_SEND_ASYNC, _MESSAGE_SEND_SYNC):
                 i, j = j, j + 4
                 name_size = struct.unpack(b'=I', data[i:j])[0]
@@ -619,11 +631,11 @@ class API(object):
                  )
                 i, j = j, j + request_size + 1 + 4 + 1 + 16 + 4
                 (request, _, request_timeout, priority, trans_id,
-                 pid_size) = struct.unpack(
+                 source_size) = struct.unpack(
                      '=%dscIb16sI' % request_size, data[i:j]
                  )
-                i, j = j, j + pid_size
-                pid = data[i:j]
+                i, j = j, j + source_size
+                source = data[i:j]
                 if j != data_size:
                     assert external is True
                     if not self.__handle_events(external, data, data_size, j):
@@ -634,7 +646,7 @@ class API(object):
                                 pattern.decode('utf-8'),
                                 request_info, request,
                                 request_timeout, priority, trans_id,
-                                binary_to_term(pid))
+                                binary_to_term(source))
                 if self.__terminate:
                     return False
             elif command in (_MESSAGE_RECV_ASYNC, _MESSAGE_RETURN_SYNC):
@@ -685,11 +697,12 @@ class API(object):
                     return False
                 assert False
             elif command == _MESSAGE_REINIT:
-                i, j = j, j + 4 + 4 + 4 + 1
+                i, j = j, j + 4 + 4 + 4 + 1 + 1
                 (self.__process_count,
                  self.__timeout_async, self.__timeout_sync,
-                 self.__priority_default) = struct.unpack(
-                     b'=IIIb', data[i:j]
+                 self.__priority_default,
+                 self.__fatal_exceptions) = struct.unpack(
+                     b'=IIIbB', data[i:j]
                  )
                 if j == data_size:
                     data = b''
