@@ -4,7 +4,7 @@
 #
 # MIT License
 #
-# Copyright (c) 2014-2018 Michael Truog <mjtruog at protonmail dot com>
+# Copyright (c) 2014-2022 Michael Truog <mjtruog at protonmail dot com>
 # Copyright (c) 2009-2013 Dmitry Vasiliev <dima@hlabs.org>
 #
 # Permission is hereby granted, free of charge, to any person obtaining a
@@ -26,12 +26,12 @@
 # DEALINGS IN THE SOFTWARE.
 #
 """
-Erlang Binary Term Format Encoding/Decoding Tests
+Erlang External Term Format Encoding/Decoding Tests
 """
 
 import sys
 import os
-sys.path.append(
+sys.path.insert(0,
     os.path.sep.join(
         os.path.dirname(
             os.path.abspath(__file__)
@@ -128,11 +128,16 @@ class DecodeTestCase(unittest.TestCase):
                          erlang.binary_to_term(b'\x83d\0\4test'))
         self.assertEqual(erlang.OtpErlangAtom(b'test'),
                          erlang.binary_to_term(b'\x83s\4test'))
+        self.assertEqual(erlang.OtpErlangAtom(u'name'),
+                         erlang.binary_to_term(b'\x83w\x04name'))
     def test_binary_to_term_predefined_atoms(self):
         self.assertEqual(True, erlang.binary_to_term(b'\x83s\4true'))
         self.assertEqual(False, erlang.binary_to_term(b'\x83s\5false'))
-        self.assertEqual(erlang.OtpErlangAtom(b'undefined'),
-                         erlang.binary_to_term(b'\x83d\0\11undefined'))
+        self.assertEqual(False, erlang.binary_to_term(b'\x83w\5false'))
+        self.assertEqual(None, erlang.binary_to_term(b'\x83s\11undefined'))
+        self.assertEqual(None, erlang.binary_to_term(b'\x83w\11undefined'))
+        self.assertEqual(None, erlang.binary_to_term(b'\x83d\0\11undefined'))
+        self.assertEqual(None, erlang.binary_to_term(b'\x83v\0\11undefined'))
     def test_binary_to_term_empty_list(self):
         self.assertEqual([], erlang.binary_to_term(b'\x83j'))
     def test_binary_to_term_string_list(self):
@@ -268,6 +273,64 @@ class DecodeTestCase(unittest.TestCase):
                          erlang.binary_to_term(b'\x83o\0\0\0\6\0\1\2\3\4\5\6'))
         self.assertEqual(-6618611909121,
                          erlang.binary_to_term(b'\x83o\0\0\0\6\1\1\2\3\4\5\6'))
+    def test_binary_to_term_pid(self):
+        pid_old_binary = (
+            b'\x83\x67\x64\x00\x0D\x6E\x6F\x6E\x6F\x64\x65\x40\x6E\x6F'
+            b'\x68\x6F\x73\x74\x00\x00\x00\x4E\x00\x00\x00\x00\x00'
+        )
+        pid_old = erlang.binary_to_term(pid_old_binary)
+        self.assertTrue(isinstance(pid_old, erlang.OtpErlangPid))
+        self.assertEqual(erlang.term_to_binary(pid_old),
+                         b'\x83gs\rnonode@nohost\x00\x00\x00N'
+                         b'\x00\x00\x00\x00\x00')
+        pid_new_binary = (
+            b'\x83\x58\x64\x00\x0D\x6E\x6F\x6E\x6F\x64\x65\x40\x6E\x6F\x68'
+            b'\x6F\x73\x74\x00\x00\x00\x4E\x00\x00\x00\x00\x00\x00\x00\x00'
+        )
+        pid_new = erlang.binary_to_term(pid_new_binary)
+        self.assertTrue(isinstance(pid_new, erlang.OtpErlangPid))
+        self.assertEqual(erlang.term_to_binary(pid_new),
+                         b'\x83Xs\rnonode@nohost\x00\x00\x00N'
+                         b'\x00\x00\x00\x00\x00\x00\x00\x00')
+    def test_binary_to_term_port(self):
+        port_old_binary = (
+            b'\x83\x66\x64\x00\x0D\x6E\x6F\x6E\x6F\x64\x65\x40\x6E\x6F\x68'
+            b'\x6F\x73\x74\x00\x00\x00\x06\x00'
+        )
+        port_old = erlang.binary_to_term(port_old_binary)
+        self.assertTrue(isinstance(port_old, erlang.OtpErlangPort))
+        self.assertEqual(erlang.term_to_binary(port_old),
+                         b'\x83fs\rnonode@nohost\x00\x00\x00\x06\x00')
+        port_new_binary = (
+            b'\x83\x59\x64\x00\x0D\x6E\x6F\x6E\x6F\x64\x65\x40\x6E\x6F\x68'
+            b'\x6F\x73\x74\x00\x00\x00\x06\x00\x00\x00\x00'
+        )
+        port_new = erlang.binary_to_term(port_new_binary)
+        self.assertTrue(isinstance(port_new, erlang.OtpErlangPort))
+        self.assertEqual(erlang.term_to_binary(port_new),
+                         b'\x83Ys\rnonode@nohost\x00\x00\x00\x06'
+                         b'\x00\x00\x00\x00')
+    def test_binary_to_term_ref(self):
+        ref_new_binary = (
+            b'\x83\x72\x00\x03\x64\x00\x0D\x6E\x6F\x6E\x6F\x64\x65\x40\x6E'
+            b'\x6F\x68\x6F\x73\x74\x00\x00\x03\xE8\x4E\xE7\x68\x00\x02\xA4'
+            b'\xC8\x53\x40'
+        )
+        ref_new = erlang.binary_to_term(ref_new_binary)
+        self.assertTrue(isinstance(ref_new, erlang.OtpErlangReference))
+        self.assertEqual(erlang.term_to_binary(ref_new),
+                         b'\x83r\x00\x03s\rnonode@nohost\x00\x00\x03\xe8'
+                         b'N\xe7h\x00\x02\xa4\xc8S@')
+        ref_newer_binary = (
+            b'\x83\x5A\x00\x03\x64\x00\x0D\x6E\x6F\x6E\x6F\x64\x65\x40\x6E'
+            b'\x6F\x68\x6F\x73\x74\x00\x00\x00\x00\x00\x01\xAC\x03\xC7\x00'
+            b'\x00\x04\xBB\xB2\xCA\xEE'
+        )
+        ref_newer = erlang.binary_to_term(ref_newer_binary)
+        self.assertTrue(isinstance(ref_newer, erlang.OtpErlangReference))
+        self.assertEqual(erlang.term_to_binary(ref_newer),
+                         b'\x83Z\x00\x03s\rnonode@nohost\x00\x00\x00\x00\x00'
+                         b'\x01\xac\x03\xc7\x00\x00\x04\xbb\xb2\xca\xee')
     def test_binary_to_term_compressed_term(self):
         self.assertRaises(erlang.ParseException,
                           erlang.binary_to_term, b'\x83P')
@@ -457,9 +520,10 @@ class EncodeTestCase(unittest.TestCase):
     def test_term_to_binary_string(self):
         self.assertEqual(b'\x83j', erlang.term_to_binary(''))
         self.assertEqual(b'\x83k\0\4test', erlang.term_to_binary('test'))
-    def test_term_to_binary_boolean(self):
-        self.assertEqual(b'\x83s\4true', erlang.term_to_binary(True))
-        self.assertEqual(b'\x83s\5false', erlang.term_to_binary(False))
+    def test_term_to_binary_predefined_atom(self):
+        self.assertEqual(b'\x83w\4true', erlang.term_to_binary(True))
+        self.assertEqual(b'\x83w\5false', erlang.term_to_binary(False))
+        self.assertEqual(b'\x83w\11undefined', erlang.term_to_binary(None))
     def test_term_to_binary_short_integer(self):
         self.assertEqual(b'\x83a\0', erlang.term_to_binary(0))
         self.assertEqual(b'\x83a\xff', erlang.term_to_binary(255))
