@@ -6,24 +6,16 @@ defmodule Nodefinder.Mixfile do
 
   def project do
     [app: :nodefinder,
-     version: "2.0.2",
+     version: "2.0.5",
      language: :erlang,
      erlc_options: [
        {:d, :erlang.list_to_atom('ERLANG_OTP_VERSION_' ++ :erlang.system_info(:otp_release))},
+       :deterministic,
        :debug_info,
-       :warnings_as_errors,
-       :strict_validation,
-       :warn_bif_clash,
-       :warn_deprecated_function,
-       :warn_export_all,
        :warn_export_vars,
-       :warn_exported_vars,
-       :warn_obsolete_guard,
-       :warn_shadow_vars,
        :warn_unused_import,
-       :warn_unused_function,
-       :warn_unused_record,
-       :warn_unused_vars],
+       #:warn_missing_spec,
+       :warnings_as_errors],
      description: description(),
      package: package(),
      deps: deps()]
