@@ -9,7 +9,7 @@
 %%%
 %%% MIT License
 %%%
-%%% Copyright (c) 2011-2018 Michael Truog <mjtruog at protonmail dot com>
+%%% Copyright (c) 2011-2022 Michael Truog <mjtruog at protonmail dot com>
 %%%
 %%% Permission is hereby granted, free of charge, to any person obtaining a
 %%% copy of this software and associated documentation files (the "Software"),
@@ -30,8 +30,8 @@
 %%% DEALINGS IN THE SOFTWARE.
 %%%
 %%% @author Michael Truog <mjtruog at protonmail dot com>
-%%% @copyright 2011-2018 Michael Truog
-%%% @version 1.7.4 {@date} {@time}
+%%% @copyright 2011-2022 Michael Truog
+%%% @version 2.0.5 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_core_i_configurator).
@@ -40,7 +40,7 @@
 -behaviour(gen_server).
 
 %% external interface
--export([start_link/1,
+-export([start_link/2,
          configure/0,
          acl_add/2,
          acl_remove/2,
@@ -49,6 +49,8 @@
          services_add/2,
          services_remove/2,
          services_restart/2,
+         services_suspend/2,
+         services_resume/2,
          services_update/2,
          services_search/3,
          service_ids/1,
@@ -65,10 +67,15 @@
          logging_formatters_set/2,
          logging_redirect_set/2,
          logging/1,
+         code_path_add/2,
+         code_path_remove/2,
+         code_path/1,
          code_status/3,
          service_start/2,
          service_stop/3,
          service_restart/2,
+         service_suspend/2,
+         service_resume/2,
          service_update/2,
          service_update_external/5,
          service_process_init_end/1,
@@ -94,9 +101,11 @@
      service_internal_release_invalid |
      service_internal_file_extension_invalid |
      service_internal_file_path_invalid |
-     service_internal_start_failed |
-     service_external_start_failed |
-     service_options_application_name_not_found, any()}.
+     service_options_application_name_not_found, any()} |
+    {service_internal_start_failed,
+     cloudi_core_i_spawn:error_reason_start_internal() | any()} |
+    {service_external_start_failed,
+     cloudi_core_i_spawn:error_reason_start_external() | any()}.
 -type error_reason_service_stop() ::
     {service_internal_module_not_found |
      service_internal_application_not_found |
@@ -106,29 +115,39 @@
 -type error_reason_service_restart() ::
     {service_internal_restart_failed |
      service_external_restart_failed, any()}.
+-type error_reason_service_suspend() ::
+    {service_internal_suspend_failed |
+     service_external_suspend_failed, any()}.
+-type error_reason_service_resume() ::
+    {service_internal_resume_failed |
+     service_external_resume_failed, any()}.
 -type error_reason_service_update() ::
     {service_internal_update_failed |
      service_external_update_failed, any()}.
 -type error_reason_services_search() ::
     service_scope_invalid.
+-type error_reason_nodes_set() ::
+    {set_invalid, any()}.
 -type error_reason_code_status() ::
-    {file, {file:filename(), file:posix() | badarg}}.
+    {file, {cloudi_service_api:file_path(), file:posix() | badarg}}.
 -export_type([error_reason_service_start/0,
               error_reason_service_stop/0,
               error_reason_service_restart/0,
+              error_reason_service_suspend/0,
+              error_reason_service_resume/0,
               error_reason_service_update/0,
               error_reason_services_search/0,
+              error_reason_nodes_set/0,
               error_reason_code_status/0]).
 
 -record(state,
     {
+        node_name :: nonempty_string(),
         time_start :: cloudi_timestamp:native_monotonic(),
+        terminate_timeout :: pos_integer(),
         configuration :: #config{}
     }).
 
--define(CATCH_EXIT(F),
-        try F catch exit:{Reason, _} -> {error, Reason} end).
-
 -ifdef(CLOUDI_CORE_STANDALONE).
 -dialyzer({no_match,
            [service_update_external/6]}).
@@ -138,8 +157,9 @@
 %%% External interface functions
 %%%------------------------------------------------------------------------
 
-start_link(#config{} = Config) ->
-    gen_server:start_link({local, ?MODULE}, ?MODULE, [Config], []).
+start_link(TerminateTimeout, #config{} = Config) ->
+    gen_server:start_link({local, ?MODULE}, ?MODULE,
+                          [TerminateTimeout, Config], []).
 
 configure() ->
     gen_server:call(?MODULE, configure, infinity).
@@ -172,6 +192,14 @@ services_restart(L, Timeout) ->
     ?CATCH_EXIT(gen_server:call(?MODULE,
                                 {services_restart, L}, Timeout)).
 
+services_suspend(L, Timeout) ->
+    ?CATCH_EXIT(gen_server:call(?MODULE,
+                                {services_suspend, L}, Timeout)).
+
+services_resume(L, Timeout) ->
+    ?CATCH_EXIT(gen_server:call(?MODULE,
+                                {services_resume, L}, Timeout)).
+
 services_update(L, Timeout) ->
     ?CATCH_EXIT(gen_server:call(?MODULE,
                                 {services_update, L}, Timeout)).
@@ -232,6 +260,17 @@ logging_redirect_set(L, Timeout) ->
 logging(Timeout) ->
     ?CATCH_EXIT(gen_server:call(?MODULE, logging, Timeout)).
 
+code_path_add(Path, Timeout) ->
+    ?CATCH_EXIT(gen_server:call(?MODULE,
+                                {code_path_add, Path}, Timeout)).
+
+code_path_remove(Path, Timeout) ->
+    ?CATCH_EXIT(gen_server:call(?MODULE,
+                                {code_path_remove, Path}, Timeout)).
+
+code_path(Timeout) ->
+    ?CATCH_EXIT(gen_server:call(?MODULE, code_path, Timeout)).
+
 code_status(TimeNative, TimeOffset, Timeout) ->
     ?CATCH_EXIT(gen_server:call(?MODULE,
                                 {code_status,
@@ -242,6 +281,7 @@ code_status(TimeNative, TimeOffset, Timeout) ->
                     Timeout :: pos_integer() | infinity) ->
     {ok, #config_service_internal{} |
          #config_service_external{}} |
+    stopped |
     {error, error_reason_service_start()}.
 
 service_start(#config_service_internal{
@@ -265,9 +305,9 @@ service_start(#config_service_internal{
                 Reload =:= false ->
                     ok
             end,
-            NewCountProcess = cloudi_concurrency:count(CountProcess),
+            CountProcessNew = cloudi_concurrency:count(CountProcess),
             service_start_internal(FoundService, GroupLeader,
-                                   NewCountProcess, Timeout);
+                                   CountProcessNew, Timeout);
         {error, _} = Error ->
             Error
     end;
@@ -275,10 +315,10 @@ service_start(#config_service_internal{
 service_start(#config_service_external{count_process = CountProcess,
                                        count_thread = CountThread} = Service,
               Timeout) ->
-    NewCountProcess = cloudi_concurrency:count(CountProcess),
-    NewCountThread = cloudi_concurrency:count(CountThread),
-    service_start_external(Service, NewCountThread,
-                           NewCountProcess, Timeout).
+    CountProcessNew = cloudi_concurrency:count(CountProcess),
+    CountThreadNew = cloudi_concurrency:count(CountThread),
+    service_start_external(Service, CountThreadNew,
+                           CountProcessNew, Timeout).
 
 -spec service_stop(#config_service_internal{} |
                    #config_service_external{},
@@ -306,10 +346,34 @@ service_restart(#config_service_internal{} = Service, Timeout) ->
 service_restart(#config_service_external{} = Service, Timeout) ->
     service_restart_external(Service, timeout_decr(Timeout)).
 
+-spec service_suspend(#config_service_internal{} |
+                      #config_service_external{},
+                      Timeout :: pos_integer() | infinity) ->
+    ok |
+    {error, error_reason_service_suspend()}.
+
+service_suspend(#config_service_internal{} = Service, Timeout) ->
+    service_suspend_internal(Service, timeout_decr(Timeout));
+
+service_suspend(#config_service_external{} = Service, Timeout) ->
+    service_suspend_external(Service, timeout_decr(Timeout)).
+
+-spec service_resume(#config_service_internal{} |
+                     #config_service_external{},
+                     Timeout :: pos_integer() | infinity) ->
+    ok |
+    {error, error_reason_service_resume()}.
+
+service_resume(#config_service_internal{} = Service, Timeout) ->
+    service_resume_internal(Service, timeout_decr(Timeout));
+
+service_resume(#config_service_external{} = Service, Timeout) ->
+    service_resume_external(Service, timeout_decr(Timeout)).
+
 -spec service_update(#config_service_update{},
                      Timeout :: pos_integer() | infinity) ->
     {ok, nonempty_list(cloudi_service_api:service_id())} |
-    {error, nonempty_list(cloudi_service_api:service_id()),
+    {aborted | error, nonempty_list(cloudi_service_api:service_id()),
      error_reason_service_update()}.
 
 service_update(#config_service_update{
@@ -324,6 +388,8 @@ service_update(#config_service_update{
     case cloudi_core_i_services_monitor:update(UpdatePlan, Timeout) of
         ok ->
             {ok, ServiceIdList};
+        {aborted, Reason} ->
+            {aborted, ServiceIdList, {ErrorReasonType, Reason}};
         {error, Reason} ->
             {error, ServiceIdList, {ErrorReasonType, Reason}}
     end.
@@ -349,8 +415,12 @@ service_terminated(ID)
 %%% Callback functions from gen_server
 %%%------------------------------------------------------------------------
 
-init([Config]) ->
-    {ok, #state{time_start = cloudi_timestamp:native_monotonic(),
+init([TerminateTimeout, Config]) ->
+    TimeStart = cloudi_timestamp:native_monotonic(),
+    NodeName = cloudi_core_i_nodes:node_name(node()),
+    {ok, #state{node_name = NodeName,
+                time_start = TimeStart,
+                terminate_timeout = TerminateTimeout,
                 configuration = Config}}.
 
 handle_call(configure, _, State) ->
@@ -362,8 +432,8 @@ handle_call(configure, _, State) ->
 handle_call({acl_add, L}, _,
             #state{configuration = Config} = State) ->
     case cloudi_core_i_configuration:acl_add(L, Config) of
-        {ok, NewConfig} ->
-            {reply, ok, State#state{configuration = NewConfig}};
+        {ok, ConfigNew} ->
+            {reply, ok, State#state{configuration = ConfigNew}};
         {error, _} = Error ->
             {reply, Error, State}
     end;
@@ -371,8 +441,8 @@ handle_call({acl_add, L}, _,
 handle_call({acl_remove, L}, _,
             #state{configuration = Config} = State) ->
     case cloudi_core_i_configuration:acl_remove(L, Config) of
-        {ok, NewConfig} ->
-            {reply, ok, State#state{configuration = NewConfig}};
+        {ok, ConfigNew} ->
+            {reply, ok, State#state{configuration = ConfigNew}};
         {error, _} = Error ->
             {reply, Error, State}
     end;
@@ -394,17 +464,17 @@ handle_call({service_subscriptions, ServiceId, Timeout}, _, State) ->
 handle_call({services_add, L}, _,
             #state{configuration = Config} = State) ->
     case cloudi_core_i_configuration:services_add(L, Config, infinity) of
-        {ok, IDs, NewConfig} ->
-            {reply, {ok, IDs}, State#state{configuration = NewConfig}};
-        {error, _} = Error ->
-            {reply, Error, State}
+        {ok, IDs, ConfigNew} ->
+            {reply, {ok, IDs}, State#state{configuration = ConfigNew}};
+        {error, Reason, ConfigNew} ->
+            {reply, {error, Reason}, State#state{configuration = ConfigNew}}
     end;
 
 handle_call({services_remove, L}, _,
             #state{configuration = Config} = State) ->
     case cloudi_core_i_configuration:services_remove(L, Config, infinity) of
-        {ok, NewConfig} ->
-            {reply, ok, State#state{configuration = NewConfig}};
+        {ok, ConfigNew} ->
+            {reply, ok, State#state{configuration = ConfigNew}};
         {error, _} = Error ->
             {reply, Error, State}
     end;
@@ -412,8 +482,26 @@ handle_call({services_remove, L}, _,
 handle_call({services_restart, L}, _,
             #state{configuration = Config} = State) ->
     case cloudi_core_i_configuration:services_restart(L, Config, infinity) of
-        {ok, NewConfig} ->
-            {reply, ok, State#state{configuration = NewConfig}};
+        ok ->
+            {reply, ok, State};
+        {error, _} = Error ->
+            {reply, Error, State}
+    end;
+
+handle_call({services_suspend, L}, _,
+            #state{configuration = Config} = State) ->
+    case cloudi_core_i_configuration:services_suspend(L, Config, infinity) of
+        ok ->
+            {reply, ok, State};
+        {error, _} = Error ->
+            {reply, Error, State}
+    end;
+
+handle_call({services_resume, L}, _,
+            #state{configuration = Config} = State) ->
+    case cloudi_core_i_configuration:services_resume(L, Config, infinity) of
+        ok ->
+            {reply, ok, State};
         {error, _} = Error ->
             {reply, Error, State}
     end;
@@ -421,8 +509,8 @@ handle_call({services_restart, L}, _,
 handle_call({services_update, L}, _,
             #state{configuration = Config} = State) ->
     case cloudi_core_i_configuration:services_update(L, Config, infinity) of
-        {ok, Result, NewConfig} ->
-            {reply, Result, State#state{configuration = NewConfig}};
+        {ok, Result, ConfigNew} ->
+            {reply, Result, State#state{configuration = ConfigNew}};
         {error, _} = Error ->
             {reply, Error, State}
     end;
@@ -435,7 +523,7 @@ handle_call({services_search, Scope, ServiceName, Timeout}, _,
                 {ok, []} ->
                     {reply, {ok, []}, State};
                 {ok, L} ->
-                    {reply, 
+                    {reply,
                      {ok,
                       cloudi_core_i_configuration:services_search(L, Config)},
                      State};
@@ -460,7 +548,7 @@ handle_call(services, _,
     {reply, {ok, cloudi_core_i_configuration:services(Config)}, State};
 
 handle_call({nodes_set, _, _} = Request, _, State) ->
-    nodes_call(Request, State);
+    nodes_set_call(Request, State);
 
 handle_call(nodes_get, _,
             #state{configuration = Config} = State) ->
@@ -475,10 +563,10 @@ handle_call({nodes_remove, _, _} = Request, _, State) ->
 handle_call({logging_set, L}, _,
             #state{configuration = Config} = State) ->
     case cloudi_core_i_configuration:logging_set(L, Config) of
-        {ok, #config{logging = LoggingConfig} = NewConfig} ->
+        {ok, #config{logging = LoggingConfig} = ConfigNew} ->
             case cloudi_core_i_logger:set(LoggingConfig) of
                 ok ->
-                    {reply, ok, State#state{configuration = NewConfig}};
+                    {reply, ok, State#state{configuration = ConfigNew}};
                 {error, _} = Error ->
                     {reply, Error, State}
             end;
@@ -491,10 +579,10 @@ handle_call({logging_file_set, FilePath}, _,
     #config{logging = LoggingConfig} = Config,
     case cloudi_core_i_logger:file_set(FilePath) of
         ok ->
-            NewConfig = Config#config{
+            ConfigNew = Config#config{
                             logging = LoggingConfig#config_logging{
                                 file = FilePath}},
-            {reply, ok, State#state{configuration = NewConfig}};
+            {reply, ok, State#state{configuration = ConfigNew}};
         {error, _} = Error ->
             {reply, Error, State}
     end;
@@ -503,27 +591,27 @@ handle_call({logging_stdout_set, Stdout}, _,
             #state{configuration = Config} = State) ->
     #config{logging = LoggingConfig} = Config,
     ok = cloudi_core_i_logger:stdout_set(Stdout),
-    NewConfig = Config#config{
+    ConfigNew = Config#config{
                     logging = LoggingConfig#config_logging{
                         stdout = Stdout}},
-    {reply, ok, State#state{configuration = NewConfig}};
+    {reply, ok, State#state{configuration = ConfigNew}};
 
 handle_call({logging_level_set, Level}, _,
             #state{configuration = Config} = State) ->
     #config{logging = LoggingConfig} = Config,
     ok = cloudi_core_i_logger:level_set(Level),
-    NewConfig = Config#config{
+    ConfigNew = Config#config{
                     logging = LoggingConfig#config_logging{
                         level = Level}},
-    {reply, ok, State#state{configuration = NewConfig}};
+    {reply, ok, State#state{configuration = ConfigNew}};
 
 handle_call({logging_syslog_set, L}, _,
             #state{configuration = Config} = State) ->
     case cloudi_core_i_configuration:logging_syslog_set(L, Config) of
         {ok, #config{logging = #config_logging{
-                         syslog = SyslogConfig}} = NewConfig} ->
+                         syslog = SyslogConfig}} = ConfigNew} ->
             ok = cloudi_core_i_logger:syslog_set(SyslogConfig),
-            {reply, ok, State#state{configuration = NewConfig}};
+            {reply, ok, State#state{configuration = ConfigNew}};
         {error, _} = Error ->
             {reply, Error, State}
     end;
@@ -532,9 +620,9 @@ handle_call({logging_formatters_set, L}, _,
             #state{configuration = Config} = State) ->
     case cloudi_core_i_configuration:logging_formatters_set(L, Config) of
         {ok, #config{logging = #config_logging{
-                         formatters = FormattersConfig}} = NewConfig} ->
+                         formatters = FormattersConfig}} = ConfigNew} ->
             ok = cloudi_core_i_logger:formatters_set(FormattersConfig),
-            {reply, ok, State#state{configuration = NewConfig}};
+            {reply, ok, State#state{configuration = ConfigNew}};
         {error, _} = Error ->
             {reply, Error, State}
     end;
@@ -543,15 +631,37 @@ handle_call({logging_redirect_set, Node}, _,
             #state{configuration = Config} = State) ->
     #config{logging = LoggingConfig} = Config,
     ok = cloudi_core_i_logger:redirect_set(Node),
-    NewConfig = Config#config{
+    ConfigNew = Config#config{
                     logging = LoggingConfig#config_logging{
                         redirect = Node}},
-    {reply, ok, State#state{configuration = NewConfig}};
+    {reply, ok, State#state{configuration = ConfigNew}};
 
 handle_call(logging, _,
             #state{configuration = Config} = State) ->
     {reply, {ok, cloudi_core_i_configuration:logging(Config)}, State};
 
+handle_call({code_path_add, Path}, _,
+            #state{configuration = Config} = State) ->
+    case cloudi_core_i_configuration:code_path_add(Path, Config) of
+        {ok, ConfigNew} ->
+            {reply, ok, State#state{configuration = ConfigNew}};
+        {error, _} = Error ->
+            {reply, Error, State}
+    end;
+
+handle_call({code_path_remove, Path}, _,
+            #state{configuration = Config} = State) ->
+    case cloudi_core_i_configuration:code_path_remove(Path, Config) of
+        {ok, ConfigNew} ->
+            {reply, ok, State#state{configuration = ConfigNew}};
+        {error, _} = Error ->
+            {reply, Error, State}
+    end;
+
+handle_call(code_path, _,
+            #state{configuration = Config} = State) ->
+    {reply, {ok, cloudi_core_i_configuration:code_path(Config)}, State};
+
 handle_call({code_status, TimeNative, TimeOffset}, _,
             #state{time_start = TimeStart,
                    configuration = Config} = State) ->
@@ -575,16 +685,9 @@ handle_call(Request, _, State) ->
 handle_cast({service_terminated, ID},
             #state{configuration = Config} = State) ->
     #config{services = Services} = Config,
-    NewServices = lists:filter(fun(Service) ->
-        case Service of
-            #config_service_internal{uuid = InternalID} ->
-                InternalID /= ID;
-            #config_service_external{uuid = ExternalID} ->
-                ExternalID /= ID
-        end
-    end, Services),
-    NewConfig = Config#config{services = NewServices},
-    {noreply, State#state{configuration = NewConfig}};
+    ServicesNew = service_terminated_remove(Services, ID),
+    ConfigNew = Config#config{services = ServicesNew},
+    {noreply, State#state{configuration = ConfigNew}};
 
 handle_cast(Request, State) ->
     {stop, cloudi_string:format("Unknown cast \"~w\"", [Request]), State}.
@@ -592,8 +695,8 @@ handle_cast(Request, State) ->
 handle_info(configure,
             #state{configuration = Config} = State) ->
     case configure(Config, infinity) of
-        {ok, NewConfig} ->
-            {noreply, State#state{configuration = NewConfig}};
+        {ok, ConfigNew} ->
+            {noreply, State#state{configuration = ConfigNew}};
         {error, _} = Error ->
             % cloudi_core application startup failed due to a problem
             % with the cloudi.conf file
@@ -603,6 +706,8 @@ handle_info(configure,
 handle_info({cloudi_service_init_end, _}, State) ->
     % a service process has initialized due to count_process_dynamic
     % (nothing to do, handled by cloudi_core_i_services_monitor)
+    % or a service process shutdown very quickly which was seen by
+    % the process monitor before this message was able to be received
     {noreply, State};
 
 handle_info({ReplyRef, _}, State) when is_reference(ReplyRef) ->
@@ -613,7 +718,14 @@ handle_info({ReplyRef, _}, State) when is_reference(ReplyRef) ->
 handle_info(Request, State) ->
     {stop, cloudi_string:format("Unknown info \"~w\"", [Request]), State}.
 
-terminate(_, _) ->
+terminate(_Reason,
+          #state{terminate_timeout = Timeout,
+                 configuration = #config{code = ConfigCode}}) ->
+    #config_code{modules = Modules,
+                 applications = Applications} = ConfigCode,
+    _ = cloudi_x_reltool_util:modules_purged(Modules, Timeout),
+    _ = cloudi_x_reltool_util:applications_remove(Applications, Timeout,
+                                                  [cloudi_core]),
     ok.
 
 code_change(_, State, _) ->
@@ -625,8 +737,8 @@ code_change(_, State, _) ->
 
 configure(#config{services = Services} = Config, Timeout) ->
     case configure_service(Services, Timeout) of
-        {ok, NewServices} ->
-            {ok, Config#config{services = NewServices}};
+        {ok, ServicesNew} ->
+            {ok, Config#config{services = ServicesNew}};
         {error, _} = Error ->
             Error
     end.
@@ -635,11 +747,13 @@ configure_service([], Configured, _) ->
     {ok, lists:reverse(Configured)};
 configure_service([Service | Services], Configured, Timeout) ->
     case service_start(Service, Timeout) of
-        {ok, NewService} ->
-            configure_service(Services, [NewService | Configured], Timeout);
+        {ok, ServiceNew} ->
+            configure_service(Services, [ServiceNew | Configured], Timeout);
+        stopped ->
+            configure_service(Services, Configured, Timeout);
         {error, Reason} = Error ->
             % wait for logging statements to be logged before crashing
-            ?LOG_FATAL_SYNC("configure failed: ~p~n~p",
+            ?LOG_FATAL_SYNC("configure failed: ~tp~n~tp",
                             [Reason, service_format(Service)]),
             Error
     end.
@@ -942,9 +1056,24 @@ service_stop_remove_internal(#config_service_internal{
             {error, {service_internal_application_not_found, Reason}}
     end.
 
-service_start_wait_pid([], Error) ->
-    Error;
-service_start_wait_pid([{MonitorRef, Pid} | M], Error) ->
+service_start_wait_pid_death(undefined, shutdown) ->
+    {stopped, shutdown};
+service_start_wait_pid_death(undefined, {shutdown, _} = Shutdown) ->
+    {stopped, Shutdown};
+service_start_wait_pid_death(undefined, Info) ->
+    {error, Info};
+service_start_wait_pid_death({stopped, _} = Stopped, shutdown) ->
+    Stopped;
+service_start_wait_pid_death({stopped, _} = Stopped, {shutdown, _}) ->
+    Stopped;
+service_start_wait_pid_death({stopped, _}, Info) ->
+    {error, Info};
+service_start_wait_pid_death({error, _} = Error, _) ->
+    Error.
+
+service_start_wait_pid([], Death) ->
+    Death;
+service_start_wait_pid([{MonitorRef, Pid} | M], Death) ->
     receive
         {cloudi_service_init_end, Pid} ->
             erlang:demonitor(MonitorRef, [flush]),
@@ -956,24 +1085,27 @@ service_start_wait_pid([{MonitorRef, Pid} | M], Error) ->
             %   it has already completed executing CloudI API functions called
             %   before the poll function and any aspects_init_after functions
             %   (so the external service is now executing the poll function)
-            service_start_wait_pid(M, Error);
+            service_start_wait_pid(M, Death);
         {'DOWN', MonitorRef, process, Pid, Info} ->
-            NewError = if
-                Error =:= undefined ->
-                    {error, {service_internal_start_failed, Info}};
-                true ->
-                    Error
-            end,
-            service_start_wait_pid(M, NewError)
+            service_start_wait_pid(M, service_start_wait_pid_death(Death, Info))
     end.
 
 service_start_wait_pids([], undefined, Service) ->
     {ok, Service};
-service_start_wait_pids([], Error, _) ->
-    Error;
-service_start_wait_pids([M | MonitorPids], Error, Service) ->
+service_start_wait_pids([], {stopped, _}, _) ->
+    % shutdown | {shutdown, _} is logged in cloudi_core_i_services_monitor and
+    % is not helpful as a return value because it will be ignored
+    stopped;
+service_start_wait_pids([], {error, Reason}, Service) ->
+    case Service of
+        #config_service_internal{} ->
+            {error, {service_internal_start_failed, Reason}};
+        #config_service_external{} ->
+            {error, {service_external_start_failed, Reason}}
+    end;
+service_start_wait_pids([M | MonitorPids], Death, Service) ->
     service_start_wait_pids(MonitorPids,
-                            service_start_wait_pid(M, Error), Service).
+                            service_start_wait_pid(M, Death), Service).
 
 service_start_wait(Pids, Service) ->
     % create the monitors on all the service's receiver pids
@@ -984,8 +1116,6 @@ service_start_wait(Pids, Service) ->
     % and pass the error reason as a return value of the configure/2 function
     % or the cloudi_service_api:services_add/2 function when a service
     % starts for the first time
-    % (restarts require cloudi_core_i_services_monitor to
-    %  call cloudi_core_i_services_monitor:process_init_begin/1)
     ok = cloudi_core_i_services_monitor:process_init_begin(Pids),
     service_start_wait_pids(MonitorPids, undefined, Service).
 
@@ -1004,6 +1134,7 @@ service_start_internal(IndexProcess, Pids,
                            dest_list_deny = DestListDeny,
                            dest_list_allow = DestListAllow,
                            options = #config_service_options{
+                               restart_all = RestartAll,
                                restart_delay = RestartDelay,
                                scope = Scope} = Options,
                            max_r = MaxR,
@@ -1018,7 +1149,8 @@ service_start_internal(IndexProcess, Pids,
                   DestRefresh, DestListDeny,
                   DestListAllow, Options, ID],
                  IndexProcess, CountProcess, 1, Scope,
-                 TimeoutTerm, RestartDelay, MaxR, MaxT, ID, Timeout) of
+                 TimeoutTerm, RestartAll, RestartDelay,
+                 MaxR, MaxT, ID, Timeout) of
         {ok, P} ->
             service_format_log(Service, P),
             service_start_internal(IndexProcess + 1, [P | Pids], Service,
@@ -1049,6 +1181,7 @@ service_start_external(IndexProcess, Pids,
                            dest_list_deny = DestListDeny,
                            dest_list_allow = DestListAllow,
                            options = #config_service_options{
+                               restart_all = RestartAll,
                                restart_delay = RestartDelay,
                                scope = Scope} = Options,
                            max_r = MaxR,
@@ -1064,7 +1197,8 @@ service_start_external(IndexProcess, Pids,
                   DestRefresh, DestListDeny,
                   DestListAllow, Options, ID],
                  IndexProcess, CountProcess, CountThread, Scope,
-                 TimeoutTerm, RestartDelay, MaxR, MaxT, ID, Timeout) of
+                 TimeoutTerm, RestartAll, RestartDelay,
+                 MaxR, MaxT, ID, Timeout) of
         {ok, P} ->
             service_format_log(Service, P),
             service_start_external(IndexProcess + 1, [P | Pids], Service,
@@ -1084,7 +1218,6 @@ service_stop_internal(#config_service_internal{
                           uuid = ID} = Service, Remove, Timeout) ->
     case cloudi_core_i_services_monitor:shutdown(ID, Timeout) of
         {ok, Pids} ->
-            shutdown_wait(Pids),
             if
                 Reload =:= true ->
                     ok = cloudi_core_i_services_internal_reload:
@@ -1124,7 +1257,6 @@ service_stop_external(#config_service_external{
                           uuid = ID}, Timeout) ->
     case cloudi_core_i_services_monitor:shutdown(ID, Timeout) of
         {ok, Pids} ->
-            shutdown_wait(Pids),
             ?LOG_INFO_SYNC("Service pids ~p stopped~n ~p",
                            [Pids, service_id(ID)]),
             ok;
@@ -1150,6 +1282,42 @@ service_restart_external(#config_service_external{
             {error, {service_external_restart_failed, Reason}}
     end.
 
+service_suspend_internal(#config_service_internal{
+                             uuid = ID}, Timeout) ->
+    case cloudi_core_i_services_monitor:suspend(ID, Timeout) of
+        ok ->
+            ok;
+        {error, Reason} ->
+            {error, {service_internal_suspend_failed, Reason}}
+    end.
+
+service_suspend_external(#config_service_external{
+                             uuid = ID}, Timeout) ->
+    case cloudi_core_i_services_monitor:suspend(ID, Timeout) of
+        ok ->
+            ok;
+        {error, Reason} ->
+            {error, {service_external_suspend_failed, Reason}}
+    end.
+
+service_resume_internal(#config_service_internal{
+                            uuid = ID}, Timeout) ->
+    case cloudi_core_i_services_monitor:resume(ID, Timeout) of
+        ok ->
+            ok;
+        {error, Reason} ->
+            {error, {service_internal_resume_failed, Reason}}
+    end.
+
+service_resume_external(#config_service_external{
+                            uuid = ID}, Timeout) ->
+    case cloudi_core_i_services_monitor:resume(ID, Timeout) of
+        ok ->
+            ok;
+        {error, Reason} ->
+            {error, {service_external_resume_failed, Reason}}
+    end.
+
 service_update_external(CountProcess, [], [], _, _, CountProcess) ->
     ok;
 service_update_external(IndexProcess, Pids, Ports, Arguments,
@@ -1179,7 +1347,7 @@ nodes_call_remote_result({Replies, BadNodes}) ->
         true ->
             {error, {remote, Errors}}
     end.
-    
+
 nodes_call_remote_result_replies([], Output) ->
     lists:reverse(Output);
 nodes_call_remote_result_replies([{_, ok} | Replies], Output) ->
@@ -1187,30 +1355,33 @@ nodes_call_remote_result_replies([{_, ok} | Replies], Output) ->
 nodes_call_remote_result_replies([{_, _} = Error | Replies], Output) ->
     nodes_call_remote_result_replies(Replies, [Error | Output]).
 
-nodes_call_remote({_, _, remote}, _) ->
+nodes_call_remote({_, _, CallType}, _, _)
+    when CallType =:= remote; CallType =:= local_only ->
     ok;
-nodes_call_remote({F, L, local}, Connect) ->
-    Nodes = if
-        Connect =:= visible ->
-            nodes();
-        Connect =:= hidden ->
-            nodes(connected)
-    end,
-    nodes_call_remote_result(global:trans({{?MODULE, L}, self()}, fun() ->
-        gen_server:multi_call(Nodes, ?MODULE,
-                              {F, L, remote}, infinity)
-    end)).
+nodes_call_remote({F, L, local}, Listen, NodeName) ->
+    case cloudi_x_cpg:listen_nodes(Listen, NodeName) of
+        [] ->
+            ok;
+        [_ | _] = Nodes ->
+            GlobalId = {{?MODULE, L}, self()},
+            nodes_call_remote_result(global:trans(GlobalId, fun() ->
+                gen_server:multi_call(Nodes, ?MODULE,
+                                      {F, L, remote}, infinity)
+            end))
+    end.
 
-nodes_call({F, L, _} = Request,
-           #state{configuration = Config} = State) ->
+nodes_call({F, L, CallType} = Request,
+           #state{node_name = NodeName,
+                  configuration = Config} = State) ->
     case cloudi_core_i_configuration:F(L, Config) of
-        {ok, Config} ->
+        {ok, Config}
+            when CallType =:= remote; CallType =:= local_only ->
             {reply, ok, State};
-        {ok, #config{nodes = #config_nodes{connect = Connect}} = NewConfig} ->
-            Result = nodes_call_remote(Request, Connect),
-            case cloudi_core_i_nodes:reconfigure(NewConfig, infinity) of
+        {ok, #config{nodes = #config_nodes{listen = Listen}} = ConfigNew} ->
+            Result = nodes_call_remote(Request, Listen, NodeName),
+            case cloudi_core_i_nodes:reconfigure(ConfigNew, infinity) of
                 ok ->
-                    {reply, Result, State#state{configuration = NewConfig}};
+                    {reply, Result, State#state{configuration = ConfigNew}};
                 {error, _} = Error ->
                     {reply, Error, State}
             end;
@@ -1218,15 +1389,33 @@ nodes_call({F, L, _} = Request,
             {reply, Error, State}
     end.
 
-shutdown_wait_monitor([]) ->
-    ok;
-shutdown_wait_monitor(Monitors) ->
-    receive
-        {'DOWN', Monitor, process, _, _} ->
-            shutdown_wait_monitor(lists:delete(Monitor, Monitors))
+nodes_set_call({nodes_set, _, remote} = Request, State) ->
+    nodes_call(Request, State);
+nodes_set_call({nodes_set, L0, local} = Request, State) ->
+    case lists:keytake(set, 1, L0) of
+        false ->
+            nodes_call(Request, State);
+        {value, {set, Value}, LN} ->
+            if
+                Value =:= all ->
+                    nodes_call({nodes_set, LN, local}, State);
+                Value =:= local ->
+                    nodes_call({nodes_set, LN, local_only}, State);
+                true ->
+                    {error, {set_invalid, Value}}
+            end
     end.
-shutdown_wait(Pids) ->
-    shutdown_wait_monitor([erlang:monitor(process, Pid) || Pid <- Pids]).
+
+service_terminated_remove([] = Services, _) ->
+    Services;
+service_terminated_remove([#config_service_internal{uuid = ID} | Services],
+                          ID) ->
+    Services;
+service_terminated_remove([#config_service_external{uuid = ID} | Services],
+                          ID) ->
+    Services;
+service_terminated_remove([Service | Services], ID) ->
+    [Service | service_terminated_remove(Services, ID)].
 
 timeout_decr(infinity) ->
     infinity;
@@ -1235,7 +1424,7 @@ timeout_decr(Timeout)
     Timeout - ?TIMEOUT_DELTA.
 
 service_format_log(Service, Pids) ->
-    ?LOG_INFO("~p ->~n    ~p", [service_format(Service), Pids]).
+    ?LOG_INFO("~tp ->~n    ~p", [service_format(Service), Pids]).
 
 service_format(Service) ->
     {ID, ServiceConfig} = cloudi_core_i_configuration:service_format(Service),
