@@ -9,7 +9,7 @@
 %%%
 %%% MIT License
 %%%
-%%% Copyright (c) 2011-2021 Michael Truog <mjtruog at protonmail dot com>
+%%% Copyright (c) 2011-2022 Michael Truog <mjtruog at protonmail dot com>
 %%%
 %%% Permission is hereby granted, free of charge, to any person obtaining a
 %%% copy of this software and associated documentation files (the "Software"),
@@ -30,8 +30,8 @@
 %%% DEALINGS IN THE SOFTWARE.
 %%%
 %%% @author Michael Truog <mjtruog at protonmail dot com>
-%%% @copyright 2011-2021 Michael Truog
-%%% @version 2.0.3 {@date} {@time}
+%%% @copyright 2011-2022 Michael Truog
+%%% @version 2.0.5 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_service_api).
@@ -92,83 +92,248 @@
     ?PRIORITY_HIGH..?PRIORITY_LOW.
 -export_type([priority/0]).
 
--type dest_refresh_delay_milliseconds() ::
-    ?DEST_REFRESH_DELAY_MIN..?DEST_REFRESH_DELAY_MAX.
--export_type([dest_refresh_delay_milliseconds/0]).
-
 -type timeout_initialize_value_milliseconds() ::
     ?TIMEOUT_INITIALIZE_MIN..?TIMEOUT_INITIALIZE_MAX.
 -type timeout_initialize_milliseconds() ::
     limit_min | limit_max | timeout_initialize_value_milliseconds().
+-type timeout_initialize_period() ::
+    {1..(?TIMEOUT_INITIALIZE_MAX div ?MILLISECONDS_IN_SECOND),
+     seconds | second} |
+    {1..(?TIMEOUT_INITIALIZE_MAX div ?MILLISECONDS_IN_MINUTE),
+     minutes | minute} |
+    {1..(?TIMEOUT_INITIALIZE_MAX div ?MILLISECONDS_IN_HOUR),
+     hours | hour} |
+    {1..(?TIMEOUT_INITIALIZE_MAX div ?MILLISECONDS_IN_DAY),
+     days | day} |
+    timeout_initialize_milliseconds().
 -export_type([timeout_initialize_value_milliseconds/0,
-              timeout_initialize_milliseconds/0]).
+              timeout_initialize_milliseconds/0,
+              timeout_initialize_period/0]).
 
 -type timeout_send_async_value_milliseconds() ::
     ?TIMEOUT_SEND_ASYNC_MIN..?TIMEOUT_SEND_ASYNC_MAX.
 -type timeout_send_async_milliseconds() ::
     limit_min | limit_max | timeout_send_async_value_milliseconds().
+-type timeout_send_async_period() ::
+    {1..(?TIMEOUT_SEND_ASYNC_MAX div ?MILLISECONDS_IN_SECOND),
+     seconds | second} |
+    {1..(?TIMEOUT_SEND_ASYNC_MAX div ?MILLISECONDS_IN_MINUTE),
+     minutes | minute} |
+    {1..(?TIMEOUT_SEND_ASYNC_MAX div ?MILLISECONDS_IN_HOUR),
+     hours | hour} |
+    {1..(?TIMEOUT_SEND_ASYNC_MAX div ?MILLISECONDS_IN_DAY),
+     days | day} |
+    timeout_send_async_milliseconds().
 -export_type([timeout_send_async_value_milliseconds/0,
-              timeout_send_async_milliseconds/0]).
+              timeout_send_async_milliseconds/0,
+              timeout_send_async_period/0]).
 
 -type timeout_send_sync_value_milliseconds() ::
     ?TIMEOUT_SEND_SYNC_MIN..?TIMEOUT_SEND_SYNC_MAX.
 -type timeout_send_sync_milliseconds() ::
     limit_min | limit_max | timeout_send_sync_value_milliseconds().
+-type timeout_send_sync_period() ::
+    {1..(?TIMEOUT_SEND_SYNC_MAX div ?MILLISECONDS_IN_SECOND),
+     seconds | second} |
+    {1..(?TIMEOUT_SEND_SYNC_MAX div ?MILLISECONDS_IN_MINUTE),
+     minutes | minute} |
+    {1..(?TIMEOUT_SEND_SYNC_MAX div ?MILLISECONDS_IN_HOUR),
+     hours | hour} |
+    {1..(?TIMEOUT_SEND_SYNC_MAX div ?MILLISECONDS_IN_DAY),
+     days | day} |
+    timeout_send_sync_milliseconds().
 -export_type([timeout_send_sync_value_milliseconds/0,
-              timeout_send_sync_milliseconds/0]).
+              timeout_send_sync_milliseconds/0,
+              timeout_send_sync_period/0]).
 
 -type timeout_terminate_value_milliseconds() ::
     ?TIMEOUT_TERMINATE_MIN..?TIMEOUT_TERMINATE_MAX.
 -type timeout_terminate_milliseconds() ::
     limit_min | limit_max | timeout_terminate_value_milliseconds().
+-type timeout_terminate_period() ::
+    {1..(?TIMEOUT_TERMINATE_MAX div ?MILLISECONDS_IN_SECOND),
+     seconds | second} |
+    timeout_terminate_milliseconds().
 -export_type([timeout_terminate_value_milliseconds/0,
-              timeout_terminate_milliseconds/0]).
+              timeout_terminate_milliseconds/0,
+              timeout_terminate_period/0]).
+
+-type dest_refresh_start_value_milliseconds() ::
+    ?DEST_REFRESH_START_MIN..?DEST_REFRESH_START_MAX.
+-type dest_refresh_start_milliseconds() ::
+    limit_min | limit_max | dest_refresh_start_value_milliseconds().
+-type dest_refresh_start_period() ::
+    {1..(?DEST_REFRESH_START_MAX div ?MILLISECONDS_IN_SECOND),
+     seconds | second} |
+    {1..(?DEST_REFRESH_START_MAX div ?MILLISECONDS_IN_MINUTE),
+     minutes | minute} |
+    {1..(?DEST_REFRESH_START_MAX div ?MILLISECONDS_IN_HOUR),
+     hours | hour} |
+    {1..(?DEST_REFRESH_START_MAX div ?MILLISECONDS_IN_DAY),
+     days | day} |
+    dest_refresh_start_milliseconds().
+-export_type([dest_refresh_start_value_milliseconds/0,
+              dest_refresh_start_milliseconds/0,
+              dest_refresh_start_period/0]).
+
+-type dest_refresh_delay_value_milliseconds() ::
+    ?DEST_REFRESH_DELAY_MIN..?DEST_REFRESH_DELAY_MAX.
+-type dest_refresh_delay_milliseconds() ::
+    limit_min | limit_max | dest_refresh_delay_value_milliseconds().
+-type dest_refresh_delay_period() ::
+    {1..(?DEST_REFRESH_DELAY_MAX div ?MILLISECONDS_IN_SECOND),
+     seconds | second} |
+    {1..(?DEST_REFRESH_DELAY_MAX div ?MILLISECONDS_IN_MINUTE),
+     minutes | minute} |
+    {1..(?DEST_REFRESH_DELAY_MAX div ?MILLISECONDS_IN_HOUR),
+     hours | hour} |
+    {1..(?DEST_REFRESH_DELAY_MAX div ?MILLISECONDS_IN_DAY),
+     days | day} |
+    dest_refresh_delay_milliseconds().
+-export_type([dest_refresh_delay_value_milliseconds/0,
+              dest_refresh_delay_milliseconds/0,
+              dest_refresh_delay_period/0]).
 
 -type request_timeout_immediate_max_value_milliseconds() ::
-    0..?TIMEOUT_MAX_ERLANG.
+    ?REQUEST_TIMEOUT_IMMEDIATE_MAX_MIN..?REQUEST_TIMEOUT_IMMEDIATE_MAX_MAX.
 -type request_timeout_immediate_max_milliseconds() ::
     limit_min | limit_max | request_timeout_immediate_max_value_milliseconds().
+-type request_timeout_immediate_max_period() ::
+    {1..(?REQUEST_TIMEOUT_IMMEDIATE_MAX_MAX div ?MILLISECONDS_IN_SECOND),
+     seconds | second} |
+    {1..(?REQUEST_TIMEOUT_IMMEDIATE_MAX_MAX div ?MILLISECONDS_IN_MINUTE),
+     minutes | minute} |
+    {1..(?REQUEST_TIMEOUT_IMMEDIATE_MAX_MAX div ?MILLISECONDS_IN_HOUR),
+     hours | hour} |
+    {1..(?REQUEST_TIMEOUT_IMMEDIATE_MAX_MAX div ?MILLISECONDS_IN_DAY),
+     days | day} |
+    request_timeout_immediate_max_milliseconds().
 -export_type([request_timeout_immediate_max_value_milliseconds/0,
-              request_timeout_immediate_max_milliseconds/0]).
+              request_timeout_immediate_max_milliseconds/0,
+              request_timeout_immediate_max_period/0]).
 
 -type response_timeout_immediate_max_value_milliseconds() ::
-    0..?TIMEOUT_MAX_ERLANG.
+    ?RESPONSE_TIMEOUT_IMMEDIATE_MAX_MIN..?RESPONSE_TIMEOUT_IMMEDIATE_MAX_MAX.
 -type response_timeout_immediate_max_milliseconds() ::
     limit_min | limit_max | response_timeout_immediate_max_value_milliseconds().
+-type response_timeout_immediate_max_period() ::
+    {1..(?RESPONSE_TIMEOUT_IMMEDIATE_MAX_MAX div ?MILLISECONDS_IN_SECOND),
+     seconds | second} |
+    {1..(?RESPONSE_TIMEOUT_IMMEDIATE_MAX_MAX div ?MILLISECONDS_IN_MINUTE),
+     minutes | minute} |
+    {1..(?RESPONSE_TIMEOUT_IMMEDIATE_MAX_MAX div ?MILLISECONDS_IN_HOUR),
+     hours | hour} |
+    {1..(?RESPONSE_TIMEOUT_IMMEDIATE_MAX_MAX div ?MILLISECONDS_IN_DAY),
+     days | day} |
+    response_timeout_immediate_max_milliseconds().
 -export_type([response_timeout_immediate_max_value_milliseconds/0,
-              response_timeout_immediate_max_milliseconds/0]).
+              response_timeout_immediate_max_milliseconds/0,
+              response_timeout_immediate_max_period/0]).
+
+-type fatal_timeout_delay_value_milliseconds() ::
+    ?FATAL_TIMEOUT_DELAY_MIN..?FATAL_TIMEOUT_DELAY_MAX.
+-type fatal_timeout_delay_milliseconds() ::
+    limit_min | limit_max | fatal_timeout_delay_value_milliseconds().
+-type fatal_timeout_delay_period() ::
+    {1..(?FATAL_TIMEOUT_DELAY_MAX div ?MILLISECONDS_IN_SECOND),
+     seconds | second} |
+    {1..(?FATAL_TIMEOUT_DELAY_MAX div ?MILLISECONDS_IN_MINUTE),
+     minutes | minute} |
+    {1..(?FATAL_TIMEOUT_DELAY_MAX div ?MILLISECONDS_IN_HOUR),
+     hours | hour} |
+    {1..(?FATAL_TIMEOUT_DELAY_MAX div ?MILLISECONDS_IN_DAY),
+     days | day} |
+    fatal_timeout_delay_milliseconds().
+-export_type([fatal_timeout_delay_value_milliseconds/0,
+              fatal_timeout_delay_milliseconds/0,
+              fatal_timeout_delay_period/0]).
 
 -type restart_delay_value_milliseconds() ::
     0..?TIMEOUT_MAX_ERLANG.
 -type restart_delay_milliseconds() ::
     limit_min | limit_max | restart_delay_value_milliseconds().
+-type restart_delay_period() ::
+    {1..(?TIMEOUT_MAX_ERLANG div ?MILLISECONDS_IN_SECOND),
+     seconds | second} |
+    {1..(?TIMEOUT_MAX_ERLANG div ?MILLISECONDS_IN_MINUTE),
+     minutes | minute} |
+    {1..(?TIMEOUT_MAX_ERLANG div ?MILLISECONDS_IN_HOUR),
+     hours | hour} |
+    {1..(?TIMEOUT_MAX_ERLANG div ?MILLISECONDS_IN_DAY),
+     days | day} |
+    restart_delay_milliseconds().
 -export_type([restart_delay_value_milliseconds/0,
-              restart_delay_milliseconds/0]).
+              restart_delay_milliseconds/0,
+              restart_delay_period/0]).
 
 -type latency_mean_time_value_milliseconds() ::
     0..?TIMEOUT_MAX_ERLANG.
 -type latency_mean_time_milliseconds() ::
     limit_min | limit_max | latency_mean_time_value_milliseconds().
+-type latency_mean_time_period() ::
+    {1..(?TIMEOUT_MAX_ERLANG div ?MILLISECONDS_IN_SECOND),
+     seconds | second} |
+    {1..(?TIMEOUT_MAX_ERLANG div ?MILLISECONDS_IN_MINUTE),
+     minutes | minute} |
+    {1..(?TIMEOUT_MAX_ERLANG div ?MILLISECONDS_IN_HOUR),
+     hours | hour} |
+    {1..(?TIMEOUT_MAX_ERLANG div ?MILLISECONDS_IN_DAY),
+     days | day} |
+    latency_mean_time_milliseconds().
 -type latency_min_time_value_milliseconds() ::
     0..?TIMEOUT_MAX_ERLANG.
 -type latency_min_time_milliseconds() ::
     limit_min | limit_max | latency_min_time_value_milliseconds().
+-type latency_min_time_period() ::
+    {1..(?TIMEOUT_MAX_ERLANG div ?MILLISECONDS_IN_SECOND),
+     seconds | second} |
+    {1..(?TIMEOUT_MAX_ERLANG div ?MILLISECONDS_IN_MINUTE),
+     minutes | minute} |
+    {1..(?TIMEOUT_MAX_ERLANG div ?MILLISECONDS_IN_HOUR),
+     hours | hour} |
+    {1..(?TIMEOUT_MAX_ERLANG div ?MILLISECONDS_IN_DAY),
+     days | day} |
+    latency_min_time_milliseconds().
 -type latency_max_time_value_milliseconds() ::
     1..?TIMEOUT_MAX_ERLANG.
 -type latency_max_time_milliseconds() ::
     limit_min | limit_max | latency_max_time_value_milliseconds().
+-type latency_max_time_period() ::
+    {1..(?TIMEOUT_MAX_ERLANG div ?MILLISECONDS_IN_SECOND),
+     seconds | second} |
+    {1..(?TIMEOUT_MAX_ERLANG div ?MILLISECONDS_IN_MINUTE),
+     minutes | minute} |
+    {1..(?TIMEOUT_MAX_ERLANG div ?MILLISECONDS_IN_HOUR),
+     hours | hour} |
+    {1..(?TIMEOUT_MAX_ERLANG div ?MILLISECONDS_IN_DAY),
+     days | day} |
+    latency_max_time_milliseconds().
 -type latency_time_value_milliseconds() ::
     1..?TIMEOUT_MAX_ERLANG.
 -type latency_time_milliseconds() ::
     limit_min | limit_max | latency_time_value_milliseconds().
+-type latency_time_period() ::
+    {1..(?TIMEOUT_MAX_ERLANG div ?MILLISECONDS_IN_SECOND),
+     seconds | second} |
+    {1..(?TIMEOUT_MAX_ERLANG div ?MILLISECONDS_IN_MINUTE),
+     minutes | minute} |
+    {1..(?TIMEOUT_MAX_ERLANG div ?MILLISECONDS_IN_HOUR),
+     hours | hour} |
+    {1..(?TIMEOUT_MAX_ERLANG div ?MILLISECONDS_IN_DAY),
+     days | day} |
+    latency_time_milliseconds().
 -export_type([latency_mean_time_value_milliseconds/0,
               latency_mean_time_milliseconds/0,
+              latency_mean_time_period/0,
               latency_min_time_value_milliseconds/0,
               latency_min_time_milliseconds/0,
+              latency_min_time_period/0,
               latency_max_time_value_milliseconds/0,
               latency_max_time_milliseconds/0,
+              latency_max_time_period/0,
               latency_time_value_milliseconds/0,
-              latency_time_milliseconds/0]).
+              latency_time_milliseconds/0,
+              latency_time_period/0]).
 
 -type acl() ::
     list(atom() | cloudi:service_name_pattern()).
@@ -180,11 +345,29 @@
 
 -type seconds() ::
     non_neg_integer().
--export_type([seconds/0]).
-
 -type period_seconds() ::
-    1..(?TIMEOUT_MAX_ERLANG div 1000).
--export_type([period_seconds/0]).
+    1..(?TIMEOUT_MAX_ERLANG div ?MILLISECONDS_IN_SECOND).
+-type period() ::
+    limit_min | limit_max |
+    {1..(?TIMEOUT_MAX_ERLANG div ?MILLISECONDS_IN_MINUTE),
+     minutes | minute} |
+    {1..(?TIMEOUT_MAX_ERLANG div ?MILLISECONDS_IN_HOUR),
+     hours | hour} |
+    {1..(?TIMEOUT_MAX_ERLANG div ?MILLISECONDS_IN_DAY),
+     days | day} |
+    period_seconds().
+-type period_gte() ::
+    limit_min |
+    {pos_integer(), minutes | minute} |
+    {pos_integer(), hours | hour} |
+    {pos_integer(), days | day} |
+    seconds().
+-type max_t() :: period_gte().
+-export_type([seconds/0,
+              period_seconds/0,
+              period/0,
+              period_gte/0,
+              max_t/0]).
 
 -type aspect_init_after_internal_f() ::
     fun((Args :: list(),
@@ -363,6 +546,9 @@
 -export_type([max_heap_size_options/0]).
 
 -type file_path() :: nonempty_string().
+-type args_internal() :: list().
+-type args_external() :: string().
+-type env_external() :: list({string(), string()}).
 -type limit_external_key() ::
     as | core | cpu | data | fsize | memlock | msgqueue | nice | nofile |
     nproc | rss | rtprio | rttime | sigpending | stack | vmem.
@@ -395,6 +581,9 @@
 -type directory_external() ::
     file_path() | undefined.
 -export_type([file_path/0,
+              args_internal/0,
+              args_external/0,
+              env_external/0,
               limit_external_key/0,
               limit_external_value/0,
               limit_external/0,
@@ -407,32 +596,32 @@
               directory_external/0]).
 
 -type service_options_rate_request_max_options() ::
-    list({period, period_seconds()} |
+    list({period, period()} |
          {value, number()}).
 -type service_options_count_process_dynamic_options() ::
-    list({period, period_seconds()} |
+    list({period, period()} |
          {rate_request_max, number()} |
          {rate_request_min, number()} |
          {count_max, number()} |
          {count_min, number()}).
 -type service_options_restart_delay_options() ::
-    list({time_exponential_min, restart_delay_milliseconds()} |
-         {time_exponential_max, restart_delay_milliseconds()} |
-         {time_linear_min, restart_delay_milliseconds()} |
-         {time_linear_slope, restart_delay_milliseconds()} |
-         {time_linear_max, restart_delay_milliseconds()} |
-         {time_absolute, restart_delay_milliseconds()}).
+    list({time_exponential_min, restart_delay_period()} |
+         {time_exponential_max, restart_delay_period()} |
+         {time_linear_min, restart_delay_period()} |
+         {time_linear_slope, restart_delay_period()} |
+         {time_linear_max, restart_delay_period()} |
+         {time_absolute, restart_delay_period()}).
 -type service_options_monkey_latency_options() ::
-    list({time_uniform_min, latency_min_time_milliseconds()} |
-         {time_uniform_max, latency_max_time_milliseconds()} |
-         {time_gaussian_mean, latency_mean_time_milliseconds()} |
+    list({time_uniform_min, latency_min_time_period()} |
+         {time_uniform_max, latency_max_time_period()} |
+         {time_gaussian_mean, latency_mean_time_period()} |
          {time_gaussian_stddev, float() | pos_integer()} |
-         {time_absolute, latency_time_milliseconds()}).
+         {time_absolute, latency_time_period()}).
 -type service_options_monkey_chaos_options() ::
     list({probability_request, float()} |
          {probability_day, float()}).
 -type service_options_internal_hibernate_options() ::
-    list({period, period_seconds()} |
+    list({period, period()} |
          {rate_request_min, number()}).
 -type service_options_internal() ::
     list({priority_default, priority()} |
@@ -440,19 +629,22 @@
          {queue_size, undefined | pos_integer()} |
          {rate_request_max,
           service_options_rate_request_max_options() | number() | undefined} |
-         {dest_refresh_start, dest_refresh_delay_milliseconds()} |
-         {dest_refresh_delay, dest_refresh_delay_milliseconds()} |
+         {dest_refresh_start, dest_refresh_start_period()} |
+         {dest_refresh_delay, dest_refresh_delay_period()} |
          {request_name_lookup, sync | async} |
          {request_timeout_adjustment, boolean()} |
          {request_timeout_immediate_max,
-          request_timeout_immediate_max_milliseconds()} |
+          request_timeout_immediate_max_period()} |
          {response_timeout_adjustment, boolean()} |
          {response_timeout_immediate_max,
-          response_timeout_immediate_max_milliseconds()} |
+          response_timeout_immediate_max_period()} |
          {count_process_dynamic,
           service_options_count_process_dynamic_options() | false} |
+         {fatal_exceptions, boolean()} |
+         {fatal_timeout, boolean()} |
+         {fatal_timeout_delay, fatal_timeout_delay_period()} |
          {timeout_terminate,
-          undefined | timeout_terminate_milliseconds()} |
+          undefined | timeout_terminate_period()} |
          {restart_all, boolean()} |
          {restart_delay,
           service_options_restart_delay_options() | false} |
@@ -515,19 +707,22 @@
          {queue_size, undefined | pos_integer()} |
          {rate_request_max,
           service_options_rate_request_max_options() | number() | undefined} |
-         {dest_refresh_start, dest_refresh_delay_milliseconds()} |
-         {dest_refresh_delay, dest_refresh_delay_milliseconds()} |
+         {dest_refresh_start, dest_refresh_start_period()} |
+         {dest_refresh_delay, dest_refresh_delay_period()} |
          {request_name_lookup, sync | async} |
          {request_timeout_adjustment, boolean()} |
          {request_timeout_immediate_max,
-          request_timeout_immediate_max_milliseconds()} |
+          request_timeout_immediate_max_period()} |
          {response_timeout_adjustment, boolean()} |
          {response_timeout_immediate_max,
-          response_timeout_immediate_max_milliseconds()} |
+          response_timeout_immediate_max_period()} |
          {count_process_dynamic,
           service_options_count_process_dynamic_options() | false} |
+         {fatal_exceptions, boolean()} |
+         {fatal_timeout, boolean()} |
+         {fatal_timeout_delay, fatal_timeout_delay_period()} |
          {timeout_terminate,
-          undefined | timeout_terminate_milliseconds()} |
+          undefined | timeout_terminate_period()} |
          {restart_all, boolean()} |
          {restart_delay,
           service_options_restart_delay_options() | false} |
@@ -574,20 +769,20 @@
                   {prefix, cloudi:service_name_pattern()} |
                   {module, atom() | file_path()} |
                   {file_path, file_path()} |
-                  {args, list()} |
-                  {env, list({string(), string()})} |
+                  {args, args_internal() | args_external()} |
+                  {env, env_external()} |
                   {dest_refresh, dest_refresh()} |
                   {protocol, 'default' | 'local' | 'tcp' | 'udp'} |
                   {buffer_size, 'default' | pos_integer()} |
-                  {timeout_init, timeout_initialize_value_milliseconds()} |
-                  {timeout_async, timeout_send_async_value_milliseconds()} |
-                  {timeout_sync, timeout_send_sync_value_milliseconds()} |
+                  {timeout_init, timeout_initialize_period()} |
+                  {timeout_async, timeout_send_async_period()} |
+                  {timeout_sync, timeout_send_sync_period()} |
                   {dest_list_deny, dest_list()} |
                   {dest_list_allow, dest_list()} |
                   {count_process, pos_integer() | float()} |
                   {count_thread, pos_integer() | float()} |
                   {max_r, non_neg_integer()} |
-                  {max_t, seconds()} |
+                  {max_t, max_t()} |
                   {options, service_options_internal() |
                             service_options_external()}).
 -type service() :: #internal{} | #external{}.
@@ -597,6 +792,28 @@
               service_proplist/0,
               service/0]).
 
+-type bytes_value() :: pos_integer().
+% integer count of outages during a time period as an ASCII byte
+% (`"X"' >= 10 and `"|"' is the cursor marking the current time)
+-type outage_byte() :: $1..$9 | $X | $| | $ .
+% 48 bytes, each represents 30 minutes, with | byte cursor (49 total)
+% first byte time period starts at 00:00 UTC
+% `"                            |                    "'
+-type outages_day_string() ::
+    nonempty_list(outage_byte()).
+% 42 bytes, each represents 4 hours, with | byte cursor (43 total)
+% first byte time period starts on Monday (ISO 8601) at 00:00 UTC
+% `"                         |1                "'
+-type outages_week_string() ::
+    nonempty_list(outage_byte()).
+% 31 bytes, each represents 1 day of a month, with | byte cursor (32 total)
+% `"       |                      1 "'
+-type outages_month_string() ::
+    nonempty_list(outage_byte()).
+% 36 bytes, each represents 1/3rd of a month, with | byte cursor (37 total)
+% `"    1  |                             "'
+-type outages_year_string() ::
+    nonempty_list(outage_byte()).
 % `"99.9999998 %"'
 -type availability() ::
     nonempty_list($0..$9 | $. | $  | $%).
@@ -630,7 +847,13 @@
 % `"0.0"'
 -type float_string_ge_0() ::
     nonempty_list($0..$9 | $.).
--export_type([availability/0,
+-export_type([bytes_value/0,
+              outage_byte/0,
+              outages_day_string/0,
+              outages_week_string/0,
+              outages_month_string/0,
+              outages_year_string/0,
+              availability/0,
               availability_approx/0,
               nanoseconds_string/0,
               nanoseconds_string_signed/0,
@@ -648,6 +871,7 @@
                   {module, atom()} |
                   {count_process, pos_integer()} |
                   {pids_erlang, list(pid())} |
+                  {size_erlang, bytes_value()} |
                   {suspended, boolean()} |
                   {uptime_total, nanoseconds_string()} |
                   {uptime_running, nanoseconds_string()} |
@@ -657,6 +881,10 @@
                   {downtime_week_restarting, nanoseconds_string_approx_gt()} |
                   {downtime_month_restarting, nanoseconds_string_approx_gt()} |
                   {downtime_year_restarting, nanoseconds_string_approx_gt()} |
+                  {outages_day_restarting, outages_day_string()} |
+                  {outages_week_restarting, outages_week_string()} |
+                  {outages_month_restarting, outages_month_string()} |
+                  {outages_year_restarting, outages_year_string()} |
                   {interrupt_day_updating, nanoseconds_string_approx_gt()} |
                   {interrupt_week_updating, nanoseconds_string_approx_gt()} |
                   {interrupt_month_updating, nanoseconds_string_approx_gt()} |
@@ -689,6 +917,7 @@
                   {count_thread, pos_integer()} |
                   {pids_os, list(pos_integer())} |
                   {pids_erlang, list(pid())} |
+                  {size_erlang, bytes_value()} |
                   {suspended, boolean()} |
                   {uptime_total, nanoseconds_string()} |
                   {uptime_running, nanoseconds_string()} |
@@ -698,6 +927,10 @@
                   {downtime_week_restarting, nanoseconds_string_approx_gt()} |
                   {downtime_month_restarting, nanoseconds_string_approx_gt()} |
                   {downtime_year_restarting, nanoseconds_string_approx_gt()} |
+                  {outages_day_restarting, outages_day_string()} |
+                  {outages_week_restarting, outages_week_string()} |
+                  {outages_month_restarting, outages_month_string()} |
+                  {outages_year_restarting, outages_year_string()} |
                   {interrupt_day_updating, nanoseconds_string_approx_gt()} |
                   {interrupt_week_updating, nanoseconds_string_approx_gt()} |
                   {interrupt_month_updating, nanoseconds_string_approx_gt()} |
@@ -749,9 +982,9 @@
                   {code_paths_add, list(string())} |
                   {code_paths_remove, list(string())} |
                   {dest_refresh, dest_refresh()} |
-                  {timeout_init, timeout_initialize_milliseconds()} |
-                  {timeout_async, timeout_send_async_milliseconds()} |
-                  {timeout_sync, timeout_send_sync_milliseconds()} |
+                  {timeout_init, timeout_initialize_period()} |
+                  {timeout_async, timeout_send_async_period()} |
+                  {timeout_sync, timeout_send_sync_period()} |
                   {dest_list_deny, dest_list()} |
                   {dest_list_allow, dest_list()} |
                   {options, service_update_plan_options_internal()}).
@@ -766,9 +999,9 @@
                   {code_paths_add, list(string())} |
                   {code_paths_remove, list(string())} |
                   {dest_refresh, dest_refresh()} |
-                  {timeout_init, timeout_initialize_milliseconds()} |
-                  {timeout_async, timeout_send_async_milliseconds()} |
-                  {timeout_sync, timeout_send_sync_milliseconds()} |
+                  {timeout_init, timeout_initialize_period()} |
+                  {timeout_async, timeout_send_async_period()} |
+                  {timeout_sync, timeout_send_sync_period()} |
                   {dest_list_deny, dest_list()} |
                   {dest_list_allow, dest_list()} |
                   {options, service_update_plan_options_external()}).
@@ -778,15 +1011,18 @@
          {queue_size, undefined | pos_integer()} |
          {rate_request_max,
           service_options_rate_request_max_options() | number() | undefined} |
-         {dest_refresh_start, dest_refresh_delay_milliseconds()} |
-         {dest_refresh_delay, dest_refresh_delay_milliseconds()} |
+         {dest_refresh_start, dest_refresh_start_period()} |
+         {dest_refresh_delay, dest_refresh_delay_period()} |
          {request_name_lookup, sync | async} |
          {request_timeout_adjustment, boolean()} |
          {request_timeout_immediate_max,
-          request_timeout_immediate_max_milliseconds()} |
+          request_timeout_immediate_max_period()} |
          {response_timeout_adjustment, boolean()} |
          {response_timeout_immediate_max,
-          response_timeout_immediate_max_milliseconds()} |
+          response_timeout_immediate_max_period()} |
+         {fatal_exceptions, boolean()} |
+         {fatal_timeout, boolean()} |
+         {fatal_timeout_delay, fatal_timeout_delay_period()} |
          {monkey_latency,
           service_options_monkey_latency_options() | system | false} |
          {monkey_chaos,
@@ -834,7 +1070,7 @@
                {sensitive, boolean()} |
                {message_queue_data, off_heap | on_heap})} |
          {hibernate,
-          list({period, period_seconds()} |
+          list({period, period()} |
                {rate_request_min, number()}) | boolean()} |
          {reload, boolean()}).
 -type service_update_plan_options_external() ::
@@ -843,15 +1079,18 @@
          {queue_size, undefined | pos_integer()} |
          {rate_request_max,
           service_options_rate_request_max_options() | number() | undefined} |
-         {dest_refresh_start, dest_refresh_delay_milliseconds()} |
-         {dest_refresh_delay, dest_refresh_delay_milliseconds()} |
+         {dest_refresh_start, dest_refresh_start_period()} |
+         {dest_refresh_delay, dest_refresh_delay_period()} |
          {request_name_lookup, sync | async} |
          {request_timeout_adjustment, boolean()} |
          {request_timeout_immediate_max,
-          request_timeout_immediate_max_milliseconds()} |
+          request_timeout_immediate_max_period()} |
          {response_timeout_adjustment, boolean()} |
          {response_timeout_immediate_max,
-          response_timeout_immediate_max_milliseconds()} |
+          response_timeout_immediate_max_period()} |
+         {fatal_exceptions, boolean()} |
+         {fatal_timeout, boolean()} |
+         {fatal_timeout_delay, fatal_timeout_delay_period()} |
          {monkey_latency,
           service_options_monkey_latency_options() | system | false} |
          {monkey_chaos,
@@ -879,7 +1118,7 @@
               service_update_plan/0]).
 
 -type node_reconnect_delay_seconds() ::
-    period_seconds().
+    period().
 -export_type([node_reconnect_delay_seconds/0]).
 -type nodes_properties() ::
     node() |
@@ -935,6 +1174,10 @@
                   {downtime_week_disconnected, nanoseconds_string_approx_gt()} |
                   {downtime_month_disconnected, nanoseconds_string_approx_gt()} |
                   {downtime_year_disconnected, nanoseconds_string_approx_gt()} |
+                  {outages_day_disconnected, outages_day_string()} |
+                  {outages_week_disconnected, outages_week_string()} |
+                  {outages_month_disconnected, outages_month_string()} |
+                  {outages_year_disconnected, outages_year_string()} |
                   {availability_day, availability_approx()} |
                   {availability_week, availability_approx()} |
                   {availability_month, availability_approx()} |
@@ -955,8 +1198,8 @@
          Timestamp :: erlang:timestamp(),
          Node :: node(),
          Pid :: pid(),
-         Module :: module(),
-         Line :: pos_integer(),
+         FileName :: nonempty_string(),
+         Line :: non_neg_integer(),
          Function :: atom() | undefined,
          Arity :: arity() | undefined,
          MetaData :: list({atom(), any()}) | #{},
@@ -975,6 +1218,14 @@
               aspect_log_after/0]).
 -type loglevel() :: loglevel_on() | off.
 -type loglevel_on() :: fatal | error | warn | info | debug | trace.
+-type logging_file_sync_value_milliseconds() ::
+    ?LOGGER_FILE_SYNC_MIN..?LOGGER_FILE_SYNC_MAX.
+-type logging_file_sync_milliseconds() ::
+    limit_min | limit_max | logging_file_sync_value_milliseconds().
+-type logging_file_sync_period() ::
+    {1..(?LOGGER_FILE_SYNC_MAX div ?MILLISECONDS_IN_SECOND),
+     seconds | second} |
+    logging_file_sync_milliseconds().
 -type logging_syslog_identity() :: nonempty_string().
 -type logging_syslog_facility() ::
     kernel | user | mail | daemon | auth0 | syslog |
@@ -1000,7 +1251,7 @@
          {host, logging_syslog_host()} |
          {port, logging_syslog_port()}).
 -type logging_formatters_set_proplist() ::
-    list({any | nonempty_list(module()),
+    list({any | nonempty_list(module() | nonempty_string()),
           list(fatal | error | warn | info | debug | trace |
                emergency | alert | critical | warning | notice |
                {level, fatal | error | warn | info | debug | trace |
@@ -1008,7 +1259,7 @@
                {output, module() | undefined} |
                {output_args, list()} |
                {output_max_r, non_neg_integer()} |
-               {output_max_t, cloudi_service_api:seconds()} |
+               {output_max_t, max_t()} |
                {formatter, module() | undefined} |
                {formatter_config, list()})}).
 -type iso8601() ::
@@ -1045,7 +1296,8 @@
          {file_read_fail_count, integer_string_gt_0()} |
          {file_read_fail_types, nonempty_list(atom())}).
 -type logging_proplist() ::
-    nonempty_list({file, string() | undefined} |
+    nonempty_list({file, nonempty_string() | undefined} |
+                  {file_sync, logging_file_sync_period()} |
                   {level, loglevel()} |
                   {redirect, node() | undefined} |
                   {syslog, logging_syslog_set_proplist() | undefined} |
@@ -1059,6 +1311,9 @@
                   {aspects_log_after, list(aspect_log_after())}).
 -export_type([loglevel/0,
               loglevel_on/0,
+              logging_file_sync_value_milliseconds/0,
+              logging_file_sync_milliseconds/0,
+              logging_file_sync_period/0,
               logging_syslog_identity/0,
               logging_syslog_facility/0,
               logging_syslog_transport/0,
@@ -1084,19 +1339,18 @@
                   {build_operating_system, nonempty_string()} |
                   {build_erlang_otp_release, nonempty_string()} |
                   {build_cloudi_time, nonempty_string()} |
+                  {build_cloudi_version, nonempty_string()} |
                   {build_cloudi_cxx_compiler_version, nonempty_string()} |
                   {build_cloudi_cxx_dependencies_versions, nonempty_string()} |
                   {build_erts_c_compiler_version, nonempty_string()} |
                   {install_erlang_erts_time, iso8601_seconds()} |
                   {install_erlang_kernel_time, iso8601_seconds()} |
                   {install_erlang_stdlib_time, iso8601_seconds()} |
-                  {install_erlang_sasl_time, iso8601_seconds()} |
                   {install_erlang_compiler_time, iso8601_seconds()} |
                   {install_cloudi_time, iso8601_seconds()} |
                   {runtime_erlang_erts_version, nonempty_string()} |
                   {runtime_erlang_kernel_version, nonempty_string()} |
                   {runtime_erlang_stdlib_version, nonempty_string()} |
-                  {runtime_erlang_sasl_version, nonempty_string()} |
                   {runtime_erlang_compiler_version, nonempty_string()} |
                   {runtime_erlang_compilation, nonempty_string()} |
                   {runtime_cloudi_version, nonempty_string()} |
