@@ -9,7 +9,7 @@
 %%%
 %%% MIT License
 %%%
-%%% Copyright (c) 2011-2020 Michael Truog <mjtruog at protonmail dot com>
+%%% Copyright (c) 2011-2022 Michael Truog <mjtruog at protonmail dot com>
 %%%
 %%% Permission is hereby granted, free of charge, to any person obtaining a
 %%% copy of this software and associated documentation files (the "Software"),
@@ -30,8 +30,8 @@
 %%% DEALINGS IN THE SOFTWARE.
 %%%
 %%% @author Michael Truog <mjtruog at protonmail dot com>
-%%% @copyright 2011-2020 Michael Truog
-%%% @version 2.0.1 {@date} {@time}
+%%% @copyright 2011-2022 Michael Truog
+%%% @version 2.0.5 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_service_api_requests).
@@ -63,7 +63,7 @@
 
 -define(FORMAT_ERLANG, "erl").
 -define(FORMAT_JSON, "json").
- 
+
 %%%------------------------------------------------------------------------
 %%% External interface functions
 %%%------------------------------------------------------------------------
@@ -144,7 +144,7 @@ cloudi_service_init([], Prefix, _Timeout, Dispatcher) ->
 
 cloudi_service_handle_request(_RequestType, Name, _Pattern,
                               _RequestInfo, Request,
-                              Timeout, _Priority, _TransId, _Pid,
+                              Timeout, _Priority, _TransId, _Source,
                               #state{prefix = Prefix} = State, _Dispatcher) ->
     Response = case cloudi_service_name:suffix(Prefix, Name) of
         "rpc/" ++ MethodSuffix ->
@@ -192,7 +192,7 @@ format_erlang_call(Method, 2, Request, Timeout) ->
             convert_term_to_erlang(Result)
     catch
         exit:invalid_input ->
-            <<>>;
+            convert_term_to_erlang({error, timeout});
         ErrorType:Error ->
             ?LOG_DEBUG("~p ~p", [ErrorType, Error]),
             <<>>
@@ -217,7 +217,7 @@ format_json_call(Method, 2, Request, Timeout) ->
             convert_term_to_json(Result, Method, true)
     catch
         exit:invalid_input ->
-            <<>>;
+            convert_term_to_json({error, timeout}, Method, true);
         ErrorType:Error ->
             ?LOG_DEBUG("~p ~p", [ErrorType, Error]),
             <<>>
@@ -392,7 +392,7 @@ convert_json_to_term(Method, Request)
     case json_decode(Request) of
         Binary when is_binary(Binary) ->
             erlang:binary_to_list(Binary);
-        [{ScopeBinary, NameBinary}] 
+        [{ScopeBinary, NameBinary}]
             when is_binary(ScopeBinary),
                  is_binary(NameBinary) ->
             {erlang:binary_to_existing_atom(ScopeBinary, utf8),
@@ -532,11 +532,14 @@ convert_json_to_term_service([{<<"buffer_size">>, Value} | Service], L) ->
     end,
     convert_json_to_term_service(Service, [{buffer_size, BufferSize} | L]);
 convert_json_to_term_service([{<<"timeout_init">>, Value} | Service], L) ->
-    convert_json_to_term_service(Service, [{timeout_init, Value} | L]);
+    TimeoutInit = convert_json_to_term_period(Value),
+    convert_json_to_term_service(Service, [{timeout_init, TimeoutInit} | L]);
 convert_json_to_term_service([{<<"timeout_async">>, Value} | Service], L) ->
-    convert_json_to_term_service(Service, [{timeout_async, Value} | L]);
+    TimeoutAsync = convert_json_to_term_period(Value),
+    convert_json_to_term_service(Service, [{timeout_async, TimeoutAsync} | L]);
 convert_json_to_term_service([{<<"timeout_sync">>, Value} | Service], L) ->
-    convert_json_to_term_service(Service, [{timeout_sync, Value} | L]);
+    TimeoutSync = convert_json_to_term_period(Value),
+    convert_json_to_term_service(Service, [{timeout_sync, TimeoutSync} | L]);
 convert_json_to_term_service([{<<"dest_list_deny">>, DenyL} | Service], L) ->
     Deny = if
         is_binary(DenyL) ->
@@ -560,7 +563,8 @@ convert_json_to_term_service([{<<"count_thread">>, Value} | Service], L) ->
 convert_json_to_term_service([{<<"max_r">>, Value} | Service], L) ->
     convert_json_to_term_service(Service, [{max_r, Value} | L]);
 convert_json_to_term_service([{<<"max_t">>, Value} | Service], L) ->
-    convert_json_to_term_service(Service, [{max_t, Value} | L]);
+    MaxT = convert_json_to_term_period(Value),
+    convert_json_to_term_service(Service, [{max_t, MaxT} | L]);
 convert_json_to_term_service([{<<"options">>, OptionsL} | Service], L) ->
     Options = if
         is_binary(OptionsL) ->
@@ -633,6 +637,23 @@ convert_json_to_term_atoms([H | L])
 convert_json_to_term_atoms([_ | _]) ->
     invalid.
 
+convert_json_to_term_period(<<"limit_max">>) ->
+    limit_max;
+convert_json_to_term_period(<<"limit_min">>) ->
+    limit_min;
+convert_json_to_term_period(<<"{", _/binary>> = Value) ->
+    cloudi_string:binary_to_term(Value);
+convert_json_to_term_period([{Unit, Multiplier}])
+    when is_integer(Multiplier),
+         Unit == <<"seconds">> orelse Unit == <<"minutes">> orelse
+         Unit == <<"hours">> orelse Unit == <<"days">> orelse
+         Unit == <<"second">> orelse Unit == <<"minute">> orelse
+         Unit == <<"hour">> orelse Unit == <<"day">> ->
+    {Multiplier, erlang:binary_to_existing_atom(Unit, utf8)};
+convert_json_to_term_period(Value)
+    when is_integer(Value) ->
+    Value.
+
 convert_term_to_json(ok, _, Space) ->
     json_encode([{<<"success">>, true}], Space);
 convert_term_to_json({error, Reason}, _, Space) ->
@@ -659,7 +680,7 @@ convert_term_to_json({ok, Statuses}, services_status = Method, Space) ->
     json_encode([{<<"success">>, true},
                  {erlang:atom_to_binary(Method, utf8),
                   [[{<<"id">>, erlang:list_to_binary(Id)} |
-                    convert_term_to_json_options(Status)]
+                    convert_term_to_json_service_status(Status)]
                    || {Id, Status} <- Statuses]}], Space);
 convert_term_to_json({ok, SuccessL}, Method, Space)
     when Method =:= services_add;
@@ -706,14 +727,14 @@ convert_term_to_json_service(#internal{prefix = Prefix,
                                        options = Options}, Id) ->
     Service0 = [{<<"count_process">>, CountProcess},
                 {<<"max_r">>, MaxR},
-                {<<"max_t">>, MaxT},
+                {<<"max_t">>, convert_term_to_json_period_seconds(MaxT)},
                 {<<"options">>, convert_term_to_json_options(Options)}],
     Service1 = if
         DestListAllow =:= undefined ->
             Service0;
         is_list(DestListAllow) ->
             [{<<"dest_list_allow">>,
-              [erlang:list_to_binary(DestAllow)
+              [cloudi_service_name:utf8(DestAllow)
                || DestAllow <- DestListAllow]} | Service0]
     end,
     Service2 = if
@@ -721,12 +742,16 @@ convert_term_to_json_service(#internal{prefix = Prefix,
             Service1;
         is_list(DestListDeny) ->
             [{<<"dest_list_deny">>,
-              [erlang:list_to_binary(DestDeny)
+              [cloudi_service_name:utf8(DestDeny)
                || DestDeny <- DestListDeny]} | Service1]
     end,
-    Service3 = [{<<"timeout_init">>, TimeoutInit},
-                {<<"timeout_async">>, TimeoutAsync},
-                {<<"timeout_sync">>, TimeoutSync} | Service2],
+    Service3 = [{<<"timeout_init">>,
+                 convert_term_to_json_period_milliseconds(TimeoutInit)},
+                {<<"timeout_async">>,
+                 convert_term_to_json_period_milliseconds(TimeoutAsync)},
+                {<<"timeout_sync">>,
+                 convert_term_to_json_period_milliseconds(TimeoutSync)} |
+                Service2],
     ServiceN = if
         DestRefresh =:= none ->
             Service3;
@@ -734,10 +759,16 @@ convert_term_to_json_service(#internal{prefix = Prefix,
             [{<<"dest_refresh">>,
               erlang:atom_to_binary(DestRefresh, utf8)} | Service3]
     end,
+    ModuleValue = if
+        is_atom(Module) ->
+            erlang:atom_to_binary(Module, utf8);
+        is_list(Module) ->
+            unicode:characters_to_binary(Module)
+    end,
     [{<<"id">>, erlang:list_to_binary(Id)},
      {<<"type">>, <<"internal">>},
-     {<<"prefix">>, erlang:list_to_binary(Prefix)},
-     {<<"module">>, erlang:atom_to_binary(Module, utf8)},
+     {<<"prefix">>, cloudi_service_name:utf8(Prefix)},
+     {<<"module">>, ModuleValue},
      {<<"args">>, cloudi_string:term_to_binary_compact(Args)} | ServiceN];
 convert_term_to_json_service(#external{prefix = Prefix,
                                        file_path = FilePath,
@@ -759,14 +790,14 @@ convert_term_to_json_service(#external{prefix = Prefix,
     Service0 = [{<<"count_process">>, CountProcess},
                 {<<"count_thread">>, CountThread},
                 {<<"max_r">>, MaxR},
-                {<<"max_t">>, MaxT},
+                {<<"max_t">>, convert_term_to_json_period_seconds(MaxT)},
                 {<<"options">>, convert_term_to_json_options(Options)}],
     Service1 = if
         DestListAllow =:= undefined ->
             Service0;
         is_list(DestListAllow) ->
             [{<<"dest_list_allow">>,
-              [erlang:list_to_binary(DestAllow)
+              [cloudi_service_name:utf8(DestAllow)
                || DestAllow <- DestListAllow]} | Service0]
     end,
     Service2 = if
@@ -774,14 +805,18 @@ convert_term_to_json_service(#external{prefix = Prefix,
             Service1;
         is_list(DestListDeny) ->
             [{<<"dest_list_deny">>,
-              [erlang:list_to_binary(DestDeny)
+              [cloudi_service_name:utf8(DestDeny)
                || DestDeny <- DestListDeny]} | Service1]
     end,
     Service3 = [{<<"protocol">>, erlang:atom_to_binary(Protocol, utf8)},
                 {<<"buffer_size">>, BufferSize},
-                {<<"timeout_init">>, TimeoutInit},
-                {<<"timeout_async">>, TimeoutAsync},
-                {<<"timeout_sync">>, TimeoutSync} | Service2],
+                {<<"timeout_init">>,
+                 convert_term_to_json_period_milliseconds(TimeoutInit)},
+                {<<"timeout_async">>,
+                 convert_term_to_json_period_milliseconds(TimeoutAsync)},
+                {<<"timeout_sync">>,
+                 convert_term_to_json_period_milliseconds(TimeoutSync)} |
+                Service2],
     ServiceN = if
         DestRefresh =:= none ->
             Service3;
@@ -791,13 +826,27 @@ convert_term_to_json_service(#external{prefix = Prefix,
     end,
     [{<<"id">>, erlang:list_to_binary(Id)},
      {<<"type">>, <<"external">>},
-     {<<"prefix">>, erlang:list_to_binary(Prefix)},
-     {<<"file_path">>, erlang:list_to_binary(FilePath)},
-     {<<"args">>, erlang:list_to_binary(Args)},
+     {<<"prefix">>, cloudi_service_name:utf8(Prefix)},
+     {<<"file_path">>, unicode:characters_to_binary(FilePath)},
+     {<<"args">>, unicode:characters_to_binary(Args)},
      {<<"env">>,
-      [erlang:list_to_binary(Key ++ "=" ++ Value)
+      [unicode:characters_to_binary(Key ++ "=" ++ Value)
        || {Key, Value} <- Env]} | ServiceN].
 
+convert_term_to_json_service_status([]) ->
+    [];
+convert_term_to_json_service_status([{pids_os, Value} | Options]) ->
+    [{<<"pids_os">>, Value} |
+     convert_term_to_json_service_status(Options)];
+convert_term_to_json_service_status([{pids_erlang, Value} | Options]) ->
+    [{<<"pids_erlang">>,
+      [erlang:list_to_binary(erlang:pid_to_list(Pid)) || Pid <- Value]} |
+     convert_term_to_json_service_status(Options)];
+convert_term_to_json_service_status([{Key, Value} | Options]) ->
+    [{erlang:atom_to_binary(Key, utf8),
+      convert_term_to_json_option(Value)} |
+     convert_term_to_json_service_status(Options)].
+
 convert_term_to_json_option([] = Value) ->
     Value;
 convert_term_to_json_option([{Key, _} | _] = Value)
@@ -852,6 +901,18 @@ convert_term_to_json_atoms([A | L]) ->
     [erlang:atom_to_binary(A, utf8) |
      convert_term_to_json_atoms(L)].
 
+convert_term_to_json_period_milliseconds(limit_max) ->
+    <<"limit_max">>;
+convert_term_to_json_period_milliseconds(limit_min) ->
+    <<"limit_min">>;
+convert_term_to_json_period_milliseconds(Timeout) ->
+    cloudi_args_type:timeout_period_to_milliseconds(Timeout).
+
+convert_term_to_json_period_seconds(limit_min) ->
+    <<"limit_min">>;
+convert_term_to_json_period_seconds(Value) ->
+    cloudi_args_type:period_to_seconds(Value, 0).
+
 json_encode(Term, true) ->
     cloudi_x_jsx:encode(Term, [{indent, 1}]);
 json_encode(Term, false) ->
