@@ -8,7 +8,7 @@
 %%%
 %%% MIT License
 %%%
-%%% Copyright (c) 2009-2020 Michael Truog <mjtruog at protonmail dot com>
+%%% Copyright (c) 2009-2022 Michael Truog <mjtruog at protonmail dot com>
 %%%
 %%% Permission is hereby granted, free of charge, to any person obtaining a
 %%% copy of this software and associated documentation files (the "Software"),
@@ -29,8 +29,8 @@
 %%% DEALINGS IN THE SOFTWARE.
 %%%
 %%% @author Michael Truog <mjtruog at protonmail dot com>
-%%% @copyright 2009-2020 Michael Truog
-%%% @version 1.8.1 {@date} {@time}
+%%% @copyright 2009-2022 Michael Truog
+%%% @version 2.0.5 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_service_db_mysql).
@@ -51,6 +51,7 @@
          cloudi_service_handle_info/3,
          cloudi_service_terminate/3]).
 
+-include_lib("cloudi_core/include/cloudi_constants.hrl").
 -include_lib("cloudi_core/include/cloudi_logger.hrl").
 
 -define(DEFAULT_DATABASE,               undefined). % required argument, string
@@ -202,7 +203,7 @@
 
 -type agent() :: cloudi:agent().
 -type service_name() :: cloudi:service_name().
--type timeout_milliseconds() :: cloudi:timeout_milliseconds().
+-type timeout_period() :: cloudi:timeout_period().
 -type module_response(Result) ::
     {{ok, Result}, NewAgent :: agent()} |
     {{error, cloudi:error_reason()}, NewAgent :: agent()}.
@@ -235,7 +236,7 @@ equery(Agent, Name, Query, Parameters)
              Name :: service_name(),
              Query :: string() | binary(),
              Parameters :: list(),
-             Timeout :: timeout_milliseconds()) ->
+             Timeout :: timeout_period()) ->
     module_response(any()).
 
 equery(Agent, Name, Query, Parameters, Timeout)
@@ -272,7 +273,7 @@ prepare_query(Agent, Name, Identifier, Query)
                     Name :: service_name(),
                     Identifier :: atom(),
                     Query :: string() | binary(),
-                    Timeout :: timeout_milliseconds()) ->
+                    Timeout :: timeout_period()) ->
     module_response(ok).
 
 prepare_query(Agent, Name, Identifier, Query, Timeout)
@@ -308,7 +309,7 @@ execute_query(Agent, Name, Identifier, Arguments)
                     Name :: service_name(),
                     Identifier :: atom(),
                     Arguments :: list(),
-                    Timeout :: timeout_milliseconds()) ->
+                    Timeout :: timeout_period()) ->
     module_response(any()).
 
 execute_query(Agent, Name, Identifier, Arguments, Timeout)
@@ -341,7 +342,7 @@ squery(Agent, Name, Query)
 -spec squery(Agent :: agent(),
              Name :: service_name(),
              Query :: string() | binary(),
-             Timeout :: timeout_milliseconds()) ->
+             Timeout :: timeout_period()) ->
     module_response(any()).
 
 squery(Agent, Name, Query, Timeout)
@@ -374,7 +375,7 @@ transaction(Agent, Name, [Query | _] = QueryList)
 -spec transaction(Agent :: agent(),
                   Name :: service_name(),
                   QueryList :: list(string() | binary()),
-                  Timeout :: timeout_milliseconds()) ->
+                  Timeout :: timeout_period()) ->
     module_response(ok | {error, any()}).
 
 transaction(Agent, Name, [Query | _] = QueryList, Timeout)
@@ -416,9 +417,9 @@ cloudi_service_init(Args, _Prefix, Timeout, Dispatcher) ->
     true = (is_list(UserName) andalso is_integer(hd(UserName))),
     true = (is_list(Password) andalso is_integer(hd(Password))),
     true = (is_integer(Port) andalso (Port > 0)),
-    TimeoutMax = cloudi_service:timeout_max(Dispatcher),
     true = (Ping =:= undefined) orelse
-           (is_integer(Ping) andalso (Ping > 0) andalso (Ping =< TimeoutMax)),
+           (is_integer(Ping) andalso
+            (Ping > 0) andalso (Ping =< ?TIMEOUT_MAX_ERLANG)),
     true = (is_integer(TimeoutConnect) andalso (TimeoutConnect > 0)),
     true = (Encoding =:= utf8) orelse (Encoding =:= latin1),
     true = ((OutputType =:= both) orelse
@@ -452,8 +453,10 @@ cloudi_service_init(Args, _Prefix, Timeout, Dispatcher) ->
                 Ping =:= undefined ->
                     ok;
                 is_integer(Ping) ->
-                    erlang:send_after(Ping, cloudi_service:self(Dispatcher),
-                                      {ping, Ping})
+                    _ = erlang:send_after(Ping,
+                                          cloudi_service:self(Dispatcher),
+                                          {ping, Ping}),
+                    ok
             end,
             {ok, #state{module = Module,
                         connection = Connection,
@@ -467,7 +470,7 @@ cloudi_service_init(Args, _Prefix, Timeout, Dispatcher) ->
 
 cloudi_service_handle_request(_RequestType, _Name, _Pattern,
                               _RequestInfo, Request,
-                              Timeout, _Priority, _TransId, _Pid,
+                              Timeout, _Priority, _TransId, _Source,
                               #state{output_type = OutputType,
                                      external_format = ExternalFormat} = State,
                               _Dispatcher) ->
@@ -491,7 +494,7 @@ cloudi_service_handle_request(_RequestType, _Name, _Pattern,
                             {T1, T2};
                         [T1, T2, T3]
                             when (T1 =:= <<"prepare">>) orelse
-                                 (T1 =:= <<"execute">>) orelse 
+                                 (T1 =:= <<"execute">>) orelse
                                  (T1 =:= <<"equery">>) ->
                             T1New = erlang:binary_to_atom(T1, utf8),
                             T2New = if
