@@ -8,7 +8,7 @@
 %%%
 %%% MIT License
 %%%
-%%% Copyright (c) 2013-2018 Michael Truog <mjtruog at protonmail dot com>
+%%% Copyright (c) 2013-2021 Michael Truog <mjtruog at protonmail dot com>
 %%%
 %%% Permission is hereby granted, free of charge, to any person obtaining a
 %%% copy of this software and associated documentation files (the "Software"),
@@ -29,8 +29,8 @@
 %%% DEALINGS IN THE SOFTWARE.
 %%%
 %%% @author Michael Truog <mjtruog at protonmail dot com>
-%%% @copyright 2013-2018 Michael Truog
-%%% @version 1.7.3 {@date} {@time}
+%%% @copyright 2013-2021 Michael Truog
+%%% @version 2.0.3 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_http_elli_handler).
@@ -61,7 +61,11 @@ handle(Req,
        #elli_state{output_type = OutputType,
                    content_type_forced = ContentTypeForced,
                    content_types_accepted = ContentTypesAccepted,
+                   content_security_policy = ContentSecurityPolicy,
+                   content_security_policy_report = ContentSecurityPolicyReport,
                    set_x_forwarded_for = SetXForwardedFor,
+                   set_x_xss_protection = SetXXSSProtection,
+                   set_x_content_type_options = SetXContentTypeOptions,
                    status_code_timeout = StatusCodeTimeout,
                    query_get_format = QueryGetFormat,
                    use_host_prefix = UseHostPrefix,
@@ -109,11 +113,10 @@ handle(Req,
             end,
             if
                 QueryGetFormat =:= text_pairs ->
-                    QSVals = cloudi_x_cow1_qs:parse_qs(QSRaw),
+                    QSVals = cloudi_x_cow_qs:parse_qs(QSRaw),
                     if
                         (OutputType =:= external) orelse
                         (OutputType =:= binary) orelse (OutputType =:= list) ->
-                            % cloudi_request_info text_pairs format
                             get_query_string_external(QSVals);
                         OutputType =:= internal ->
                             % cloudi_key_value format
@@ -168,13 +171,10 @@ handle(Req,
                     NameIncoming ++ [$/ |
                         cloudi_string:lowercase(erlang:binary_to_list(Method))]
             end,
-            PeerShort = erlang:list_to_binary(inet_parse:ntoa(ClientIpAddr)),
-            PeerLong = cloudi_ip_address:to_binary(ClientIpAddr),
-            PeerPort = erlang:integer_to_binary(ClientPort),
-            HeadersIncoming1 = [{<<"peer">>, PeerShort},
-                                {<<"peer-port">>, PeerPort},
-                                {<<"source-address">>, PeerLong},
-                                {<<"source-port">>, PeerPort},
+            SourceAddress = cloudi_ip_address:to_binary(ClientIpAddr),
+            SourcePort = erlang:integer_to_binary(ClientPort),
+            HeadersIncoming1 = [{<<"source-address">>, SourceAddress},
+                                {<<"source-port">>, SourcePort},
                                 {<<"url-path">>, PathRaw} |
                                 HeadersIncoming0],
             HeadersIncomingN = if
@@ -182,7 +182,7 @@ handle(Req,
                     case lists:keyfind(<<"x-forwarded-for">>, 1,
                                        HeadersIncoming0) of
                         false ->
-                            [{<<"x-forwarded-for">>, PeerShort} |
+                            [{<<"x-forwarded-for">>, SourceAddress} |
                              HeadersIncoming1];
                         _ ->
                             HeadersIncoming1
@@ -218,7 +218,11 @@ handle(Req,
                      _} = Result = handle_response(NameIncoming,
                                                    HeadersOutgoing,
                                                    Response, Req, OutputType,
-                                                   ContentTypeForced),
+                                                   ContentTypeForced,
+                                                   SetXContentTypeOptions,
+                                                   SetXXSSProtection,
+                                                   ContentSecurityPolicy,
+                                                   ContentSecurityPolicyReport),
                     ?LOG_TRACE_APPLY(fun request_time_end_success/5,
                                      [HttpCode, MethodHTTP,
                                       NameIncoming, NameOutgoing,
@@ -360,12 +364,7 @@ header_content_type(Headers) ->
 
 % format for external services, http headers passed as key-value pairs
 headers_external_incoming(L) ->
-    erlang:iolist_to_binary(headers_external_incoming_text(L)).
-
-headers_external_incoming_text([] = L) ->
-    L;
-headers_external_incoming_text([{K, V} | L]) when is_binary(K) ->
-    [[K, 0, V, 0] | headers_external_incoming_text(L)].
+    cloudi_request_info:key_value_new(L, text_pairs).
 
 headers_external_outgoing(<<>>) ->
     [];
@@ -375,30 +374,19 @@ headers_external_outgoing([{_, _} | _] = ResponseInfo) ->
     ResponseInfo;
 headers_external_outgoing(ResponseInfo)
     when is_binary(ResponseInfo) ->
-    headers_external_outgoing_text(binary:split(ResponseInfo, <<0>>, [global])).
+    cloudi_response_info:key_value_parse(ResponseInfo, list).
 
-headers_external_outgoing_text([<<>>]) ->
-    [];
-headers_external_outgoing_text([K, <<>>]) ->
-    [{K, <<>>}];
-headers_external_outgoing_text([K, V | L]) ->
-    [{K, V} | headers_external_outgoing_text(L)].
-
-get_query_string_external([]) ->
-    <<>>;
-get_query_string_external(QsVals) ->
-    erlang:iolist_to_binary(get_query_string_external_text(QsVals)).
-
-get_query_string_external_text([] = L) ->
-    L;
-get_query_string_external_text([{K, V} | L]) ->
-    if
-        V =:= true ->
-            [[K, 0, <<"true">>, 0] | get_query_string_external_text(L)];
-        is_binary(V) ->
-            [[K, 0, V, 0] | get_query_string_external_text(L)]
+header_set_if_not(Key, Value, Headers) ->
+    case lists:keyfind(Key, 1, Headers) of
+        {_, _} ->
+            Headers;
+        false ->
+            [{Key, Value} | Headers]
     end.
 
+get_query_string_external(QsVals) ->
+    cloudi_request_info:key_value_new(QsVals, text_pairs).
+
 request_time_start() ->
     cloudi_timestamp:microseconds_monotonic().
 
@@ -458,7 +446,9 @@ handle_request(Name, Headers, Body, _Req,
     end.
 
 handle_response(NameIncoming, HeadersOutgoing0, Response,
-                _Req, OutputType, ContentTypeForced) ->
+                _Req, OutputType, ContentTypeForced,
+                SetXContentTypeOptions, SetXXSSProtection,
+                ContentSecurityPolicy, ContentSecurityPolicyReport) ->
     ResponseBinary = if
         (((OutputType =:= external) orelse
           (OutputType =:= internal)) andalso
@@ -470,7 +460,7 @@ handle_response(NameIncoming, HeadersOutgoing0, Response,
          is_list(Response)) ->
             erlang:list_to_binary(Response)
     end,
-    {HttpCode, HeadersOutgoingN} = case lists:keytake(<<"status">>, 1,
+    {HttpCode, HeadersOutgoing2} = case lists:keytake(<<"status">>, 1,
                                                       HeadersOutgoing0) of
         false ->
             {200, HeadersOutgoing0};
@@ -479,9 +469,9 @@ handle_response(NameIncoming, HeadersOutgoing0, Response,
             {erlang:binary_to_integer(hd(binary:split(Status, <<" ">>))),
              HeadersOutgoing1}
     end,
-    ResponseHeadersOutgoing = if
-        HeadersOutgoingN =/= [] ->
-            HeadersOutgoingN;
+    HeadersOutgoing3 = if
+        HeadersOutgoing2 =/= [] ->
+            HeadersOutgoing2;
         ContentTypeForced =/= undefined ->
             [{<<"content-type">>, ContentTypeForced}];
         true ->
@@ -510,7 +500,59 @@ handle_response(NameIncoming, HeadersOutgoing0, Response,
                     end
             end
     end,
-    HttpCode = 200,
+    {ContentTypeHTML,
+     ContentTypeSet} = case lists:keyfind(<<"content-type">>, 1,
+                                          HeadersOutgoing3) of
+        {_,  <<"text/html", _/binary>>} ->
+            {true, true};
+        {_,  <<_/binary>>} ->
+            {false, true};
+        false ->
+            {false, false}
+    end,
+    HeadersOutgoing4 = if
+        ContentTypeSet =:= true ->
+            if
+                SetXContentTypeOptions =:= true ->
+                    header_set_if_not(<<"X-Content-Type-Options">>,
+                                      <<"nosniff">>,
+                                      HeadersOutgoing3);
+                SetXContentTypeOptions =:= false ->
+                    HeadersOutgoing3
+            end;
+        ContentTypeSet =:= false ->
+            HeadersOutgoing3
+    end,
+    HeadersOutgoingN = if
+        ContentTypeHTML =:= true ->
+            HeadersOutgoing5 = if
+                SetXXSSProtection =:= true ->
+                    header_set_if_not(<<"X-XSS-Protection">>,
+                                      <<"0">>,
+                                      HeadersOutgoing4);
+                SetXXSSProtection =:= false ->
+                    HeadersOutgoing4
+            end,
+            HeadersOutgoing6 = if
+                is_binary(ContentSecurityPolicyReport) ->
+                    header_set_if_not(<<"content-security-policy-report-only">>,
+                                      ContentSecurityPolicyReport,
+                                      HeadersOutgoing5);
+                ContentSecurityPolicyReport =:= undefined ->
+                    HeadersOutgoing5
+            end,
+            if
+                is_binary(ContentSecurityPolicy) ->
+                    header_set_if_not(<<"content-security-policy">>,
+                                      ContentSecurityPolicy,
+                                      HeadersOutgoing6);
+                ContentSecurityPolicy =:= undefined ->
+                    HeadersOutgoing6
+            end;
+        ContentTypeHTML =:= false ->
+            HeadersOutgoing4
+    end,
+    ResponseHeadersOutgoing = HeadersOutgoingN,
     {HttpCode,
      ResponseHeadersOutgoing,
      ResponseBinary}.
