@@ -10,7 +10,7 @@
 %%%
 %%% MIT License
 %%%
-%%% Copyright (c) 2012-2021 Michael Truog <mjtruog at protonmail dot com>
+%%% Copyright (c) 2012-2022 Michael Truog <mjtruog at protonmail dot com>
 %%%
 %%% Permission is hereby granted, free of charge, to any person obtaining a
 %%% copy of this software and associated documentation files (the "Software"),
@@ -31,8 +31,8 @@
 %%% DEALINGS IN THE SOFTWARE.
 %%%
 %%% @author Michael Truog <mjtruog at protonmail dot com>
-%%% @copyright 2012-2021 Michael Truog
-%%% @version 2.0.3 {@date} {@time}
+%%% @copyright 2012-2022 Michael Truog
+%%% @version 2.0.5 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_service_map_reduce).
@@ -50,6 +50,7 @@
          cloudi_service_handle_info/3,
          cloudi_service_terminate/3]).
 
+-include_lib("cloudi_core/include/cloudi_constants.hrl").
 -include_lib("cloudi_core/include/cloudi_logger.hrl").
 -include_lib("cloudi_core/include/cloudi_service.hrl").
 
@@ -62,8 +63,8 @@
         %  service configuration).
 -define(DEFAULT_LOG_EXECUTION_TIME,                true).
 -define(DEFAULT_RETRY,                                3).
-        % Max retries with a Timeout value of timeout_max
-        % before cloudi_service_map_reduce_resend is called.
+        % Max retries with a Timeout value of ?TIMEOUT_MAX_ERLANG
+        % before exiting with retry_max.
         % The Timeout value needs to be provided in the map_send_args()
         % and is typically increasing due to service request failures.
 -define(DEFAULT_RETRY_DELAY,                          0). % milliseconds
@@ -97,8 +98,6 @@
             :: non_neg_integer(),
         retry_delay
             :: non_neg_integer(),
-        timeout_max
-            :: cloudi_service:timeout_value_milliseconds(),
         map_requests
             :: #{cloudi_service:trans_id() := #map_send{}},
         time_running
@@ -139,8 +138,6 @@
             :: non_neg_integer(),
         retry_delay
             :: non_neg_integer(),
-        timeout_max
-            :: cloudi_service:timeout_value_milliseconds(),
         time_start
             :: cloudi_timestamp:seconds_monotonic()
     }).
@@ -273,7 +270,7 @@ cloudi_service_init(Args, Prefix, Timeout, Dispatcher) ->
         {retry,                  ?DEFAULT_RETRY},
         {retry_delay,            ?DEFAULT_RETRY_DELAY}],
     [MapReduceModule, MapReduceArgs, Name, Concurrency, LogExecutionTime,
-     Retry, RetryDelay] =
+     Retry, RetryDelay0] =
         cloudi_proplists:take_values(Defaults, Args),
     TimeStart = cloudi_timestamp:seconds_monotonic(),
     true = is_atom(MapReduceModule) andalso (MapReduceModule /= undefined),
@@ -281,8 +278,10 @@ cloudi_service_init(Args, Prefix, Timeout, Dispatcher) ->
     true = is_number(Concurrency) andalso (Concurrency > 0),
     true = is_boolean(LogExecutionTime),
     true = is_integer(Retry) andalso (Retry >= 0),
-    true = is_integer(RetryDelay) andalso
-           (RetryDelay >= 0) andalso (RetryDelay =< 4294967295),
+    RetryDelayN = cloudi_args_type:
+                  period_to_milliseconds(RetryDelay0, 0, ?TIMEOUT_MAX_ERLANG),
+    true = ((Retry == 0) andalso (RetryDelayN == 0)) orelse
+           ((Retry > 0) andalso (RetryDelayN >= 0)),
     case application:load(MapReduceModule) of
         ok ->
             ok = cloudi_x_reltool_util:application_start(MapReduceModule,
@@ -293,7 +292,6 @@ cloudi_service_init(Args, Prefix, Timeout, Dispatcher) ->
         {error, _} ->
             ok = cloudi_x_reltool_util:module_loaded(MapReduceModule)
     end,
-    TimeoutMax = cloudi_service:timeout_max(Dispatcher),
     % cloudi_service_init/4 is always executed by the service process
     Service = self(),
     Service ! #init_begin{service = Service,
@@ -305,8 +303,7 @@ cloudi_service_init(Args, Prefix, Timeout, Dispatcher) ->
                           concurrency = Concurrency,
                           log_execution_time = LogExecutionTime,
                           retry = Retry,
-                          retry_delay = RetryDelay,
-                          timeout_max = TimeoutMax,
+                          retry_delay = RetryDelayN,
                           time_start = TimeStart},
     ok = cloudi_service:subscribe(Dispatcher, Name),
     {ok, #init_state{service = Service}}.
@@ -427,7 +424,6 @@ init(#init_begin{service = Service,
                  log_execution_time = LogExecutionTime,
                  retry = Retry,
                  retry_delay = RetryDelay,
-                 timeout_max = TimeoutMax,
                  time_start = TimeStart},
      Dispatcher) ->
     MapCount = cloudi_concurrency:count(Concurrency),
@@ -448,7 +444,6 @@ init(#init_begin{service = Service,
                             log_execution_time = LogExecutionTime,
                             retry = Retry,
                             retry_delay = RetryDelay,
-                            timeout_max = TimeoutMax,
                             map_requests = MapRequests,
                             time_running = TimeStart}};
                 {error, _} = Error ->
@@ -581,12 +576,11 @@ map_check_done(#state{map_requests = MapRequests} = State) ->
     end.
 
 retry(#map_send{send_args = [_Dispatcher, _Name, _Request,
-                             TimeoutMax],
+                             ?TIMEOUT_MAX_ERLANG],
                 retry_count = RetryCount} = MapRequest,
       #state{service = Service,
              retry = Retry,
-             retry_delay = RetryDelay,
-             timeout_max = TimeoutMax}) ->
+             retry_delay = RetryDelay}) ->
     if
         RetryCount < Retry ->
             retry_delay(RetryDelay, Service, MapRequest);
@@ -594,12 +588,11 @@ retry(#map_send{send_args = [_Dispatcher, _Name, _Request,
             erlang:exit(retry_max)
     end;
 retry(#map_send{send_args = [_Dispatcher, _Name, _Request,
-                             TimeoutMax, _PatternPid],
+                             ?TIMEOUT_MAX_ERLANG, _PatternPid],
                 retry_count = RetryCount} = MapRequest,
       #state{service = Service,
              retry = Retry,
-             retry_delay = RetryDelay,
-             timeout_max = TimeoutMax}) ->
+             retry_delay = RetryDelay}) ->
     if
         RetryCount < Retry ->
             retry_delay(RetryDelay, Service, MapRequest);
@@ -607,12 +600,11 @@ retry(#map_send{send_args = [_Dispatcher, _Name, _Request,
             erlang:exit(retry_max)
     end;
 retry(#map_send{send_args = [_Dispatcher, _Name, _RequestInfo, _Request,
-                             TimeoutMax, _Priority],
+                             ?TIMEOUT_MAX_ERLANG, _Priority],
                 retry_count = RetryCount} = MapRequest,
       #state{service = Service,
              retry = Retry,
-             retry_delay = RetryDelay,
-             timeout_max = TimeoutMax}) ->
+             retry_delay = RetryDelay}) ->
     if
         RetryCount < Retry ->
             retry_delay(RetryDelay, Service, MapRequest);
@@ -620,12 +612,11 @@ retry(#map_send{send_args = [_Dispatcher, _Name, _RequestInfo, _Request,
             erlang:exit(retry_max)
     end;
 retry(#map_send{send_args = [_Dispatcher, _Name, _RequestInfo, _Request,
-                             TimeoutMax, _Priority, _PatternPid],
+                             ?TIMEOUT_MAX_ERLANG, _Priority, _PatternPid],
                 retry_count = RetryCount} = MapRequest,
       #state{service = Service,
              retry = Retry,
-             retry_delay = RetryDelay,
-             timeout_max = TimeoutMax}) ->
+             retry_delay = RetryDelay}) ->
     if
         RetryCount < Retry ->
             retry_delay(RetryDelay, Service, MapRequest);
