@@ -8,7 +8,7 @@
 %%%
 %%% MIT License
 %%%
-%%% Copyright (c) 2015-2019 Michael Truog <mjtruog at protonmail dot com>
+%%% Copyright (c) 2015-2022 Michael Truog <mjtruog at protonmail dot com>
 %%%
 %%% Permission is hereby granted, free of charge, to any person obtaining a
 %%% copy of this software and associated documentation files (the "Software"),
@@ -29,8 +29,8 @@
 %%% DEALINGS IN THE SOFTWARE.
 %%%
 %%% @author Michael Truog <mjtruog at protonmail dot com>
-%%% @copyright 2015-2019 Michael Truog
-%%% @version 1.8.1 {@date} {@time}
+%%% @copyright 2015-2022 Michael Truog
+%%% @version 2.0.5 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_service_monitoring_cloudi).
@@ -104,10 +104,6 @@
         metrics = [] :: metric_list()
     }).
 
-% cloudi_core_i_service_internal and cloudi_core_i_service_external
-% use maps when available/stable
--define(MAP_SIZE(M),         maps:size(M)).
-
 %%%------------------------------------------------------------------------
 %%% External interface functions
 %%%------------------------------------------------------------------------
@@ -201,7 +197,7 @@ basic_update(ProcessInfo0) ->
 services_state(Timeout) ->
     try sys:get_state(cloudi_core_i_services_monitor, Timeout) of
         State ->
-            5 = erlang:tuple_size(State),
+            8 = erlang:tuple_size(State),
             state = erlang:element(1, State),
             {ok, erlang:element(2, State)}
     catch
@@ -214,16 +210,16 @@ services_init(undefined, ProcessInfo0, _, _, _, _) ->
 services_init(Interval, ProcessInfo0,
               MetricPrefix, UseAspectsOnly, Driver, EnvironmentLookup) ->
     {ok, Services} = services_state(Interval * 1000),
-    ets:new(?ETS_CONFIG,
-            [set, public, named_table,
-             {read_concurrency, true}]),
+    ?ETS_CONFIG = ets:new(?ETS_CONFIG,
+                          [set, public, named_table,
+                           {read_concurrency, true}]),
     true = ets:insert(?ETS_CONFIG, [{init, MetricPrefix, Driver}]),
-    ets:new(?ETS_PID2METRIC,
-            [set, public, named_table,
-             {read_concurrency, true}]),
-    ets:new(?ETS_REF2METRIC,
-            [set, public, named_table,
-             {read_concurrency, true}]),
+    ?ETS_PID2METRIC = ets:new(?ETS_PID2METRIC,
+                              [set, public, named_table,
+                               {read_concurrency, true}]),
+    ?ETS_REF2METRIC = ets:new(?ETS_REF2METRIC,
+                              [set, public, named_table,
+                               {read_concurrency, true}]),
     {InsertsN,
      ProcessInfoN} = cloudi_x_key2value:fold1(fun(_ID, Pids,
                                                   #service{} = Service, A) ->
@@ -406,7 +402,8 @@ services_update(ServicesOld, ServicesNew, ProcessInfo0, QueuedEmptySize,
             true = ets:insert(?ETS_PID2METRIC, InsertsN),
             _ = ets:select_delete(?ETS_PID2METRIC,
                                   [{{PidOld, '_', '_'},[],[true]}
-                                   || PidOld <- DeletesN])
+                                   || PidOld <- DeletesN]),
+            ok
     end,
     {services_metrics(CountInternal, CountExternal,
                       ConcurrencyInternal, ConcurrencyExternal,
@@ -580,7 +577,7 @@ service_process_metrics({ServiceMemory, ServiceMessages, ServiceReductionsNow},
                         is_pid(InfoPid) ->
                             process_info_update(InfoPid, ProcessInfo1)
                     end,
-                    {?MAP_SIZE(erlang:element(3, State)),  % send_timeouts
+                    {maps:size(erlang:element(3, State)),  % send_timeouts
                      erlang:element(10, State),            % queued
                      erlang:element(11, State),            % queued_size
                      erlang:element(12, State),            % queued_word_size
@@ -610,7 +607,7 @@ service_process_metrics({ServiceMemory, ServiceMessages, ServiceReductionsNow},
                         {ok, DispatcherState} -> % gen_server/proc_lib
                             31 = erlang:tuple_size(DispatcherState),
                             state = erlang:element(1, DispatcherState),
-                            ?MAP_SIZE(erlang:element(3, DispatcherState));
+                            maps:size(erlang:element(3, DispatcherState));
                         {error, _} ->
                             undefined
                     end,
@@ -745,7 +742,7 @@ service_process_metrics({ServiceMemory, ServiceMessages, ServiceReductionsNow},
         {ok, {_, State}} -> % gen_statem
             41 = erlang:tuple_size(State),
             state = erlang:element(1, State),
-            Outgoing = ?MAP_SIZE(erlang:element(3, State)),  % send_timeouts
+            Outgoing = maps:size(erlang:element(3, State)),  % send_timeouts
             QueuedRequests = erlang:element(10, State),      % queued
             QueuedRequestsSize0 = erlang:element(11, State), % queued_size
             WordSize = erlang:element(12, State),            % queued_word_size
@@ -1162,7 +1159,7 @@ aspect_terminate_before_external_f() ->
 aspect_log_f(OutputTime)
     when OutputTime =:= 'before'; OutputTime =:= 'after' ->
     fun(Level, _Timestamp, _Node, _Pid,
-        _Module, _Line, _Function, _Arity, _MetaData, _LogMessage) ->
+        _FileName, _Line, _Function, _Arity, _MetaData, _LogMessage) ->
         % only remote function calls here to allow a module reload
         case ?MODULE:aspect_cloudi() of
             {_, MetricPrefix, Driver} ->
