@@ -72,7 +72,7 @@
 %%%
 %%% MIT License
 %%%
-%%% Copyright (c) 2014-2019 Michael Truog <mjtruog at protonmail dot com>
+%%% Copyright (c) 2014-2022 Michael Truog <mjtruog at protonmail dot com>
 %%%
 %%% Permission is hereby granted, free of charge, to any person obtaining a
 %%% copy of this software and associated documentation files (the "Software"),
@@ -93,8 +93,8 @@
 %%% DEALINGS IN THE SOFTWARE.
 %%%
 %%% @author Michael Truog <mjtruog at protonmail dot com>
-%%% @copyright 2014-2019 Michael Truog
-%%% @version 1.8.0 {@date} {@time}
+%%% @copyright 2014-2022 Michael Truog
+%%% @version 2.0.5 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_service_queue).
@@ -108,6 +108,7 @@
          cloudi_service_handle_info/3,
          cloudi_service_terminate/3]).
 
+-include_lib("cloudi_core/include/cloudi_constants.hrl").
 -include_lib("cloudi_core/include/cloudi_logger.hrl").
 -include_lib("cloudi_core/include/cloudi_service.hrl").
 
@@ -148,7 +149,7 @@
         timeout :: cloudi_service:timeout_value_milliseconds(),
         priority :: cloudi_service:priority(),
         trans_id :: cloudi_service:trans_id(),
-        pid :: cloudi_service:source()
+        source :: cloudi_service:source()
     }).
 -type request_destination_mode() :: #destination_request{}.
 % fault_isolation: both
@@ -173,7 +174,7 @@
         trans_id :: cloudi_service:trans_id()
     }).
 -type request_both_mode() :: #both_request{} | #both_response{}.
-% cloudi_write_ahead_logging 
+% cloudi_write_ahead_logging
 -type request() :: request_destination_mode() |
                    request_both_mode().
 -export_type([request/0]).
@@ -206,7 +207,7 @@ cloudi_service_init(Args, Prefix, _Timeout, Dispatcher) ->
         {retry,                       ?DEFAULT_RETRY},
         {retry_delay,           ?DEFAULT_RETRY_DELAY},
         {fault_isolation,   ?DEFAULT_FAULT_ISOLATION}],
-    [FilePath, FileSizeLimit, Compression, Checksum, Retry, RetryDelay,
+    [FilePath, FileSizeLimit, Compression, Checksum, Retry, RetryDelay0,
      Mode] = cloudi_proplists:take_values(Defaults, Args),
     false = cloudi_service_name:pattern(Prefix),
     true = is_list(FilePath) andalso is_integer(hd(FilePath)),
@@ -220,10 +221,10 @@ cloudi_service_init(Args, Prefix, _Timeout, Dispatcher) ->
            (Checksum =:= sha224) orelse (Checksum =:= sha256) orelse
            (Checksum =:= sha384) orelse (Checksum =:= sha512),
     true = is_integer(Retry) andalso (Retry >= 0),
-    true = is_integer(RetryDelay) andalso
-           (RetryDelay >= 0) andalso (RetryDelay =< 4294967295),
-    true = ((Retry == 0) andalso (RetryDelay == 0)) orelse
-           ((Retry > 0) andalso (RetryDelay >= 0)),
+    RetryDelayN = cloudi_args_type:
+                  period_to_milliseconds(RetryDelay0, 0, ?TIMEOUT_MAX_ERLANG),
+    true = ((Retry == 0) andalso (RetryDelayN == 0)) orelse
+           ((Retry > 0) andalso (RetryDelayN >= 0)),
     true = ((Mode =:= destination) orelse (Mode =:= both)),
     I = erlang:integer_to_list(cloudi_service:process_index(Dispatcher)),
     Environment = cloudi_x_trie:store("I", I,
@@ -245,11 +246,11 @@ cloudi_service_init(Args, Prefix, _Timeout, Dispatcher) ->
                 logging = Logging,
                 mode = Mode,
                 retry = Retry,
-                retry_delay = RetryDelay,
+                retry_delay = RetryDelayN,
                 retry_f = RetryF}}.
 
 cloudi_service_handle_request(RequestType, Name, Pattern, RequestInfo, Request,
-                              Timeout, Priority, TransId, Pid,
+                              Timeout, Priority, TransId, Source,
                               #state{service = Service,
                                      file_size_limit = FileSizeLimit,
                                      logging = Logging,
@@ -264,7 +265,7 @@ cloudi_service_handle_request(RequestType, Name, Pattern, RequestInfo, Request,
                                         timeout = Timeout,
                                         priority = Priority,
                                         trans_id = TransId,
-                                        pid = Pid},
+                                        source = Source},
     case cloudi_write_ahead_logging:
          store_start(ChunkRequest, Logging) of
         {Chunk, NextLogging} ->
@@ -284,7 +285,7 @@ cloudi_service_handle_request(RequestType, Name, Pattern, RequestInfo, Request,
             {reply, <<>>, State}
     end;
 cloudi_service_handle_request(_RequestType, Name, Pattern, RequestInfo, Request,
-                              Timeout, Priority, TransId, _Pid,
+                              Timeout, Priority, TransId, _Source,
                               #state{service = Service,
                                      file_size_limit = FileSizeLimit,
                                      logging = Logging,
@@ -344,14 +345,14 @@ cloudi_service_handle_info(#return_async_active{response_info = ResponseInfo,
                           name = Name,
                           pattern = Pattern,
                           trans_id = TransId,
-                          pid = Pid},
+                          source = Source},
      NewLogging} = cloudi_write_ahead_logging:
                    erase(QueueTransId, Logging),
     % PERSISTENCE END:
     % at this point the service request is no longer persisted
     cloudi_service:return_nothrow(Dispatcher, RequestType, Name, Pattern,
                                   ResponseInfo, Response,
-                                  Timeout, TransId, Pid),
+                                  Timeout, TransId, Source),
     {noreply, State#state{logging = NewLogging}};
 cloudi_service_handle_info(#return_async_active{response_info = ResponseInfo,
                                                 response = Response,
@@ -452,11 +453,11 @@ retry(#destination_request{name = Name,
                            timeout = Timeout,
                            priority = Priority,
                            trans_id = TransId,
-                           pid = Pid}, true, destination,
+                           source = Source}, true, destination,
       Dispatcher, Service) ->
     Age = (cloudi_trans_id:microseconds() -
            cloudi_trans_id:microseconds(TransId)) div 1000 + 100, % milliseconds
-    case erlang:is_process_alive(Pid) of
+    case erlang:is_process_alive(Source) of
         false ->
             {error, timeout};
         true when Age >= Timeout ->
