@@ -8,7 +8,7 @@
 %%%
 %%% MIT License
 %%%
-%%% Copyright (c) 2011-2017 Michael Truog <mjtruog at gmail dot com>
+%%% Copyright (c) 2011-2021 Michael Truog <mjtruog at protonmail dot com>
 %%%
 %%% Permission is hereby granted, free of charge, to any person obtaining a
 %%% copy of this software and associated documentation files (the "Software"),
@@ -28,13 +28,13 @@
 %%% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 %%% DEALINGS IN THE SOFTWARE.
 %%%
-%%% @author Michael Truog <mjtruog [at] gmail (dot) com>
-%%% @copyright 2011-2017 Michael Truog
-%%% @version 1.7.1 {@date} {@time}
+%%% @author Michael Truog <mjtruog at protonmail dot com>
+%%% @copyright 2011-2021 Michael Truog
+%%% @version 2.0.2 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_x_supool_sup).
--author('mjtruog [at] gmail (dot) com').
+-author('mjtruog at protonmail dot com').
 
 -behaviour(supervisor).
 
@@ -75,8 +75,8 @@ start_link(Name, Count,
         {ok, _} ->
             PoolWorker = erlang:whereis(Name),
             true = is_pid(PoolWorker),
-            % needs to be the first message to the pool worker process to
-            % make sure pool processes exist before they are requested
+            % start needs to be the first message to the pool worker process
+            % to ensure pool processes exist before they are requested
             PoolWorker ! {start, ChildSpecs},
             Result;
         _ ->
@@ -110,7 +110,9 @@ init([Name, Options]) ->
     [MaxR, MaxT] = take_values(Defaults, Options),
     true = is_integer(MaxR) andalso (MaxR >= 0),
     true = is_integer(MaxT) andalso (MaxT >= 1),
-    {ok, {{one_for_one, MaxR, MaxT}, 
+    % brutal_kill is used for Shutdown due to usage of which_children/1
+    % (avoids blocking during supervisor shutdown)
+    {ok, {{one_for_one, MaxR, MaxT},
           [{cloudi_x_supool,
             {cloudi_x_supool, pool_worker_start_link, [Name, self()]},
             permanent, brutal_kill, worker, [cloudi_x_supool]}]}}.
