@@ -15,11 +15,18 @@
 %%% The Erlang pid is bitwise-XORed from 72 bits down to 32 bits.
 %%% The version 3 (MD5), version 4 (random), and version 5 (SHA)
 %%% methods are provided as specified within the RFC.
+%%%
+%%% The ordered version 1 variant is not present in the RFC,
+%%% though it is the most standards-compliant way of providing
+%%% timestamp UUID ordering.  Timestamp ordering has been used in many
+%%% non-standard UUID formats based on version 1 and typically limited to
+%%% the same data present in the version 1 UUID
+%%% (e.g., "Version 6" at http://gh.peabody.io/uuidv6/).
 %%% @end
 %%%
 %%% MIT License
 %%%
-%%% Copyright (c) 2011-2018 Michael Truog <mjtruog at protonmail dot com>
+%%% Copyright (c) 2011-2022 Michael Truog <mjtruog at protonmail dot com>
 %%%
 %%% Permission is hereby granted, free of charge, to any person obtaining a
 %%% copy of this software and associated documentation files (the "Software"),
@@ -40,8 +47,8 @@
 %%% DEALINGS IN THE SOFTWARE.
 %%%
 %%% @author Michael Truog <mjtruog at protonmail dot com>
-%%% @copyright 2011-2018 Michael Truog
-%%% @version 1.7.4 {@date} {@time}
+%%% @copyright 2011-2022 Michael Truog
+%%% @version 2.0.5 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_x_uuid).
@@ -86,9 +93,18 @@
 -define(ERLANG_OTP_VERSION_18_FEATURES, true).
 -ifdef(ERLANG_OTP_VERSION_18).
 -else.
+-define(ERLANG_OTP_VERSION_19_FEATURES, true).
 -ifdef(ERLANG_OTP_VERSION_19).
 -else.
 -define(ERLANG_OTP_VERSION_20_FEATURES, true).
+-ifdef(ERLANG_OTP_VERSION_20).
+-else.
+-ifdef(OTP_RELEASE). % Erlang/OTP >= 21.0
+% able to use -elif here
+-else.
+-error("Erlang/OTP version invalid").
+-endif.
+-endif.
 -endif.
 -endif.
 -endif.
@@ -99,9 +115,11 @@
 -else.
 -type timestamp_type_internal() :: 'erlang_now' | 'os'.
 -endif.
+-type v1_variant() :: 'rfc4122' | 'ordered'.
 
 -record(uuid_state,
     {
+        variant :: v1_variant(),
         node_id :: <<_:48>>,
         clock_seq :: 0..16383,
         timestamp_type :: timestamp_type_internal(),
@@ -115,8 +133,37 @@
               timestamp_type/0,
               state/0]).
 
+-ifdef(ERLANG_OTP_VERSION_19_FEATURES).
+-type iso8601() ::
+    nonempty_list($0..$9 | $T | $- | $: | $. | $Z).
+-type uuid_string_list() ::
+    nonempty_list($0..$9 | $a..$f | $-).
+-else.
+-type iso8601() ::
+    nonempty_string().
+-type uuid_string_list() ::
+    nonempty_string().
+-endif.
+-type uuid_string_binary() ::
+    <<_:256>> | <<_:288>>.
+-type uuid_string() :: uuid_string_list() | uuid_string_binary().
+-export_type([iso8601/0,
+              uuid_string_list/0,
+              uuid_string_binary/0,
+              uuid_string/0]).
+
 -include("cloudi_x_uuid.hrl").
 
+% Erlang Binary Term Format constants
+% info from http://erlang.org/doc/apps/erts/erl_ext_dist.html
+-define(TAG_VERSION, 131).
+-define(TAG_PID_EXT, 103).
+-define(TAG_NEW_PID_EXT, 88).
+
+% 16#01b21dd213814000 is the number of 100-ns intervals between the
+% UUID epoch 1582-10-15 00:00:00 and the UNIX epoch 1970-01-01 00:00:00.
+-define(GREGORIAN_EPOCH_OFFSET, 16#01b21dd213814000).
+
 %%%------------------------------------------------------------------------
 %%% External interface functions
 %%%------------------------------------------------------------------------
@@ -148,7 +195,8 @@ new(Pid) when is_pid(Pid) ->
 -spec new(Pid :: pid(),
           Options :: timestamp_type() |
                      list({timestamp_type, timestamp_type()} |
-                          {mac_address, list(non_neg_integer())})) ->
+                          {mac_address, list(non_neg_integer())} |
+                          {variant, v1_variant()})) ->
     state().
 
 new(Pid, TimestampType)
@@ -171,30 +219,58 @@ new(Pid, Options)
         false ->
             mac_address()
     end,
+    Variant = case lists:keyfind(variant, 1, Options) of
+        {variant, Value3} ->
+            Value3;
+        false ->
+            rfc4122
+    end,
 
     % make the version 1 UUID specific to the Erlang node and pid
 
-    % (when the pid format changes, handle the different format)
-    ExternalTermFormatVersion = 131,
-    PidExtType = 103,
-    <<ExternalTermFormatVersion:8,
-      PidExtType:8,
-      PidBin/binary>> = erlang:term_to_binary(Pid),
-    % 72 bits for the Erlang pid
-    <<PidID1:8, PidID2:8, PidID3:8, PidID4:8, % ID (Node specific, 15 bits)
-      PidSR1:8, PidSR2:8, PidSR3:8, PidSR4:8, % Serial (extra uniqueness)
-      PidCR1:8                                % Node Creation Count
-      >> = binary:part(PidBin, erlang:byte_size(PidBin), -9),
     % 48 bits for the first MAC address found is included with the
-    % distributed Erlang node name to create a node specific value in 16 bits
-    Node32 = cloudi_x_quickrand_hash:jenkins_32([MacAddress,
-                                        erlang:atom_to_binary(node(), utf8)],
-                                       PidCR1),
+    % distributed Erlang node name to be hashed with the node creation count
+    NodeData = [MacAddress, erlang:atom_to_binary(node(), utf8)],
+
+    % Reduce the node information to 16 bits and the pid information to 32 bits
+    % for use as the UUID v1 Node Id
+    {NodeCreationCount, PidData} = case erlang:term_to_binary(Pid) of
+        % (when the pid format changes, handle the different formats)
+        <<?TAG_VERSION:8,
+          ?TAG_PID_EXT:8,PidBin/binary>> ->
+            % 72 bits for the Erlang pid
+            <<PidID1:8,PidID2:8,PidID3:8,PidID4:8,% ID (Node specific, 15 bits)
+              PidSR1:8,PidSR2:8,PidSR3:8,PidSR4:8,% Serial (extra uniqueness)
+              PidCR:8                             % Node Creation Count
+              >> = binary:part(PidBin, erlang:byte_size(PidBin), -9),
+            {PidCR,
+             [PidID1, PidID2, PidID3, PidID4,
+              PidSR1, PidSR2, PidSR3, PidSR4]};
+        % format supported in Erlang/OTP 19.0-rc1
+        % required for Erlang/OTP 23.0 (and Erlang/OTP 22.0-rc2)
+        <<?TAG_VERSION:8,
+          ?TAG_NEW_PID_EXT:8,PidBin/binary>> ->
+            % 96 bits for the Erlang pid
+            <<PidID1:8,PidID2:8,PidID3:8,PidID4:8,% ID (Node specific, 15 bits)
+              PidSR1:8,PidSR2:8,PidSR3:8,PidSR4:8,% Serial (extra uniqueness)
+              PidCR:32                            % Node Creation Count
+              >> = binary:part(PidBin, erlang:byte_size(PidBin), -12),
+            {PidCR,
+             [PidID1, PidID2, PidID3, PidID4,
+              PidSR1, PidSR2, PidSR3, PidSR4]}
+    end,
+    Node32 = cloudi_x_quickrand_hash:jenkins_32(NodeData, NodeCreationCount),
     Node16 = (Node32 bsr 16) bxor (Node32 band 16#FFFF),
-    % reduce the Erlang pid to 32 bits
-    Pid32 = cloudi_x_quickrand_hash:jenkins_32([PidID1, PidID2, PidID3, PidID4,
-                                       PidSR1, PidSR2, PidSR3, PidSR4]),
-    ClockSeq = pseudo_random(16384) - 1,
+    Pid32 = cloudi_x_quickrand_hash:jenkins_32(PidData),
+    NodeId = <<Node16:16/big-unsigned-integer,
+               Pid32:32/big-unsigned-integer>>,
+
+    ClockSeq = if
+        Variant =:= rfc4122 ->
+            pseudo_random(16384) - 1;
+        Variant =:= ordered ->
+            pseudo_random(8192) - 1
+    end,
     TimestampTypeInternal = if
         TimestampType =:= os ->
             os;
@@ -211,8 +287,8 @@ new(Pid, Options)
             end
     end,
     TimestampLast = timestamp(TimestampTypeInternal),
-    #uuid_state{node_id = <<Node16:16/big-unsigned-integer,
-                            Pid32:32/big-unsigned-integer>>,
+    #uuid_state{variant = Variant,
+                node_id = NodeId,
                 clock_seq = ClockSeq,
                 timestamp_type = TimestampTypeInternal,
                 timestamp_last = TimestampLast}.
@@ -224,16 +300,15 @@ new(Pid, Options)
 %%-------------------------------------------------------------------------
 
 -spec get_v1(State :: state()) ->
-    {cloudi_x_uuid(), NewState :: state()}.
+    {cloudi_x_uuid(), StateNew :: state()}.
 
-get_v1(#uuid_state{node_id = NodeId,
+get_v1(#uuid_state{variant = rfc4122,
+                   node_id = NodeId,
                    clock_seq = ClockSeq,
                    timestamp_type = TimestampTypeInternal,
                    timestamp_last = TimestampLast} = State) ->
     MicroSeconds = timestamp(TimestampTypeInternal, TimestampLast),
-    % 16#01b21dd213814000 is the number of 100-ns intervals between the
-    % UUID epoch 1582-10-15 00:00:00 and the UNIX epoch 1970-01-01 00:00:00.
-    Time = MicroSeconds * 10 + 16#01b21dd213814000,
+    Time = MicroSeconds * 10 + ?GREGORIAN_EPOCH_OFFSET,
     % will be larger than 60 bits after 5236-03-31 21:21:00
     <<TimeHigh:12, TimeMid:16, TimeLow:32>> = <<Time:60>>,
     {<<TimeLow:32, TimeMid:16,
@@ -242,12 +317,29 @@ get_v1(#uuid_state{node_id = NodeId,
        1:1, 0:1,            % RFC 4122 variant bits
        ClockSeq:14,
        NodeId/binary>>,
+     State#uuid_state{timestamp_last = MicroSeconds}};
+get_v1(#uuid_state{variant = ordered,
+                   node_id = NodeId,
+                   clock_seq = ClockSeq,
+                   timestamp_type = TimestampTypeInternal,
+                   timestamp_last = TimestampLast} = State) ->
+    MicroSeconds = timestamp(TimestampTypeInternal, TimestampLast),
+    Time = MicroSeconds * 10 + ?GREGORIAN_EPOCH_OFFSET,
+    % will be larger than 60 bits after 5236-03-31 21:21:00
+    <<TimeHigh:48, TimeLow:12>> = <<Time:60>>,
+    {<<TimeHigh:48,
+       0:1, 0:1, 0:1, 1:1,  % version 1 bits
+       TimeLow:12,
+       1:1, 1:1, 1:1,       % ordered (future definition) variant bits
+       ClockSeq:13,
+       NodeId/binary>>,
      State#uuid_state{timestamp_last = MicroSeconds}}.
 
 %%-------------------------------------------------------------------------
 %% @doc
 %% ===Get the current time value in a manner consistent with the v1 UUID.===
-%% The result is an integer in microseconds.
+%% The result is an integer in microseconds since the UNIX epoch.
+%% (The UNIX epoch is 1970-01-01T00:00:00Z)
 %% @end
 %%-------------------------------------------------------------------------
 
@@ -260,7 +352,8 @@ get_v1_time() ->
 %%-------------------------------------------------------------------------
 %% @doc
 %% ===Get the current time value in a manner consistent with the v1 UUID.===
-%% The result is an integer in microseconds.
+%% The result is an integer in microseconds since the UNIX epoch.
+%% (The UNIX epoch is 1970-01-01T00:00:00Z)
 %% @end
 %%-------------------------------------------------------------------------
 
@@ -282,14 +375,23 @@ get_v1_time(#uuid_state{timestamp_type = TimestampTypeInternal}) ->
 
 get_v1_time(Value)
     when is_binary(Value), byte_size(Value) == 16 ->
-    <<TimeLow:32, TimeMid:16,
-      0:1, 0:1, 0:1, 1:1,  % version 1 bits
-      TimeHigh:12,
-      1:1, 0:1,            % RFC 4122 variant bits
-      _:14,
-      _:48>> = Value,
-    <<Time:60>> = <<TimeHigh:12, TimeMid:16, TimeLow:32>>,
-    ((Time - 16#01b21dd213814000) div 10). % microseconds since UNIX epoch
+    <<Time:60>> = case Value of
+        <<TimeLow:32, TimeMid:16,
+          0:1, 0:1, 0:1, 1:1,  % version 1 bits
+          TimeHigh:12,
+          1:1, 0:1,            % RFC 4122 variant bits
+          _:14,
+          _:48>> ->
+            <<TimeHigh:12, TimeMid:16, TimeLow:32>>;
+        <<TimeHigh:48,
+          0:1, 0:1, 0:1, 1:1,  % version 1 bits
+          TimeLow:12,
+          1:1, 1:1, 1:1,       % ordered (future definition) variant bits
+          _:13,
+          _:48>> ->
+            <<TimeHigh:48, TimeLow:12>>
+    end,
+    (Time - ?GREGORIAN_EPOCH_OFFSET) div 10.
 -else.
 get_v1_time(erlang) ->
     timestamp(erlang_now);
@@ -305,14 +407,23 @@ get_v1_time(#uuid_state{timestamp_type = TimestampTypeInternal}) ->
 
 get_v1_time(Value)
     when is_binary(Value), byte_size(Value) == 16 ->
-    <<TimeLow:32, TimeMid:16,
-      0:1, 0:1, 0:1, 1:1,  % version 1 bits
-      TimeHigh:12,
-      1:1, 0:1,            % RFC 4122 variant bits
-      _:14,
-      _:48>> = Value,
-    <<Time:60>> = <<TimeHigh:12, TimeMid:16, TimeLow:32>>,
-    ((Time - 16#01b21dd213814000) div 10). % microseconds since UNIX epoch
+    <<Time:60>> = case Value of
+        <<TimeLow:32, TimeMid:16,
+          0:1, 0:1, 0:1, 1:1,  % version 1 bits
+          TimeHigh:12,
+          1:1, 0:1,            % RFC 4122 variant bits
+          _:14,
+          _:48>> ->
+            <<TimeHigh:12, TimeMid:16, TimeLow:32>>;
+        <<TimeHigh:48,
+          0:1, 0:1, 0:1, 1:1,  % version 1 bits
+          TimeLow:12,
+          1:1, 1:1, 1:1,       % ordered (future definition) variant bits
+          _:13,
+          _:48>> ->
+            <<TimeHigh:48, TimeLow:12>>
+    end,
+    (Time - ?GREGORIAN_EPOCH_OFFSET) div 10.
 -endif.
 
 %%-------------------------------------------------------------------------
@@ -324,7 +435,7 @@ get_v1_time(Value)
 
 -spec get_v1_datetime(Value :: timestamp_type() | state() | cloudi_x_uuid() |
                                erlang:timestamp()) ->
-    string().
+    iso8601().
 
 get_v1_datetime({_, _, MicroSeconds} = Timestamp) ->
     {{DateYYYY, DateMM, DateDD},
@@ -358,7 +469,7 @@ get_v1_datetime(Value) ->
 -spec get_v1_datetime(Value :: timestamp_type() | state() | cloudi_x_uuid() |
                                erlang:timestamp(),
                       MicroSecondsOffset :: integer()) ->
-    string().
+    iso8601().
 
 get_v1_datetime(Value, MicroSecondsOffset)
     when is_integer(MicroSecondsOffset) ->
@@ -384,6 +495,13 @@ is_v1(<<_TimeLow:32, _TimeMid:16,
         _ClockSeq:14,
         _NodeId:48>>) ->
     true;
+is_v1(<<_TimeHigh:48,
+        0:1, 0:1, 0:1, 1:1,  % version 1 bits
+        _TimeLow:12,
+        1:1, 1:1, 1:1,       % ordered (future definition) variant bits
+        _ClockSeq:13,
+        _NodeId:48>>) ->
+    true;
 is_v1(_) ->
     false.
 
@@ -414,7 +532,7 @@ get_v3(Data) when is_binary(Data) ->
 %%-------------------------------------------------------------------------
 
 -spec get_v3(Namespace :: dns | url | oid | x500 | binary(),
-             Data :: binary() | iolist()) ->
+             Data :: iodata()) ->
     cloudi_x_uuid().
 
 get_v3(dns, Data) ->
@@ -426,13 +544,7 @@ get_v3(oid, Data) ->
 get_v3(x500, Data) ->
     get_v3(?UUID_NAMESPACE_X500, Data);
 get_v3(Namespace, Data) when is_binary(Namespace) ->
-    DataBin = if
-        is_binary(Data) ->
-            Data;
-        is_list(Data) ->
-            erlang:iolist_to_binary(Data)
-    end,
-    get_v3(<<Namespace/binary, DataBin/binary>>).
+    get_v3(erlang:iolist_to_binary([Namespace, Data])).
 
 %%-------------------------------------------------------------------------
 %% @doc
@@ -464,7 +576,7 @@ get_v3_compat(Data) when is_binary(Data) ->
 %%-------------------------------------------------------------------------
 
 -spec get_v3_compat(Namespace :: dns | url | oid | x500 | binary(),
-                    Data :: binary() | iolist()) ->
+                    Data :: iodata()) ->
     cloudi_x_uuid().
 
 get_v3_compat(dns, Data) ->
@@ -476,13 +588,7 @@ get_v3_compat(oid, Data) ->
 get_v3_compat(x500, Data) ->
     get_v3_compat(?UUID_NAMESPACE_X500, Data);
 get_v3_compat(Namespace, Data) when is_binary(Namespace) ->
-    DataBin = if
-        is_binary(Data) ->
-            Data;
-        is_list(Data) ->
-            erlang:iolist_to_binary(Data)
-    end,
-    get_v3_compat(<<Namespace/binary, DataBin/binary>>).
+    get_v3_compat(erlang:iolist_to_binary([Namespace, Data])).
 
 %%-------------------------------------------------------------------------
 %% @doc
@@ -536,23 +642,19 @@ get_v4(cached) ->
       1:1, 0:1,            % RFC 4122 variant bits
       Rand3:62>>;
 get_v4(Cache) when element(1, Cache) =:= cloudi_x_quickrand_cache ->
-    {<<Rand1:48, _:4, Rand2:12, _:2, Rand3:62>>, NewCache} =
+    {<<Rand1:48, _:4, Rand2:12, _:2, Rand3:62>>, CacheNew} =
         cloudi_x_quickrand_cache:rand_bytes(16, Cache),
     {<<Rand1:48,
        0:1, 1:1, 0:1, 0:1,  % version 4 bits
        Rand2:12,
        1:1, 0:1,            % RFC 4122 variant bits
-       Rand3:62>>, NewCache}.
+       Rand3:62>>, CacheNew}.
 
 %%-------------------------------------------------------------------------
 %% @doc
-%% ===Get a v4 UUID (using Wichmann-Hill 2006).===
-%% cloudi_x_random_wh06_int:uniform/1 repeats every 2.66e36 (2^121) approx.
-%% (see B.A. Wichmann and I.D.Hill, in
-%%  'Generating good pseudo-random numbers',
-%%  Computational Statistics and Data Analysis 51 (2006) 1614-1622)
-%% a single cloudi_x_random_wh06_int:uniform/1 call can provide a maximum of 124 bits
-%% (see cloudi_x_random_wh06_int.erl for details)
+%% ===Get a v4 UUID (using a 256-bit Marsaglia multiply-with-carry PRNG).===
+%% cloudi_x_quickrand:mwc256_128/1 repeats every 5.79e76 (2^255) approx.
+%% (see cloudi_x_quickrand.erl for details)
 %% @end
 %%-------------------------------------------------------------------------
 
@@ -561,7 +663,7 @@ get_v4(Cache) when element(1, Cache) =:= cloudi_x_quickrand_cache ->
 
 get_v4_urandom() ->
     % random 122 bits
-    Rand = cloudi_x_random_wh06_int:uniform(5316911983139663491615228241121378304) - 1,
+    Rand = cloudi_x_quickrand:mwc256_128(16#4000000000000000000000000000000) - 1,
     <<Rand1:48, Rand2:12, Rand3:62>> = <<Rand:122>>,
     <<Rand1:48,
       0:1, 1:1, 0:1, 0:1,  % version 4 bits
@@ -614,7 +716,7 @@ get_v5(Data) when is_binary(Data) ->
 %%-------------------------------------------------------------------------
 
 -spec get_v5(Namespace :: dns | url | oid | x500 | binary(),
-             Data :: binary() | iolist()) ->
+             Data :: iodata()) ->
     cloudi_x_uuid().
 
 get_v5(dns, Data) ->
@@ -626,13 +728,7 @@ get_v5(oid, Data) ->
 get_v5(x500, Data) ->
     get_v5(?UUID_NAMESPACE_X500, Data);
 get_v5(Namespace, Data) when is_binary(Namespace) ->
-    DataBin = if
-        is_binary(Data) ->
-            Data;
-        is_list(Data) ->
-            erlang:iolist_to_binary(Data)
-    end,
-    get_v5(<<Namespace/binary, DataBin/binary>>).
+    get_v5(erlang:iolist_to_binary([Namespace, Data])).
 
 %%-------------------------------------------------------------------------
 %% @doc
@@ -664,7 +760,7 @@ get_v5_compat(Data) when is_binary(Data) ->
 %%-------------------------------------------------------------------------
 
 -spec get_v5_compat(Namespace :: dns | url | oid | x500 | binary(),
-                    Data :: binary() | iolist()) ->
+                    Data :: iodata()) ->
     cloudi_x_uuid().
 
 get_v5_compat(dns, Data) ->
@@ -676,13 +772,7 @@ get_v5_compat(oid, Data) ->
 get_v5_compat(x500, Data) ->
     get_v5_compat(?UUID_NAMESPACE_X500, Data);
 get_v5_compat(Namespace, Data) when is_binary(Namespace) ->
-    DataBin = if
-        is_binary(Data) ->
-            Data;
-        is_list(Data) ->
-            erlang:iolist_to_binary(Data)
-    end,
-    get_v5_compat(<<Namespace/binary, DataBin/binary>>).
+    get_v5_compat(erlang:iolist_to_binary([Namespace, Data])).
 
 %%-------------------------------------------------------------------------
 %% @doc
@@ -709,7 +799,7 @@ is_v5(_) ->
 %%-------------------------------------------------------------------------
 
 -spec uuid_to_list(cloudi_x_uuid()) ->
-    iolist().
+    nonempty_list(non_neg_integer()).
 
 uuid_to_list(<<B1:32/unsigned-integer,
                B2:16/unsigned-integer,
@@ -725,7 +815,7 @@ uuid_to_list(<<B1:32/unsigned-integer,
 %%-------------------------------------------------------------------------
 
 -spec uuid_to_string(Value :: cloudi_x_uuid()) ->
-    string().
+    uuid_string_list().
 
 uuid_to_string(Value) ->
     uuid_to_string(Value, standard).
@@ -740,7 +830,7 @@ uuid_to_string(Value) ->
                      Option :: standard | nodash |
                                list_standard | list_nodash |
                                binary_standard | binary_nodash) ->
-    string() | binary().
+    uuid_string().
 
 uuid_to_string(<<Value:128/unsigned-integer>>, standard) ->
     [N01, N02, N03, N04, N05, N06, N07, N08,
@@ -796,7 +886,7 @@ uuid_to_string(<<Value:128/unsigned-integer>>, binary_nodash) ->
 %% @end
 %%-------------------------------------------------------------------------
 
--spec string_to_uuid(string() | binary()) ->
+-spec string_to_uuid(uuid_string()) ->
     cloudi_x_uuid().
 
 string_to_uuid([N01, N02, N03, N04, N05, N06, N07, N08, $-,
@@ -899,6 +989,12 @@ is_uuid(<<_:48,
     (Version == 3) orelse
     (Version == 4) orelse
     (Version == 5);
+is_uuid(<<_:48,
+          Version:4/unsigned-integer,
+          _:12,
+          1:1, 1:1, 1:1,       % ordered (future definition) variant bits
+          _:61>>) ->
+    (Version == 1);
 is_uuid(_) ->
     false.
 
@@ -921,18 +1017,37 @@ increment(<<TimeLow:32, TimeMid:16,
             1:1, 0:1,            % RFC 4122 variant bits
             ClockSeq:14,
             NodeId:48>>) ->
-    NextClockSeq = ClockSeq + 1,
-    NewClockSeq = if
-        NextClockSeq == 16384 ->
+    ClockSeqNext = ClockSeq + 1,
+    ClockSeqNew = if
+        ClockSeqNext == 16384 ->
             0;
         true ->
-            NextClockSeq
+            ClockSeqNext
     end,
     <<TimeLow:32, TimeMid:16,
       0:1, 0:1, 0:1, 1:1,  % version 1 bits
       TimeHigh:12,
       1:1, 0:1,            % RFC 4122 variant bits
-      NewClockSeq:14,
+      ClockSeqNew:14,
+      NodeId:48>>;
+increment(<<TimeHigh:48,
+            0:1, 0:1, 0:1, 1:1,  % version 1 bits
+            TimeLow:12,
+            1:1, 1:1, 1:1,       % ordered (future definition) variant bits
+            ClockSeq:13,
+            NodeId:48>>) ->
+    ClockSeqNext = ClockSeq + 1,
+    ClockSeqNew = if
+        ClockSeqNext == 8192 ->
+            0;
+        true ->
+            ClockSeqNext
+    end,
+    <<TimeHigh:48,
+      0:1, 0:1, 0:1, 1:1,  % version 1 bits
+      TimeLow:12,
+      1:1, 1:1, 1:1,       % ordered (future definition) variant bits
+      ClockSeqNew:13,
       NodeId:48>>;
 increment(<<Rand1:48,
             Version:4/unsigned-integer,
@@ -944,31 +1059,33 @@ increment(<<Rand1:48,
                                                     Rand2:12,
                                                     Rand3:62,
                                                     0:6>>,
-    NextValue = Value + 1,
-    NewValue = if
-        NextValue == 5316911983139663491615228241121378304 ->
+    ValueNext = Value + 1,
+    ValueNew = if
+        ValueNext == 5316911983139663491615228241121378304 ->
             1;
         true ->
-            NextValue
+            ValueNext
     end,
-    <<NewRand1:48,
-      NewRand2:12,
-      NewRand3:62,
-      0:6>> = <<NewValue:16/little-unsigned-integer-unit:8>>,
-    <<NewRand1:48,
+    <<Rand1New:48,
+      Rand2New:12,
+      Rand3New:62,
+      0:6>> = <<ValueNew:16/little-unsigned-integer-unit:8>>,
+    <<Rand1New:48,
       Version:4/unsigned-integer,
-      NewRand2:12,
+      Rand2New:12,
       1:1, 0:1,            % RFC 4122 variant bits
-      NewRand3:62>>;
-increment(#uuid_state{clock_seq = ClockSeq} = State) ->
-    NextClockSeq = ClockSeq + 1,
-    NewClockSeq = if
-        NextClockSeq == 16384 ->
+      Rand3New:62>>;
+increment(#uuid_state{variant = Variant,
+                      clock_seq = ClockSeq} = State) ->
+    ClockSeqNext = ClockSeq + 1,
+    ClockSeqNew = if
+        (Variant =:= rfc4122) andalso (ClockSeqNext == 16384);
+        (Variant =:= ordered) andalso (ClockSeqNext == 8192) ->
             0;
         true ->
-            NextClockSeq
+            ClockSeqNext
     end,
-    State#uuid_state{clock_seq = NewClockSeq}.
+    State#uuid_state{clock_seq = ClockSeqNew}.
 
 %%-------------------------------------------------------------------------
 %% @doc
@@ -977,7 +1094,7 @@ increment(#uuid_state{clock_seq = ClockSeq} = State) ->
 %%-------------------------------------------------------------------------
 
 -spec mac_address() ->
-    list(non_neg_integer()).
+    nonempty_list(non_neg_integer()).
 
 mac_address() ->
     {ok, Ifs} = inet:getifaddrs(),
@@ -1090,6 +1207,28 @@ test() ->
     V1uuidL1 = [V1uuid11, V1uuid9, V1uuid8, V1uuid7,
                 V1uuid6, V1uuid10, V1uuid5],
     true = V1uuidL0 == lists:usort(V1uuidL1),
+    % version 1 ordered variant
+    V1uuidOrdered0 = cloudi_x_uuid:string_to_uuid("1ea55423c2f912f6e847e7a9663898ef"),
+    V1uuidOrdered1 = cloudi_x_uuid:string_to_uuid("1ea554248aa31464e847e7a9663898ef"),
+    true = V1uuidOrdered1 > V1uuidOrdered0,
+    true = cloudi_x_uuid:is_v1(V1uuidOrdered0),
+    true = cloudi_x_uuid:is_v1(V1uuidOrdered1),
+    V1uuidOrdered2 = cloudi_x_uuid:increment(V1uuidOrdered1),
+    true = cloudi_x_uuid:is_v1(V1uuidOrdered2),
+    <<_:48,
+      0:1, 0:1, 0:1, 1:1,  % version 1 bits
+      _:12,
+      1:1, 1:1, 1:1,       % ordered (future definition) variant bits
+      _:13,
+      _:48>> = V1uuidOrdered2,
+    V1OrderedState0 = cloudi_x_uuid:new(self(), [{variant, ordered}]),
+    {V1uuidOrdered3, _} = cloudi_x_uuid:get_v1(V1OrderedState0),
+    <<_:48,
+      0:1, 0:1, 0:1, 1:1,  % version 1 bits
+      _:12,
+      1:1, 1:1, 1:1,       % ordered (future definition) variant bits
+      _:13,
+      _:48>> = V1uuidOrdered3,
 
     % version 3 tests
     % $ python
@@ -1354,10 +1493,16 @@ pseudo_random(N) ->
 -ifdef(TEST).
 -include_lib("eunit/include/eunit.hrl").
 
-internal_test_() ->
-    [
+-ifdef(CLOUDI_TEST_TIMEOUT).
+-define(TEST_TIMEOUT, ?CLOUDI_TEST_TIMEOUT). % seconds
+-else.
+-define(TEST_TIMEOUT, 10). % seconds
+-endif.
+
+module_test_() ->
+    {timeout, ?TEST_TIMEOUT, [
         {"internal tests", ?_assertEqual(ok, test())}
-    ].
+    ]}.
 
 -endif.
 
