@@ -4,7 +4,7 @@
 #
 # MIT License
 #
-# Copyright (c) 2011-2021 Michael Truog <mjtruog at protonmail dot com>
+# Copyright (c) 2011-2022 Michael Truog <mjtruog at protonmail dot com>
 #
 # Permission is hereby granted, free of charge, to any person obtaining a
 # copy of this software and associated documentation files (the "Software"),
@@ -68,7 +68,7 @@ class Task(threading.Thread):
         print('terminate http_req %s' % self.__name)
 
     def __request(self, request_type, name, pattern, request_info, request,
-                  timeout, priority, trans_id, pid):
+                  timeout, priority, trans_id, source):
         # pylint: disable=unused-argument
         # pylint: disable=too-many-arguments
         http_qs = API.info_key_value_parse(request)
@@ -86,7 +86,7 @@ class Task(threading.Thread):
         })
         self.__api.return_(request_type, name, pattern,
                            response_info, response.encode('utf-8'),
-                           timeout, trans_id, pid)
+                           timeout, trans_id, source)
 
 def _main():
     thread_count = API.thread_count()
