@@ -4,7 +4,7 @@
 //
 // MIT License
 //
-// Copyright (c) 2014-2017 Michael Truog <mjtruog at gmail dot com>
+// Copyright (c) 2014-2022 Michael Truog <mjtruog at protonmail dot com>
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software"),
@@ -29,21 +29,23 @@ require 'CloudI.php';
 
 define('DESTINATION', '/tests/msg_size/erlang');
 
-class Task //extends \Thread
+class Task
 {
     private $api;
+    private $thread_index;
 
-    public function __construct($api)
+    public function __construct($thread_index)
     {
-        $this->api = $api;
+        $this->api = null;
+        $this->thread_index = $thread_index;
     }
 
     public function run()
     {
         try
         {
+            $this->api = new \CloudI\API($this->thread_index);
             $this->api->subscribe('php', $this, 'request');
-
             $result = $this->api->poll();
             assert($result === false);
         }
@@ -59,10 +61,10 @@ class Task //extends \Thread
 
     public function request($request_type, $name, $pattern,
                             $request_info, $request,
-                            $timeout, $priority, $trans_id, $pid)
+                            $timeout, $priority, $trans_id, $source)
     {
         list(, $i) = unpack('L', substr($request, 0, 4));
-        if ($i == 4294967295)
+        if ($i == 1073741823)
             $i = 0;
         else
             $i++;
@@ -71,30 +73,36 @@ class Task //extends \Thread
              " (with timeout {$timeout} ms)\n";
         $this->api->forward_($request_type, DESTINATION,
                              $request_info, $request,
-                             $timeout, $priority, $trans_id, $pid);
+                             $timeout, $priority, $trans_id, $source);
     }
 }
 
 $thread_count = \CloudI\API::thread_count();
 assert($thread_count == 1);
-$main_thread = new Task(new \CloudI\API(0));
+$main_thread = new Task(0);
 $main_thread->run();
 
 /*
-// commented out due to PHP threads not having
+// commented out due to PHP ZTS (Zend Thread Safety) not having
 // readily available installation packages
+assert(PHP_ZTS == 1);
+use parallel\Runtime;
+
 $thread_count = \CloudI\API::thread_count();
 assert($thread_count >= 1);
-    
+
 $threads = array();
-for ($i = 0; $i < $thread_count; $i++)
+for ($thread_index = 0; $thread_index < $thread_count; $thread_index++)
 {
-    $threads[$i] = new Task(new \CloudI\API($i));
+    $t = new Runtime();
+    $t->run(function () use ($thread_index) {
+        $task = new Task($thread_index);
+        $task->run();
+    });
+    $threads[$thread_index] = $t;
 }
 foreach ($threads as $t)
-    $t->start();
-foreach ($threads as $t)
-    $t->join();
+    $t->close();
 */
 
 ?>
