@@ -4,7 +4,7 @@
 #
 # MIT License
 #
-# Copyright (c) 2011-2020 Michael Truog <mjtruog at protonmail dot com>
+# Copyright (c) 2011-2022 Michael Truog <mjtruog at protonmail dot com>
 #
 # Permission is hereby granted, free of charge, to any person obtaining a
 # copy of this software and associated documentation files (the "Software"),
@@ -43,9 +43,10 @@ class Task(threading.Thread):
     """
     api_name = None
 
-    def __init__(self, api, terminate):
+    def __init__(self, thread_index, terminate):
         threading.Thread.__init__(self)
-        self.__api = api
+        self.__api = None
+        self.__thread_index = thread_index
         self.__terminate_exception = terminate
 
     def run(self):
@@ -53,6 +54,7 @@ class Task(threading.Thread):
         run the msg_size thread
         """
         try:
+            self.__api = API(self.__thread_index)
             self.__api.subscribe(Task.api_name, _request)
 
             result = self.__api.poll()
@@ -64,11 +66,11 @@ class Task(threading.Thread):
         print('terminate msg_size %s' % Task.api_name)
 
 def _request(api, request_type, name, pattern, request_info, request,
-             timeout, priority, trans_id, pid):
+             timeout, priority, trans_id, source):
     # pylint: disable=unused-argument
     # pylint: disable=too-many-arguments
     i = struct.unpack('=I', request[:4])[0]
-    if i == 4294967295:
+    if i == 1073741823:
         i = 0
     else:
         i += 1
@@ -77,14 +79,14 @@ def _request(api, request_type, name, pattern, request_info, request,
         i, Task.api_name, _DESTINATION, timeout,
     ))
     api.forward_(request_type, _DESTINATION, request_info, request,
-                 timeout, priority, trans_id, pid)
+                 timeout, priority, trans_id, source)
 
 def _main():
     thread_count = API.thread_count()
     assert thread_count >= 1
 
     Task.api_name = 'python'
-    threads = [Task(API(thread_index), TerminateException)
+    threads = [Task(thread_index, TerminateException)
                for thread_index in range(thread_count)]
     for thread in threads:
         thread.start()
