@@ -3,7 +3,7 @@
 //
 // MIT License
 //
-// Copyright (c) 2017 Michael Truog <mjtruog at gmail dot com>
+// Copyright (c) 2017-2022 Michael Truog <mjtruog at protonmail dot com>
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software"),
@@ -55,7 +55,7 @@ Task.prototype.run = function () {
 };
 Task.prototype.request = function (request_type, name, pattern,
                                    request_info, request,
-                                   timeout, priority, trans_id, pid) {
+                                   timeout, priority, trans_id, source) {
     process.stdout.write('null javascript\n');
     return null;
 };
